@@ -31,11 +31,11 @@ def act_install_as_ruby_extension(action, group, kz)
   dl = Kz::Downloader.new(uri, filename)
   dl.signal_connect("completed") do
     Kz::Actions.install_action(kz, filename)
-    kz.statusbar.set_text(_("installed Ruby extension successfully"), name)
+    kz.statusbar.set_text(_("Installed Ruby extension successfully"), name)
     Kz.pop_statusbar(kz, name, statusbar_timeout)
   end
   dl.signal_connect("error") do
-    kz.statusbar.set_text(_("failed Ruby extension install"), name)
+    kz.statusbar.set_text(_("Failed Ruby extension install"), name)
     Kz.pop_statusbar(kz, name, statusbar_timeout)
   end
   Kz::DownloaderGroup.add(dl)
@@ -43,5 +43,5 @@ def act_install_as_ruby_extension(action, group, kz)
 end
 def act_install_as_ruby_extension_config(config, kz)
   config[:label] = N_("Install as Ruby extension")
-  config[:tooltip] = N_("download and register as Ruby extension")
+  config[:tooltip] = N_("Download and register as Ruby extension")
 end
