@@ -11,7 +11,7 @@ def move_to_anchor(kz)
     end
   end
   target_anchor = yield(anchors, index)
-  kz.load_url(target_anchor.link) if target_anchor and target_anchor.link
+  kz.load_uri(target_anchor.link) if target_anchor and target_anchor.link
 end
 
 def act_move_to_previous_anchor(action, group, kz)
