@@ -23,7 +23,7 @@ def act_text_link(action, group, kz)
     if new_tab
       kz.open_new_tab(normalized_uri)
     else
-      kz.load_url(normalized_uri)
+      kz.load_uri(normalized_uri)
     end
   end
 end
