@@ -20,8 +20,10 @@
 
 #include "kz-rb-ext.h"
 #include "kz-bookmark.h"
+#include "kz-bookmark-separator.h"
 
 #define _SELF(obj) (KZ_BOOKMARK(RVAL2GOBJ(obj)))
+#define RVAL2KZBOOKMARKFOLDER(obj) (KZ_BOOKMARK_FOLDER(RVAL2GOBJ(obj)))
 
 static VALUE
 rb_kz_bookmark_is_editable(VALUE self)
@@ -32,66 +34,66 @@ rb_kz_bookmark_is_editable(VALUE self)
 static VALUE
 rb_kz_bookmark_is_separator(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_is_separator(_SELF(self)));
+    return CBOOL2RVAL(KZ_IS_BOOKMARK_SEPARATOR(_SELF(self)));
 }
 
 static VALUE
 rb_kz_bookmark_is_folder(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_is_folder(_SELF(self)));
+    return CBOOL2RVAL(KZ_IS_BOOKMARK_FOLDER(_SELF(self)));
 }
 
 static VALUE
 rb_kz_bookmark_is_pure_folder(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_is_pure_folder(_SELF(self)));
+    return CBOOL2RVAL(KZ_IS_BOOKMARK_FOLDER(_SELF(self)));
 }
 
 static VALUE
 rb_kz_bookmark_get_folded(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_get_folded(_SELF(self)));
+    return CBOOL2RVAL(kz_bookmark_folder_get_folded(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
 rb_kz_bookmark_set_folded(VALUE self, VALUE folded)
 {
-    kz_bookmark_set_folded(_SELF(self), RVAL2CBOOL(folded));
+    kz_bookmark_folder_set_folded(RVAL2KZBOOKMARKFOLDER(self), RVAL2CBOOL(folded));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_insert_before(VALUE self, VALUE child, VALUE sibling)
 {
-    kz_bookmark_insert_before(_SELF(self), _SELF(child), _SELF(sibling));
+    kz_bookmark_folder_insert_before(RVAL2KZBOOKMARKFOLDER(self), _SELF(child), _SELF(sibling));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_append(VALUE self, VALUE child)
 {
-    kz_bookmark_append(_SELF(self), _SELF(child));
+    kz_bookmark_folder_append(RVAL2KZBOOKMARKFOLDER(self), _SELF(child));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_prepend(VALUE self, VALUE child)
 {
-    kz_bookmark_prepend(_SELF(self), _SELF(child));
+    kz_bookmark_folder_prepend(RVAL2KZBOOKMARKFOLDER(self), _SELF(child));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_remove(VALUE self, VALUE child)
 {
-    kz_bookmark_remove(_SELF(self), _SELF(child));
+    kz_bookmark_folder_remove(RVAL2KZBOOKMARKFOLDER(self), _SELF(child));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_remove_all(VALUE self)
 {
-    kz_bookmark_remove_all(_SELF(self));
+    kz_bookmark_folder_remove_all(RVAL2KZBOOKMARKFOLDER(self));
     return Qnil;
 }
 
@@ -110,13 +112,13 @@ rb_kz_bookmark_get_parent_file(VALUE self)
 static VALUE
 rb_kz_bookmark_get_children(VALUE self)
 {
-    return GLIST2ARY(kz_bookmark_get_children(_SELF(self)));
+    return GLIST2ARY(kz_bookmark_folder_get_children(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
 rb_kz_bookmark_has_children(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_has_children(_SELF(self)));
+    return CBOOL2RVAL(kz_bookmark_folder_has_children(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
@@ -134,67 +136,67 @@ rb_kz_bookmark_prev(VALUE self)
 static VALUE
 rb_kz_bookmark_sort(VALUE self, VALUE type)
 {
-    kz_bookmark_sort(_SELF(self), RVAL2CSTR(type));
+    kz_bookmark_folder_sort(RVAL2KZBOOKMARKFOLDER(self), RVAL2CSTR(type));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_get_current(VALUE self)
 {
-    return UINT2NUM(kz_bookmark_get_current(_SELF(self)));
+    return UINT2NUM(kz_bookmark_folder_get_current_position(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
 rb_kz_bookmark_set_current(VALUE self, VALUE pos)
 {
-    kz_bookmark_set_current(_SELF(self), NUM2UINT(pos));
+    kz_bookmark_folder_set_current_position(RVAL2KZBOOKMARKFOLDER(self), NUM2UINT(pos));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_get_lock(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_get_lock(_SELF(self)));
+    return CBOOL2RVAL(kz_bookmark_folder_get_lock(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
 rb_kz_bookmark_set_lock(VALUE self, VALUE lock)
 {
-    kz_bookmark_set_lock(_SELF(self), RVAL2CBOOL(lock));
+    kz_bookmark_folder_set_lock(RVAL2KZBOOKMARKFOLDER(self), RVAL2CBOOL(lock));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_get_auto_refresh(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_get_auto_refresh(_SELF(self)));
+    return CBOOL2RVAL(kz_bookmark_folder_get_auto_refresh(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
 rb_kz_bookmark_set_auto_refresh(VALUE self, VALUE auto_refresh)
 {
-    kz_bookmark_set_auto_refresh(_SELF(self), RVAL2CBOOL(auto_refresh));
+    kz_bookmark_folder_set_auto_refresh(RVAL2KZBOOKMARKFOLDER(self), RVAL2CBOOL(auto_refresh));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_get_javascript(VALUE self)
 {
-    return CBOOL2RVAL(kz_bookmark_get_javascript(_SELF(self)));
+    return CBOOL2RVAL(kz_bookmark_folder_get_javascript(RVAL2KZBOOKMARKFOLDER(self)));
 }
 
 static VALUE
 rb_kz_bookmark_set_javascript(VALUE self, VALUE javascript)
 {
-    kz_bookmark_set_javascript(_SELF(self), RVAL2CBOOL(javascript));
+    kz_bookmark_folder_set_javascript(RVAL2KZBOOKMARKFOLDER(self), RVAL2CBOOL(javascript));
     return Qnil;
 }
 
 static VALUE
 rb_kz_bookmark_find_bookmark_from_uri(VALUE self, VALUE key_uri)
 {
-    return GOBJ2RVAL(kz_bookmark_find_bookmark_from_uri(_SELF(self),
-                                                        RVAL2CSTR(key_uri)));
+    return GOBJ2RVAL(kz_bookmark_folder_find_bookmark_from_uri(RVAL2KZBOOKMARKFOLDER(self),
+                                                               RVAL2CSTR(key_uri)));
 }
 
 void
@@ -205,7 +207,7 @@ _kz_rb_bookmark_init(VALUE mKz)
     cKzBookmark = G_DEF_CLASS(KZ_TYPE_BOOKMARK, "Bookmark", mKz);
 
     rb_define_const(cKzBookmark, "SORT_LAST_MODIFIED",
-                    CSTR2RVAL(KZ_BOOKMARK_SORT_LAST_MODIFIED));
+                    CSTR2RVAL(KZ_BOOKMARK_FOLDER_SORT_LAST_MODIFIED));
 
     rb_define_method(cKzBookmark, "editable?", rb_kz_bookmark_is_editable, 0);
     rb_define_method(cKzBookmark, "separator?", rb_kz_bookmark_is_separator, 0);
