@@ -1,7 +1,7 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 
 /*
- *  Copyright (C) 2007 Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009 Kouhei Sutou <kou@cozmixng.org>
  *  Copyright (C) 2006 Eriko Sato
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,27 +21,20 @@
 
 #include "kz-rb-ext.h"
 
-#include "kz-embed.h"
-#include "kz-embed.h"
+#include <kz-embed.h>
 
 #define _SELF(obj) (KZ_EMBED(RVAL2GOBJ(obj)))
 #define RVAL2KZBOOKMARK(obj) (KZ_BOOKMARK(RVAL2GOBJ(obj)))
+#define RVAL2KZBOOKMARKFOLDER(obj) (KZ_BOOKMARK_FOLDER(RVAL2GOBJ(obj)))
 #define RVAL2KZNAVI(obj) (KZ_NAVI(RVAL2GOBJ(obj)))
 #define RVAL2KZNAVLINK(obj) (RVAL2GENUM(obj, KZ_TYPE_EMBED_NAV_LINK))
 #define RVAL2KZRF(obj) (RVAL2GENUM(obj, KZ_TYPE_EMBED_RELOAD_FLAG))
 #define RVAL2KZCT(obj) (RVAL2GENUM(obj, KZ_TYPE_EMBED_COPY_TYPE))
 
 static VALUE
-rb_kz_embed_get_window(VALUE self)
+rb_kz_embed_load_uri(VALUE self, VALUE url)
 {
-    return GOBJ2RVAL(kz_app_get_window_from_tab(KZ_RB_GET_APP,
-                                                RVAL2GOBJ(self)));
-}
-
-static VALUE
-rb_kz_embed_load_url(VALUE self, VALUE url)
-{
-    kz_embed_load_url(_SELF(self), RVAL2CSTR(url));
+    kz_embed_load_uri(_SELF(self), RVAL2CSTR(url));
     return self;
 }
 
@@ -52,60 +45,6 @@ rb_kz_embed_view_source(VALUE self, VALUE url)
     return self;
 }
 
-static VALUE
-rb_kz_embed_is_loading(VALUE self)
-{
-    gboolean loading;
-    loading = kz_embed_is_loading(_SELF(self));
-    return CBOOL2RVAL(loading);
-}
-
-static VALUE
-rb_kz_embed_get_title(VALUE self)
-{
-    const gchar *title;
-    title = kz_embed_get_title(_SELF(self));
-    return CSTR2RVAL(title);
-}
-
-static VALUE
-rb_kz_embed_get_location(VALUE self)
-{
-    const gchar *location;
-    location = kz_embed_get_location(_SELF(self));
-    return CSTR2RVAL(location);
-}
-
-static VALUE
-rb_kz_embed_ensure_title(VALUE self)
-{
-    gchar *title;
-    VALUE rb_title;
-    title = kz_embed_ensure_title(_SELF(self));
-    rb_title = CSTR2RVAL(title);
-    g_free(title);
-    return rb_title;
-}
-
-static VALUE
-rb_kz_embed_get_link_message(VALUE self)
-{
-    gchar *message;
-    VALUE rb_message;
-    message = kz_embed_get_link_message(_SELF(self));
-    rb_message = CSTR2RVAL(message);
-    g_free(message);
-    return rb_message;
-}
-
-static VALUE
-rb_kz_embed_get_progress(VALUE self)
-{
-    gdouble progress;
-    progress = kz_embed_get_progress(_SELF(self));
-    return rb_float_new(progress);
-}
-
 static VALUE
 rb_kz_embed_can_cut_selection(VALUE self)
 {
@@ -233,24 +172,12 @@ rb_kz_embed_get_dest_anchors(VALUE self)
 }
 
 static VALUE
-rb_kz_embed_copy_page(VALUE self, VALUE dkzembed, VALUE type)
+rb_kz_embed_copy_page(VALUE self, VALUE dkzembed)
 {
-    kz_embed_copy_page(_SELF(self), _SELF(dkzembed), RVAL2KZCT(type));
+    kz_embed_copy_page(_SELF(self), _SELF(dkzembed));
     return self;
 }
 
-static VALUE
-rb_kz_embed_shistory_copy(VALUE source, VALUE dest, VALUE back_history,
-			  VALUE forward_history, VALUE set_current)
-{
-    gboolean shistory_copy;
-    shistory_copy = kz_embed_shistory_copy(_SELF(source), _SELF(dest),
-                                           RVAL2CBOOL(back_history),
-                                           RVAL2CBOOL(forward_history),
-                                           RVAL2CBOOL(set_current));
-    return CBOOL2RVAL(shistory_copy);
-}
-
 static VALUE
 rb_kz_embed_shistory_get_pos(VALUE self)
 {
@@ -301,13 +228,6 @@ rb_kz_embed_go_forward(VALUE self)
     return self;
 }
 
-static VALUE
-rb_kz_embed_go_up(VALUE self)
-{
-    kz_embed_go_up(_SELF(self));
-    return self;
-}
-
 static VALUE
 rb_kz_embed_can_go_back(VALUE self)
 {
@@ -324,14 +244,6 @@ rb_kz_embed_can_go_forward(VALUE self)
     return CBOOL2RVAL(can_go_forward);
 }
 
-static VALUE
-rb_kz_embed_can_go_up(VALUE self)
-{
-    gboolean can_go_up;
-    can_go_up = kz_embed_can_go_up(_SELF(self));
-    return CBOOL2RVAL(can_go_up);
-}
-
 static VALUE
 rb_kz_embed_can_go_nav_link(VALUE self, VALUE link)
 {
@@ -404,37 +316,6 @@ rb_kz_embed_go_history_index(VALUE self, VALUE index)
     return self;
 }
 
-static VALUE
-rb_kz_embed_do_command(VALUE self, VALUE command)
-{
-    kz_embed_do_command(_SELF(self), STR2CSTR(command));
-    return self;
-}
-
-static VALUE
-rb_kz_embed_can_do_command(VALUE self, VALUE command)
-{
-    gboolean can_do_command;
-    can_do_command = kz_embed_can_do_command(_SELF(self),
-                                             STR2CSTR(command));
-    return RVAL2CBOOL(can_do_command);
-}
-
-static VALUE
-rb_kz_embed_get_lock(VALUE self)
-{
-    gboolean get_lock;
-    get_lock = kz_embed_get_lock(_SELF(self));
-    return RVAL2CBOOL(get_lock);
-}
-
-static VALUE
-rb_kz_embed_set_lock(VALUE self, VALUE lock)
-{
-    kz_embed_set_lock(_SELF(self), CBOOL2RVAL(lock));
-    return self;
-}
-
 static VALUE
 rb_kz_embed_get_body_text(VALUE self)
 {
@@ -487,12 +368,12 @@ rb_kz_embed_get_printer_list(VALUE self)
 static VALUE
 rb_kz_embed_create_thumbnail(int argc, VALUE *argv, VALUE self)
 {
-    EggPixbufThumbSize size;
+    EggPixbufThumbnailSize size;
     VALUE rb_size;
 
     rb_scan_args(argc, argv, "01", &rb_size);
     if (NIL_P(rb_size))
-        size = EGG_PIXBUF_THUMB_LARGE;
+        size = EGG_PIXBUF_THUMBNAIL_LARGE;
     else
         size = NUM2INT(rb_size); /* FIXME */
     kz_embed_create_thumbnail(_SELF(self), size);
@@ -507,36 +388,6 @@ rb_kz_embed_save_with_content(VALUE self, VALUE rawfilename)
     return RVAL2CBOOL(save);
 }
 
-static VALUE
-rb_kz_embed_zoom_set(VALUE self, VALUE zoom, VALUE reflow)
-{
-    kz_embed_zoom_set(_SELF(self), NUM2INT(zoom), RVAL2CBOOL(reflow));
-    return self;
-}
-
-static VALUE
-rb_kz_embed_zoom_get(VALUE self)
-{
-    gint zoom_get;
-    zoom_get = kz_embed_zoom_get(_SELF(self));
-    return INT2NUM(zoom_get);
-}
-
-static VALUE
-rb_kz_embed_set_text_size(VALUE self, VALUE zoom, VALUE reflow)
-{
-    kz_embed_set_text_size(_SELF(self), NUM2INT(zoom), RVAL2CBOOL(reflow));
-    return self;
-}
-
-static VALUE
-rb_kz_embed_get_text_size(VALUE self)
-{
-    gint size;
-    size = kz_embed_get_text_size(_SELF(self));
-    return INT2NUM(size);
-}
-
 static VALUE
 rb_kz_embed_get_html_with_contents(VALUE self, VALUE storedir)
 {
@@ -548,21 +399,6 @@ rb_kz_embed_get_html_with_contents(VALUE self, VALUE storedir)
     return rb_html;
 }
 
-
-static VALUE
-rb_kz_embed_set_history(VALUE self, VALUE history)
-{
-    kz_embed_set_history(_SELF(self), RVAL2KZBOOKMARK(history));
-    return self;
-}
-
-static VALUE
-rb_kz_embed_get_history(VALUE self, VALUE history)
-{
-    kz_embed_get_history(_SELF(self), RVAL2KZBOOKMARK(history));
-    return self;
-}
-
 static VALUE
 rb_kz_embed_get_last_modified(VALUE self)
 {
@@ -642,23 +478,10 @@ _kz_rb_embed_init(VALUE mKz)
     G_DEF_CLASS(KZ_TYPE_EMBED_RELOAD_FLAG, "EmbedReloadFlag", mKz);
     G_DEF_CLASS(KZ_TYPE_EMBED_GO_ACTION, "EmbedGoAction", mKz);
     G_DEF_CLASS(KZ_TYPE_EMBED_NAV_LINK, "EmbedNavLink", mKz);
-    G_DEF_CLASS(KZ_TYPE_EMBED_COPY_TYPE, "EmbedCopyType", mKz);
 
-    rb_define_method(cKzEmbed, "window", rb_kz_embed_get_window, 0);
-
-    rb_define_method(cKzEmbed, "load_url", rb_kz_embed_load_url, 1);
+    rb_define_method(cKzEmbed, "load_uri", rb_kz_embed_load_uri, 1);
     rb_define_method(cKzEmbed, "view_source",
                      rb_kz_embed_view_source, 1);
-    rb_define_method(cKzEmbed, "loading?", rb_kz_embed_is_loading, 0);
-    rb_define_method(cKzEmbed, "title", rb_kz_embed_get_title, 0);
-    rb_define_method(cKzEmbed, "location",
-                     rb_kz_embed_get_location, 0);
-    rb_define_method(cKzEmbed, "ensure_title",
-                     rb_kz_embed_ensure_title, 0);
-    rb_define_method(cKzEmbed, "link_message",
-                     rb_kz_embed_get_link_message, 0);
-    rb_define_method(cKzEmbed, "progress",
-                     rb_kz_embed_get_progress, 0);
     rb_define_method(cKzEmbed, "can_cut_selection?",
                      rb_kz_embed_can_cut_selection, 0);
     rb_define_method(cKzEmbed, "can_copy_selection?",
@@ -680,8 +503,6 @@ _kz_rb_embed_init(VALUE mKz)
     rb_define_method(cKzEmbed, "links", rb_kz_embed_get_links, -1);
     rb_define_method(cKzEmbed, "dest_anchors", rb_kz_embed_get_dest_anchors, 0);
     rb_define_method(cKzEmbed, "copy_page", rb_kz_embed_copy_page, 1);
-    rb_define_method(cKzEmbed, "shistory_copy",
-                     rb_kz_embed_shistory_copy, 5);
     rb_define_method(cKzEmbed, "shistory_get_pos",
                      rb_kz_embed_shistory_get_pos, 0);
     rb_define_method(cKzEmbed, "shistory_get_nth",
@@ -690,12 +511,10 @@ _kz_rb_embed_init(VALUE mKz)
     rb_define_method(cKzEmbed, "stop_load", rb_kz_embed_stop_load, 0);
     rb_define_method(cKzEmbed, "go_back", rb_kz_embed_go_back, 0);
     rb_define_method(cKzEmbed, "go_forward", rb_kz_embed_go_forward, 0);
-    rb_define_method(cKzEmbed, "go_up", rb_kz_embed_go_up, 0);
     rb_define_method(cKzEmbed, "can_go_back?",
                      rb_kz_embed_can_go_back, 0);
     rb_define_method(cKzEmbed, "can_go_forward?",
                      rb_kz_embed_can_go_forward, 0);
-    rb_define_method(cKzEmbed, "can_go_up?", rb_kz_embed_can_go_up, 0);
     rb_define_method(cKzEmbed, "can_go_nav_link?",
                      rb_kz_embed_can_go_nav_link, 1);
     rb_define_method(cKzEmbed, "go_nav_link",
@@ -714,11 +533,6 @@ _kz_rb_embed_init(VALUE mKz)
                      rb_kz_embed_get_nav_links, 1);
     rb_define_method(cKzEmbed, "go_history_index",
                      rb_kz_embed_go_history_index, 1);
-    rb_define_method(cKzEmbed, "do_command", rb_kz_embed_do_command, 1);
-    rb_define_method(cKzEmbed, "can_do_command?",
-                     rb_kz_embed_can_do_command, 1);
-    rb_define_method(cKzEmbed, "lock?", rb_kz_embed_get_lock, 0);
-    rb_define_method(cKzEmbed, "set_lock", rb_kz_embed_set_lock, 0);
     rb_define_method(cKzEmbed, "get_body_text",
                      rb_kz_embed_get_body_text, 0);
     rb_define_method(cKzEmbed, "set_encoding",
@@ -734,19 +548,10 @@ _kz_rb_embed_init(VALUE mKz)
                      rb_kz_embed_create_thumbnail, -1);
     rb_define_method(cKzEmbed, "save_with_content",
                      rb_kz_embed_save_with_content, 1);
-    rb_define_method(cKzEmbed, "zoom_set", rb_kz_embed_zoom_set, 2);
-    rb_define_method(cKzEmbed, "zoom_get", rb_kz_embed_zoom_get, 0);
-    rb_define_method(cKzEmbed, "set_text_size",
-                     rb_kz_embed_set_text_size, 2);
-    rb_define_method(cKzEmbed, "get_text_size",
-                     rb_kz_embed_get_text_size, 0);
     rb_define_method(cKzEmbed, "get_html_with_contents",
                      rb_kz_embed_get_html_with_contents, 1);
-    rb_define_method(cKzEmbed, "set_history", rb_kz_embed_set_history, 1);
-    rb_define_method(cKzEmbed, "get_history", rb_kz_embed_get_history, 1);
     rb_define_method(cKzEmbed, "get_last_modified",
                      rb_kz_embed_get_last_modified, 0);
-    rb_define_method(cKzEmbed, "zoom_get", rb_kz_embed_zoom_get, 0);
     rb_define_method(cKzEmbed, "fine_scroll", rb_kz_embed_fine_scroll, 2);
     rb_define_method(cKzEmbed, "page_up", rb_kz_embed_page_up, 0);
     rb_define_method(cKzEmbed, "page_down", rb_kz_embed_page_down, 0);
