@@ -22,6 +22,7 @@
 
 #define _SELF(obj) RVAL2KZNB(obj)
 #define RVAL2GDKSD(obj) RVAL2GENUM(obj, GDK_TYPE_SCROLL_DIRECTION)
+#define RVAL2KZWEB(obj) (KZ_WEB(RVAL2GOBJ(obj)))
 
 static VALUE
 rb_kz_notebook_new(VALUE self, VALUE kz)
@@ -36,21 +37,21 @@ rb_kz_notebook_new(VALUE self, VALUE kz)
 static VALUE
 rb_kz_notebook_open_new_tab(int argc, VALUE *argv, VALUE self)
 {
-    VALUE rb_embed, rb_label, pos;
+    VALUE rb_web, rb_label, pos;
     KzNotebook *notebook;
-    KzEmbed *embed;
+    KzWeb *web;
     KzTabLabel *label;
     gint inserted_pos;
 
-    rb_scan_args(argc, argv, "21", &rb_embed, &rb_label, &pos);
+    rb_scan_args(argc, argv, "21", &rb_web, &rb_label, &pos);
 
     notebook = _SELF(self);
-    embed = RVAL2KZEMB(rb_embed);
+    web = RVAL2KZWEB(rb_web);
     label = RVAL2KZTL(rb_label);
     if (NIL_P(pos)) {
-        inserted_pos = kz_notebook_open_new_tab(notebook, embed, label);
+        inserted_pos = kz_notebook_open_new_tab(notebook, web, label);
     } else {
-        inserted_pos = kz_notebook_open_new_tab_at_pos(notebook, embed, label,
+        inserted_pos = kz_notebook_open_new_tab_at_pos(notebook, web, label,
                                                        NUM2INT(pos));
     }
 
@@ -61,7 +62,7 @@ static VALUE
 rb_kz_notebook_close_tab(VALUE self, VALUE widget)
 {
     gboolean result;
-    result = kz_notebook_close_tab(_SELF(self), RVAL2GTKWIDGET(widget));
+    result = kz_notebook_close_tab(_SELF(self), RVAL2KZWEB(widget));
     return CBOOL2RVAL(result);
 }
 
