@@ -24,6 +24,8 @@
 #define RVAL2GTKACTION(obj) (GTK_ACTION(RVAL2GOBJ(obj)))
 #define RVAL2GNODE(obj) (GTK_GNODE(RVAL2GOBJ(obj)))
 #define RVAL2KZBOOKMARK(obj) (KZ_BOOKMARK(RVAL2GOBJ(obj)))
+#define RVAL2KZBOOKMARKFOLDER(obj) (KZ_BOOKMARK_FOLDER(RVAL2GOBJ(obj)))
+#define RVAL2KZWEB(obj) (KZ_WEB(RVAL2GOBJ(obj)))
 
 static ID id_default;
 
@@ -73,7 +75,7 @@ rb_kz_window_open_new_tab_with_parent(VALUE self, VALUE url, VALUE parent)
 static VALUE
 rb_kz_window_close_tab(VALUE self, VALUE widget)
 {
-    kz_window_close_tab(_SELF(self), RVAL2GTKWIDGET(widget));
+    kz_window_close_tab(_SELF(self), RVAL2KZWEB(widget));
     return self;
 }
 
@@ -92,8 +94,9 @@ rb_kz_window_move_tab(VALUE self, VALUE dest, VALUE widget)
 }
 
 static VALUE
-rb_kz_window_load_url(VALUE self, VALUE url)
+rb_kz_window_load_uri(VALUE self, VALUE url)
 {
+    /* kz_window_load_url() should be kz_window_load_uri() */
     kz_window_load_url(_SELF(self), RVAL2CSTR(url));
     return self;
 }
@@ -152,9 +155,9 @@ rb_kz_window_get_uri(VALUE self)
 static VALUE
 rb_kz_window_get_tab_label(VALUE self, VALUE widget)
 {
-    GtkWidget *Widget;
+    KzTabLabel *Widget;
     Widget = kz_window_get_tab_label(_SELF(self),
-                                     RVAL2GTKWIDGET(widget));
+                                     RVAL2KZWEB(widget));
     return GOBJ2RVAL(Widget);
 }
 
@@ -197,7 +200,7 @@ rb_kz_window_get_mouse_event_info(VALUE self)
 static VALUE
 rb_kz_window_append_closed_tab(VALUE self, VALUE bookmark)
 {
-    kz_window_append_closed_tab(_SELF(self), RVAL2KZBOOKMARK(bookmark));
+    kz_window_append_closed_tab(_SELF(self), RVAL2KZBOOKMARKFOLDER(bookmark));
     return self;
 }
 
@@ -263,13 +266,13 @@ rb_kz_window_notebook(VALUE self)
 static VALUE
 rb_kz_window_nth_page(VALUE self, VALUE n)
 {
-  return GOBJ2RVAL(KZ_WINDOW_NTH_PAGE(_SELF(self), NUM2INT(n)));
+  return GOBJ2RVAL(KZ_WINDOW_NTH_WEB(_SELF(self), NUM2INT(n)));
 }
 
 static VALUE
 rb_kz_window_current_page(VALUE self)
 {
-  return GOBJ2RVAL(KZ_WINDOW_CURRENT_PAGE(_SELF(self)));
+  return GOBJ2RVAL(KZ_WINDOW_CURRENT_WEB(_SELF(self)));
 }
 
 void
@@ -298,7 +301,7 @@ _kz_rb_window_init(VALUE mKz)
     rb_define_method(cKzWindow, "close_all_tab",
                      rb_kz_window_close_all_tab, 0);
     rb_define_method(cKzWindow, "move_tab", rb_kz_window_move_tab, 2);
-    rb_define_method(cKzWindow, "load_url", rb_kz_window_load_url, 1);
+    rb_define_method(cKzWindow, "load_uri", rb_kz_window_load_uri, 1);
     rb_define_method(cKzWindow, "store_state",
                      rb_kz_window_store_state, 0);
     rb_define_method(cKzWindow, "restore_state",
