@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSDialogs.cpp 3494 2008-03-24 13:01:09Z pal_gene $
+ *  $Id: GtkNSSDialogs.cpp 3505 2008-06-27 10:43:46Z kous $
  */
 
 /*
@@ -53,6 +53,9 @@
 #include <nsEmbedString.h>
 #undef MOZILLA_STRICT_API
 #include <nsIDOMWindow.h>
+#ifdef HAVE_GECKO_1_9
+#  include <nsIMutableArray.h>
+#endif
 
 #include <gtk/gtk.h>
 #include <time.h>
