@@ -64,7 +64,7 @@ EmbedWindow::Init(EmbedPrivate *aOwner)
 }
 
 nsresult
-EmbedWindow::CreateWindow(void)
+EmbedWindow::CreateEmbedWindow(void)
 {
   nsresult rv;
   GtkWidget *ownerAsWidget = GTK_WIDGET(mOwner->mOwningWidget);
