@@ -90,8 +90,7 @@ kz_gecko_single_class_init (KzGeckoSingleClass *klass)
 {
 	GObjectClass *object_class;
 
-	kz_gecko_single_parent_class = g_type_class_peek_parent (klass);
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->constructor  = constructor;
 	object_class->dispose	   = dispose;
@@ -115,21 +114,25 @@ constructor (GType                  type,
              GObjectConstructParam *props)
 {
 	GObject *object;
+	GObjectClass *klass;
+	KzGeckoSinglePrivate *priv;
+	KzProfile *profile;
 
-	if (!the_kz_gecko_single) 
+	if (the_kz_gecko_single) 
+		return G_OBJECT(g_object_ref(the_kz_gecko_single));
+
+	klass = G_OBJECT_CLASS(kz_gecko_single_parent_class);
+	object = klass->constructor(type, n_props, props);
+
+	the_kz_gecko_single = KZ_GECKO_SINGLE(object);
+
+	priv = KZ_GECKO_SINGLE_GET_PRIVATE(object);
+
+	mozilla_register_components();
+
+	if (priv->kz_app)
 	{
-		KzGeckoSinglePrivate *priv;
-		KzProfile *profile;
-		GObjectClass *klass = G_OBJECT_CLASS(kz_gecko_single_parent_class);
-		object = klass->constructor(type, n_props, props);
-
-		the_kz_gecko_single = KZ_GECKO_SINGLE(object);
-
-		priv = KZ_GECKO_SINGLE_GET_PRIVATE(object);
-		
 		profile = kz_app_get_profile(priv->kz_app);
-
-		mozilla_register_components();
 		set_user_agent(profile);
 		g_signal_connect(profile, "changed::Global",
 				G_CALLBACK(cb_global_profile_changed), NULL);
@@ -140,10 +143,6 @@ constructor (GType                  type,
 		g_signal_connect(profile, "changed::Privacy",
 				G_CALLBACK(cb_privacy_profile_changed), NULL);
 	}
-	else
-	{
-		object = G_OBJECT(g_object_ref(G_OBJECT(the_kz_gecko_single)));
-	}
 	return object;
 }
 
@@ -158,7 +157,8 @@ set_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_KZ_APP:
-		priv->kz_app = KZ_APP(g_object_ref(g_value_get_object(value)));
+		if (g_value_get_object(value))
+			priv->kz_app = KZ_APP(g_object_ref(g_value_get_object(value)));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -218,8 +218,8 @@ KzGeckoSingle *
 kz_gecko_single_new (KzApp *kz_app)
 {
 	return KZ_GECKO_SINGLE(g_object_new(KZ_TYPE_GECKO_SINGLE,
-				    "kz-app", kz_app,
-				    NULL));
+			     		    "kz-app", kz_app,
+					    NULL));
 }
 
 
@@ -284,8 +284,7 @@ cb_font_profile_changed (KzProfile *profile,
 	case 's': // size
 		if (g_str_has_prefix(key, "size_variable_") ||
 		    g_str_has_prefix(key, "size_fixed_") ||
-		    g_str_has_prefix(key, "min-size_variable_") || 
-		    g_str_has_prefix(key, "min-size_fixed_"))
+		    g_str_has_prefix(key, "minimum-size_"))
 		{
 			gint value;
 			kz_profile_get_value(profile, "Font", key, 
@@ -354,7 +353,7 @@ cb_global_profile_changed (KzProfile *profile,
 			if (item)
 			{
 				mozilla_prefs_set_proxy(item);
-				g_object_unref(G_OBJECT(item));
+				g_object_unref(item);
 			}
 		}
 		break;
