@@ -395,7 +395,7 @@ NS_METHOD KzFilePicker::SanityCheck (PRBool *retIsSane)
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				errorText);
+				"%s", errorText);
 
 		if (mParentWidget)
 			gtk_window_set_transient_for(GTK_WINDOW(errorDialog),
