@@ -41,6 +41,12 @@
 #	include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+/* GTK_CHECK_CAST is masked by GTK_DISABLE_DEPRECATED in gtk+-2.14 */
+#ifndef GTK_CHECK_CAST
+#undef GTK_MOZ_EMBED
+#define GTK_MOZ_EMBED(obj)	(G_TYPE_CHECK_INSTANCE_CAST((obj), GTK_TYPE_MOZ_EMBED, GtkMozEmbed))
+#endif
+
 KzMozSelectionListener::KzMozSelectionListener () :
 	mKzGeckoEmbed (nsnull),
 	mWebBrowser (nsnull),
