@@ -33,6 +33,9 @@
 
 #define MOZILLA_CLIENT
 
+#ifdef G_OS_WIN32
+#include <mozilla-config.h>
+#endif
 #include "kz-mozwrapper.h"
 #include <gtkmozembed_internal.h>
 
@@ -43,9 +46,11 @@
 #include <nsIDocShell.h>
 #include <nsIDocShellTreeItem.h>
 #include <nsIDocShellTreeOwner.h>
+#ifndef G_OS_WIN32
 #define nsAString_h___
 #include <nsIContent.h>
 #undef nsAString_h___
+#endif
 #define nsString_h___
 #include <nsIPrintOptions.h>
 #undef nsString_h___
@@ -94,17 +99,18 @@
 
 typedef nsString              nsAFlatString;
 typedef nsCString             nsAFlatCString;
+#ifndef G_OS_WIN32
 #  define nsString_h___
 #  define nsAString_h___
 #  include <nsPresContext.h>
 #  undef nsAString_h___
 #  undef nsString_h___
+#endif
 
 #include "kz-mozprogresslistener.h"
 #include "kz-mozutils.h"
 #include "utils.h"
 #include "kz-bookmark.h"
-#include "kz-mozthumbnailer.h"
 
 # include <nsISSLStatus.h>
 # include <nsISSLStatusProvider.h>
@@ -144,7 +150,7 @@ KzMozWrapper::Init (KzGeckoEmbed *kzembed)
 					getter_AddRefs(mWebBrowser));
 	if (!mWebBrowser) return NS_ERROR_FAILURE;
 
-	mWebBrowser->GetContentDOMWindow (getter_AddRefs (mDOMWindow));
+	mWebBrowser->GetContentDOMWindow(getter_AddRefs(mDOMWindow));
 
 	/* this codes was taken from Galeon-1.3.17 */
 	/* This will instantiate an about:blank doc if necessary, but don't
@@ -168,6 +174,7 @@ KzMozWrapper::Init (KzGeckoEmbed *kzembed)
 	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 	GetListener();
 	AttachListeners();
+
 #if 0
 	mKzMozSelectionListener = new KzMozSelectionListener;
 	mKzMozSelectionListener->Init(kzembed);
@@ -193,16 +200,11 @@ KzMozWrapper::Init (KzGeckoEmbed *kzembed)
 	if (NS_FAILED (rv)) g_warning("failed to add a history listener");
 #endif
 
-// SSL related code
-// These codes are picked from mozilla/GaleonWrapper in Galeon 1.3.21.
-// Original copyright notice.
-// Copyright (C) 2000 Marco Pesenti Gritti
+	nsCOMPtr<nsIDocShell> docShell;
+	rv = GetDocShell(getter_AddRefs(docShell));
 
-	nsCOMPtr<nsIDocShell> docShell (do_GetInterface (mWebBrowser, &rv));
-	NS_ENSURE_SUCCESS (rv, rv);
-
-	rv = docShell->GetSecurityUI (getter_AddRefs (mSecurityInfo));
-	NS_ENSURE_SUCCESS (rv, rv);
+	rv = docShell->GetSecurityUI(getter_AddRefs(mSecurityInfo));
+	NS_ENSURE_SUCCESS(rv, rv);
 
 	return NS_OK;
 }
@@ -212,10 +214,10 @@ nsresult
 KzMozWrapper::GetListener (void)
 {
   	if (mEventTarget) return NS_ERROR_FAILURE;
-	
+
   	nsCOMPtr<nsIDOMWindow> domWindowExternal;
   	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindowExternal));
-  
+
   	nsCOMPtr<nsIDOMWindow2> domWindow;
         domWindow = do_QueryInterface(domWindowExternal);
 	if (!domWindow) return NS_ERROR_FAILURE;
@@ -244,7 +246,7 @@ nsresult
 KzMozWrapper::DetachListeners(void)
 {
 	if (!mEventTarget) return NS_ERROR_FAILURE;
-	
+
 	const PRUnichar domLiteral[] = { 'D', 'O', 'M', 'L', 'i', 'n', 'k',
 	  				 'A', 'd', 'd', 'e', 'd', '\0' };
 	return mEventTarget->RemoveEventListener(nsEmbedString(domLiteral),
@@ -266,31 +268,21 @@ KzMozWrapper::Destroy (void)
 // Our own methods
 //
 
+#include <nsPIDOMWindow.h>
 nsresult
 KzMozWrapper::GetDocShell (nsIDocShell **aDocShell)
 {
         if (!mWebBrowser) return NS_ERROR_FAILURE;
 
-        nsCOMPtr<nsIDocShellTreeItem> browserAsItem;
-        browserAsItem = do_QueryInterface(mWebBrowser);
-        if (!browserAsItem) return NS_ERROR_FAILURE;
+        nsCOMPtr<nsIDOMWindow> domWindow;
+        mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
 
-        // get the owner for that item
-        nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-        browserAsItem->GetTreeOwner(getter_AddRefs(treeOwner));
-        if (!treeOwner) return NS_ERROR_FAILURE;
+        if (!domWindow)
+                return NS_ERROR_FAILURE;
 
-        // get the primary content shell as an item
-        nsCOMPtr<nsIDocShellTreeItem> contentItem;
-        treeOwner->GetPrimaryContentShell(getter_AddRefs(contentItem));
-        if (!contentItem) return NS_ERROR_FAILURE;
+        nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
 
-        // QI that back to a docshell
-        nsCOMPtr<nsIDocShell> DocShell;
-        DocShell = do_QueryInterface(contentItem);
-        if (!DocShell) return NS_ERROR_FAILURE;
-
-        *aDocShell = DocShell.get();
+        *aDocShell = domWindowPrivate->GetDocShell();
         NS_IF_ADDREF(*aDocShell);
 
         return NS_OK;
@@ -2056,26 +2048,10 @@ KzMozWrapper::ShowPageCertificate (void)
 	return certDialogs->ViewCert (NULL, serverCert);
 }
 
-#ifdef MOZ_NSICANVASRENDERINGCONTEXTINTERNAL_HAVE_GETINPUTSTREAM_
-nsresult
-KzMozWrapper::CreateThumbnail (const gchar *uri,
-			       guint last_modified,
-			       EggPixbufThumbSize size)
-{
-	nsCOMPtr<nsIDOMWindow> domWindow;
-	GetDOMWindow(getter_AddRefs(domWindow));
-
- 	if (!KzMozThumbnailer::CreateThumbnail(domWindow, uri,
-					       last_modified, size))
-		return NS_ERROR_FAILURE;
-
-	return NS_OK;
-}
-#endif
-
 gboolean
 KzMozWrapper::GetRootRange (nsIDOMDocument *domDoc, nsIDOMRange *range)
 {
+#ifndef G_OS_WIN32
 	nsCOMPtr<nsIContent> rootContent;
 	nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(domDoc);
 	if (htmlDoc)
@@ -2099,6 +2075,9 @@ KzMozWrapper::GetRootRange (nsIDOMDocument *domDoc, nsIDOMRange *range)
 	range->SetEnd(rootNode,childcount);
 
 	return TRUE;
+#else
+	return FALSE;
+#endif
 }
 
 gboolean
@@ -2107,6 +2086,7 @@ KzMozWrapper::GetSelectedRange (nsIDOMDocument *domDoc,
 				nsISelection *selection,
 				gboolean backward)
 {
+#ifndef G_OS_WIN32
 	nsCOMPtr<nsIContent> rootContent;
 	nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(domDoc);
 	if (htmlDoc)
@@ -2164,11 +2144,15 @@ KzMozWrapper::GetSelectedRange (nsIDOMDocument *domDoc,
 	}
 
 	return TRUE;
+#else
+	return FALSE;
+#endif
 }
 
 gboolean
 KzMozWrapper::SetZoomOnDocShell (float aZoom, nsIDocShell *DocShell)
 {
+#ifndef G_OS_WIN32
 	nsCOMPtr<nsPresContext> PresContext;
 	nsresult rv = DocShell->GetPresContext (getter_AddRefs(PresContext));
 	if (NS_FAILED(rv) || !PresContext) return FALSE;
@@ -2190,4 +2174,7 @@ KzMozWrapper::SetZoomOnDocShell (float aZoom, nsIDocShell *DocShell)
 	if NS_FAILED(rv) return FALSE;
 	
 	return TRUE;
+#else
+	return FALSE;
+#endif
 }
