@@ -34,9 +34,6 @@
 #include "kz-gecko-embed.h"
 #include "kz-mozselectionlistener.h"
 #include "kz-mozeventlistener.h"
-#ifdef MOZ_NSICANVASRENDERINGCONTEXTINTERNAL_HAVE_GETINPUTSTREAM_
-#include "egg-pixbuf-thumbnail.h"
-#endif
 
 class nsISecureBrowserUI;
 class nsISSLStatus;
@@ -149,12 +146,6 @@ public:
 
 	nsresult ShowPageCertificate     (void);
 
-#ifdef MOZ_NSICANVASRENDERINGCONTEXTINTERNAL_HAVE_GETINPUTSTREAM_
-	nsresult CreateThumbnail	 (const gchar *uri,
-					  guint last_modified,
-					  EggPixbufThumbSize size);
-#endif
-
 	nsCOMPtr<nsIWebBrowser>    mWebBrowser;
 protected:
 	nsCOMPtr<nsIDOMEventTarget>   mEventTarget;
