@@ -159,7 +159,7 @@ kz_hyper_estraier_search_class_init (KzHyperEstraierSearchClass *klass)
 	GObjectClass *object_class;
 
 	parent_class = g_type_class_peek_parent (klass);
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->dispose     = dispose;
 }
@@ -402,7 +402,7 @@ create_search_result_html (KzSearch *search, const gchar *text)
 
 		thumb_filename =
 			egg_pixbuf_get_thumb_filename(uri,
-						      EGG_PIXBUF_THUMB_LARGE);
+						      EGG_PIXBUF_THUMBNAIL_LARGE);
 		thumb_uri = g_strdup_printf("history-search:?image=%s",
 					    thumb_filename);
 		g_free(thumb_filename);
