@@ -36,7 +36,7 @@
 #include "kz-window.h"
 #include "kz-tab-label.h"
 #include "kz-actions-download.h"
-#include "kz-embed.h"
+#include "kz-web.h"
 #include "kz-favicon.h"
 #include "kz-xml.h"
 #include "kz-actions-dynamic.h"
@@ -61,7 +61,7 @@ typedef enum {
 
 typedef struct
 {
-	KzEmbed *kzembed;
+	KzWeb *kzweb;
 	gchar *filename;  /* file name for edior */
 	gpointer element; /* pointer indicated textarea */
 } EditorInfo;
@@ -104,7 +104,7 @@ open_location (GtkAction *action, KzWindow *kz, LocationType location, WindowTyp
 		break;
 	case NEW_TAB:
 		kz_window_open_new_tab_with_parent(kz, uri,
-						   GTK_WIDGET(KZ_WINDOW_CURRENT_EMBED(kz)));
+						   GTK_WIDGET(KZ_WINDOW_CURRENT_WEB(kz)));
 		break;
 	case NEW_WINDOW:
 	{
@@ -217,22 +217,22 @@ act_popup_open_frame (GtkAction *action, KzWindow *kz)
 }
 
 static void
-cb_embed_changed (KzEmbed *embed, EditorInfo *einfo)
+cb_web_changed (KzWeb *web, EditorInfo *einfo)
 {
-	g_return_if_fail(KZ_IS_EMBED(embed));
+	g_return_if_fail(KZ_IS_WEB(web));
 	g_return_if_fail(einfo);
 
-	g_signal_handlers_disconnect_by_func(embed,
-			 		     G_CALLBACK(cb_embed_changed),
+	g_signal_handlers_disconnect_by_func(web,
+					     G_CALLBACK(cb_web_changed),
 					     einfo);
 	if (!einfo) return;
 
 	if (einfo->filename)
 		g_free(einfo->filename);
 	einfo->filename = NULL;
-	if (einfo->kzembed)
-		g_object_unref(einfo->kzembed);
-	einfo->kzembed  = NULL;
+	if (einfo->kzweb)
+		g_object_unref(einfo->kzweb);
+	einfo->kzweb  = NULL;
 	einfo->element  = NULL;
 }
 
@@ -249,11 +249,11 @@ cb_editor_exit (GPid pid, gint status, gpointer data)
 	if (!data) return;
 
 	einfo = (EditorInfo*)data;
-	if (einfo->kzembed)
+	if (einfo->kzweb)
 	{
-		g_signal_handlers_disconnect_by_func(einfo->kzembed,
-			 		     G_CALLBACK(cb_embed_changed),
-					     einfo);
+		g_signal_handlers_disconnect_by_func(einfo->kzweb,
+						     G_CALLBACK(cb_web_changed),
+						     einfo);
 	}
 
 	if (einfo->filename)
@@ -264,9 +264,9 @@ cb_editor_exit (GPid pid, gint status, gpointer data)
 		/* set text into textarea */
 		if (ret)
 		{
-			if (KZ_EMBED(einfo->kzembed) && einfo->element)
+			if (KZ_WEB(einfo->kzweb) && einfo->element)
 			{
-				kz_embed_set_text_into_textarea(einfo->kzembed,
+				kz_web_set_text_into_textarea(einfo->kzweb,
 								einfo->element,
 								text);
 			}
@@ -277,11 +277,11 @@ cb_editor_exit (GPid pid, gint status, gpointer data)
 		einfo->filename = NULL;
 	}
 	
-	if (einfo->kzembed)
+	if (einfo->kzweb)
 	{
-		g_object_unref(einfo->kzembed);
+		g_object_unref(einfo->kzweb);
 	}
-	einfo->kzembed  = NULL;
+	einfo->kzweb  = NULL;
 	einfo->element  = NULL;
 	
 	g_free(einfo);
@@ -293,7 +293,7 @@ static void
 act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 {
 	const KzEmbedEventMouse *event;
-	KzEmbed *embed;
+	KzWeb *web;
 	GPid pid;
 	GSpawnFlags flags;
 	gint argc;
@@ -302,8 +302,8 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	EditorInfo *einfo = NULL;
 	
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	embed = KZ_WINDOW_CURRENT_EMBED(kz);
-	if (!embed)
+	web = KZ_WINDOW_CURRENT_WEB(kz);
+	if (!web)
 		return;
 
 	event = kz_window_get_mouse_event_info(kz);
@@ -316,9 +316,9 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	if (event->cinfo.context & KZ_CONTEXT_TEXTAREA)
 	{
 		einfo = g_new0(EditorInfo, 1);
-		einfo->kzembed  = g_object_ref(embed);
+		einfo->kzweb  = g_object_ref(web);
 		einfo->element  = event->cinfo.element;
-		text = kz_embed_get_text_from_textarea(embed,
+		text = kz_web_get_text_from_textarea(web,
 						       einfo->element);
 		/* text in textare store in temporary file */
 		if (text)
@@ -340,10 +340,10 @@ act_popup_launch_editor (GtkAction *action, KzWindow *kz)
 	}
 	
 	/* */
-	g_signal_connect(embed, "kz-net-start",
-			 G_CALLBACK(cb_embed_changed), einfo);
-	g_signal_connect(embed, "destroy",
-			 G_CALLBACK(cb_embed_changed), einfo);
+	g_signal_connect(web, "kz-net-start",
+			 G_CALLBACK(cb_web_changed), einfo);
+	g_signal_connect(web, "destroy",
+			 G_CALLBACK(cb_web_changed), einfo);
 	/* launch editor */
 	g_shell_parse_argv(command,
 			   &argc,
@@ -511,16 +511,16 @@ cb_encoding_menuitem_destroy (GtkWidget *menuitem, gchar *encoding)
 static void
 cb_encoding_menuitem_activate (GtkWidget *menuitem, KzWindow *kz)
 {
-	KzEmbed *embed = KZ_WINDOW_CURRENT_EMBED(kz);
+	KzWeb *web = KZ_WINDOW_CURRENT_WEB(kz);
 
-	if (GTK_CHECK_MENU_ITEM(menuitem)->active && embed)
+	if (GTK_CHECK_MENU_ITEM(menuitem)->active && web)
 	{
 		const gchar* code
 			= g_object_get_data(G_OBJECT(menuitem),
 					    KZ_ACTIONS_POPUP_LANGUAGE_KEY);
 		GtkAction *action;
 
-		kz_embed_set_encoding(embed, code);
+		kz_web_set_encoding(web, code);
 
 		action = gtk_action_group_get_action(kz->actions, "Reload");
 
@@ -608,7 +608,7 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 	GtkWidget *encoding_submenu;
 	GtkWidget *menuitem;
 	GSList *group = NULL;
-	KzEmbed *embed = KZ_WINDOW_CURRENT_EMBED(kz);
+	KzWeb *web = KZ_WINDOW_CURRENT_WEB(kz);
 	gchar *encodings_xml;
 	gchar *current_encoding = NULL;
 	gboolean forced = FALSE;
@@ -628,9 +628,9 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 	encoding_submenu = gtk_menu_new();
 
 	/* get current charset */
-	if(embed)
+	if(web)
 	{
-		kz_embed_get_encoding(embed,
+		kz_web_get_encoding(web,
 				      &current_encoding, &forced);
 	}
 
@@ -755,19 +755,19 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 
 	for (i = 0; i < num; i++)
 	{
-		KzEmbed *kzembed = KZ_WINDOW_NTH_EMBED(kz, i);
+		KzWeb *kzweb = KZ_WINDOW_NTH_WEB(kz, i);
 		GtkWidget *favicon;
 		gchar *title, *escaped, *markup_title;
 		GtkWidget *menuitem;
 		KzTabLabel *tab;
 		KzTabLabelState state;
 
-		if (!kzembed)
+		if (!kzweb)
                         continue;
 			
-		tab = kz_window_get_tab_label(kz, kzembed);
+		tab = kz_window_get_tab_label(kz, kzweb);
 		state = kz_tab_label_get_state(tab);
-		title = kz_embed_ensure_title(kzembed);
+		title = kz_web_ensure_title(kzweb);
 		escaped = g_markup_escape_text(title,
 					       strlen(title));
 		menuitem = gtk_image_menu_item_new_with_label(title);
@@ -791,7 +791,7 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 		
 		/* favicon */
 		favicon = kz_favicon_get_widget(kzfav,
-						kz_embed_get_location(kzembed),
+						kz_web_get_location(kzweb),
 						KZ_ICON_SIZE_BOOKMARK_MENU);
 		if (favicon)
 		{
@@ -800,7 +800,7 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 						      favicon);
 		}
 
-		g_object_set_data (G_OBJECT(menuitem), KZ_ACTIONS_POPUP_TAB_KEY, kzembed);
+		g_object_set_data (G_OBJECT(menuitem), KZ_ACTIONS_POPUP_TAB_KEY, kzweb);
 
 		g_signal_connect(menuitem, "activate",
 				 G_CALLBACK(cb_tablist_menuitem_activate), kz);
