@@ -40,14 +40,12 @@ get_tab_object (KzWindow *kz)
 
 	if (!kztab)
 	{
-		GtkWidget *label;
-		GtkWidget *kzembed;
+		gint current_page;
 
-		kzembed = GTK_WIDGET(KZ_WINDOW_CURRENT_PAGE(kz));
-		label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-						   kzembed);
-		if (KZ_IS_TAB_LABEL(label))
-			kztab = KZ_TAB_LABEL(label);
+		current_page = kz_notebook_get_current_page(KZ_NOTEBOOK(kz->notebook));
+
+		kztab = kz_notebook_get_nth_tab_label(KZ_NOTEBOOK(kz->notebook),
+						      current_page);
 	}
 
 	return kztab;
@@ -63,8 +61,8 @@ act_tab_reload (GtkAction *action, KzWindow *kz)
 	kztab = g_object_get_qdata(G_OBJECT(kz), kztab_quark);
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	kz_embed_reload(kztab->kzembed,
-			KZ_EMBED_RELOAD_NORMAL);
+	kz_web_reload(kztab->kzweb,
+			KZ_WEB_RELOAD_NORMAL);
 }
 
 static void
@@ -77,7 +75,7 @@ act_tab_stop (GtkAction *action, KzWindow *kz)
 	kztab = g_object_get_qdata(G_OBJECT(kz), kztab_quark);
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	kz_embed_stop_load(kztab->kzembed);
+	kz_web_stop_load(kztab->kzweb);
 }
 
 static void
@@ -90,7 +88,7 @@ act_tab_close (GtkAction *action, KzWindow *kz)
 	kztab = g_object_get_qdata(G_OBJECT(kz), kztab_quark);
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	kz_window_close_tab(kztab->kz, GTK_WIDGET(kztab->kzembed));
+	kz_window_close_tab(kztab->kz, kztab->kzweb);
 }
 
 enum {
@@ -113,7 +111,7 @@ close_tabs (KzWindow *kz, KzNotebookCloseCondition condition)
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	notebook = KZ_NOTEBOOK(kztab->kz->notebook);
 
-	pos = kz_notebook_page_num(notebook, GTK_WIDGET(kztab->kzembed));
+	pos = kz_notebook_page_num(notebook, GTK_WIDGET(kztab->kzweb));
 
 	kz_notebook_close_tabs(notebook, condition, pos);
 }
@@ -157,7 +155,7 @@ get_all_under_current_tab (GNode *node, gpointer data)
 static void
 act_tab_close_all_child (GtkAction *action, KzWindow *kz)
 {
-	KzEmbed *kzembed;
+	KzWeb *kzweb;
 	KzTabLabel *kztab;
 	GNode *tree, *parent;
 	GList *list = NULL, *node;
@@ -166,9 +164,9 @@ act_tab_close_all_child (GtkAction *action, KzWindow *kz)
 
 	kztab = g_object_get_qdata(G_OBJECT(kz), kztab_quark);
 
-	kzembed = kztab->kzembed;
+	kzweb = kztab->kzweb;
 	tree = kz_window_get_tree(kz);
-	parent = g_node_find(tree, G_IN_ORDER, G_TRAVERSE_ALL, kzembed);
+	parent = g_node_find(tree, G_IN_ORDER, G_TRAVERSE_ALL, kzweb);
 	g_return_if_fail(parent);
 
 	g_node_traverse(parent, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
@@ -177,8 +175,8 @@ act_tab_close_all_child (GtkAction *action, KzWindow *kz)
 	{
 		GtkWidget *widget = node->data;
 
-		if (widget != GTK_WIDGET(kzembed))
-			kz_window_close_tab(kz, widget);
+		if (widget != GTK_WIDGET(kzweb))
+			kz_window_close_tab(kz, KZ_WEB(widget));
 	}
 	g_list_free(list);
 }
@@ -202,21 +200,17 @@ act_tab_close_all_same_site (GtkAction *action, KzWindow *kz)
 	num = kz_notebook_get_n_pages(notebook);
 	if (num < 2) return;
 
-	src = g_strsplit(kz_embed_get_location(KZ_EMBED(kztab->kzembed)),
+	src = g_strsplit(kz_web_get_location(KZ_WEB(kztab->kzweb)),
 			"/", 0);
 
 	for (i = num - 1; i >= 0; i--)
 	{
-		GtkWidget *widget = KZ_WINDOW_NTH_PAGE(kz, i);
-		kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-							widget));
-		target = g_strsplit(kz_embed_get_location(KZ_EMBED(kztab->kzembed)),
-				"/", 0);
+		KzWeb *web;
+		web = KZ_WINDOW_NTH_WEB(kz, i);
+		target = g_strsplit(kz_web_get_location(web), "/", 0);
 
 		if(!(strcmp(src[2] , target[2])))
-		{
-			kz_window_close_tab(kz, widget);
-		}
+			kz_window_close_tab(kz, web);
 		g_strfreev(target);
 	}
 	g_strfreev(src);
@@ -232,7 +226,7 @@ act_tab_copy_title (GtkAction *action, KzWindow *kz)
 	kztab = g_object_get_qdata(G_OBJECT(kz), kztab_quark);
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	gtkutil_copy_text(kz_embed_get_title(KZ_EMBED(kztab->kzembed)));
+	gtkutil_copy_text(kz_web_get_title(KZ_WEB(kztab->kzweb)));
 }
 
 static void
@@ -245,7 +239,7 @@ act_tab_copy_location (GtkAction *action, KzWindow *kz)
 	kztab = g_object_get_qdata(G_OBJECT(kz), kztab_quark);
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	gtkutil_copy_text(kz_embed_get_location(KZ_EMBED(kztab->kzembed)));
+	gtkutil_copy_text(kz_web_get_location(KZ_WEB(kztab->kzweb)));
 }
 
 static void
@@ -260,9 +254,9 @@ act_tab_copy_title_and_location (GtkAction *action, KzWindow *kz)
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	str = g_strdup_printf("<a href=\"%s\" title=\"%s\">%s</a>",
-			      kz_embed_get_location(KZ_EMBED(kztab->kzembed)),
-			      kz_embed_get_title(KZ_EMBED(kztab->kzembed)),
-			      kz_embed_get_title(KZ_EMBED(kztab->kzembed)));
+			      kz_web_get_location(KZ_WEB(kztab->kzweb)),
+			      kz_web_get_title(KZ_WEB(kztab->kzweb)),
+			      kz_web_get_title(KZ_WEB(kztab->kzweb)));
 	gtkutil_copy_text(str);
 	g_free(str);
 }
@@ -314,7 +308,7 @@ act_tab_toggle_javascript (GtkAction *action, KzWindow *kz)
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action));
 
-	kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), active);
+	kz_web_set_allow_javascript(KZ_WEB(kztab->kzweb), active);
 	kz_tab_label_set_javascript(kztab, active);
 }
 
@@ -332,23 +326,22 @@ act_tab_toggle_images (GtkAction *action, KzWindow *kz)
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action));
 
-	kz_embed_set_allow_images(KZ_EMBED(kztab->kzembed), active);
+	kz_web_set_allow_images(KZ_WEB(kztab->kzweb), active);
 }
 
 static void
 act_tab_copy_tab (GtkAction *action, KzWindow *kz)
 {
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
-	KzEmbed *kzembed, *dkzembed;
+	KzWeb *src_web, *dest_web;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_EMBED(widget));
 	
-	kzembed  = KZ_EMBED(widget);	
-	dkzembed = KZ_EMBED(kz_window_open_new_tab(kz, NULL));
+	src_web  = KZ_WINDOW_CURRENT_WEB(kz);
+	if (!src_web)
+		return;
+	dest_web = KZ_WEB(kz_window_open_new_tab(kz, NULL));
 
-	kz_embed_copy_page(kzembed, dkzembed, KZ_EMBED_COPY_NORMAL);
-	kz_embed_shistory_copy(kzembed, dkzembed, TRUE, TRUE, TRUE);
+	kz_web_copy_page(src_web, dest_web);
 }
 
 GtkActionEntry kz_actions_tab[] =
@@ -457,34 +450,32 @@ set_popup_menu_sensitive (KzTabLabel *kztab)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	pos = kz_notebook_page_num(KZ_NOTEBOOK(kztab->kz->notebook),
-				   GTK_WIDGET(kztab->kzembed));
+				   GTK_WIDGET(kztab->kzweb));
 	num = kz_notebook_get_n_pages(KZ_NOTEBOOK(kztab->kz->notebook));
 
 	action = gtk_action_group_get_action(kz->tabpop_actions, "TabStop");
-	g_object_set(action, "sensitive", 
-		     kz_embed_is_loading(kztab->kzembed),
-		     NULL);
+	gtk_action_set_sensitive(action, kz_web_is_loading(kztab->kzweb));
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabAddFeedBookmark");
-	if (kztab->kzembed &&
-	    kz_embed_get_nav_link(kztab->kzembed, KZ_EMBED_LINK_RSS))
+	if (kztab->kzweb &&
+	    kz_web_get_nav_link(kztab->kzweb, KZ_WEB_LINK_RSS))
 		has_feed = TRUE;
 	else
 		has_feed = FALSE;
-	g_object_set(action, "sensitive", has_feed, NULL);
+	gtk_action_set_sensitive(action, has_feed);
 	
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabCloseAllBackward");
-	g_object_set(action, "sensitive", !(pos == 0), NULL);
+	gtk_action_set_sensitive(action, !(pos == 0));
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabCloseAllForward");
-	g_object_set(action, "sensitive", !(pos == num -1), NULL);
+	gtk_action_set_sensitive(action, !(pos == num -1));
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabCloseAllInactiveTabs");
-	g_object_set(action, "sensitive", num > 1, NULL);
+	gtk_action_set_sensitive(action, num > 1);
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabToggleLock");
@@ -498,17 +489,17 @@ set_popup_menu_sensitive (KzTabLabel *kztab)
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabToggleJavascript");
-	javascript = kz_embed_get_allow_javascript(KZ_EMBED(kztab->kzembed));
+	javascript = kz_web_get_allow_javascript(KZ_WEB(kztab->kzweb));
 	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), javascript);
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabToggleImages");
-	images = kz_embed_get_allow_images(KZ_EMBED(kztab->kzembed));
+	images = kz_web_get_allow_images(KZ_WEB(kztab->kzweb));
 	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), images);
 
 	action = gtk_action_group_get_action(kz->tabpop_actions,
 					     "TabClose");
-	g_object_set(action, "sensitive", !lock, NULL);
+	gtk_action_set_sensitive(action, !lock);
 }
 
 
