@@ -32,7 +32,7 @@
 
 #include <gtk/gtk.h>
 #include "kz-window.h"
-#include "kz-embed.h"
+#include "kz-web.h"
 #include "kz-bookmark.h"
 
 G_BEGIN_DECLS
@@ -43,11 +43,11 @@ extern const gint     kz_actions_len;
 GtkActionGroup *kz_actions_create_group              (KzWindow      *kz);
 
 void            kz_actions_set_sensitive             (KzWindow      *kz,
-						      KzEmbed       *kzembed);
+						      KzWeb       *kzweb);
 void            kz_actions_set_tab_sensitive         (KzWindow      *kz,
-						      KzEmbed       *kzembed);
+						      KzWeb       *kzweb);
 void            kz_actions_set_selection_sensitive   (KzWindow      *kz,
-						      KzEmbed       *kzembed);
+						      KzWeb       *kzweb);
 
 KzBookmark     *kz_actions_get_bookmark_for_action   (KzWindow      *kz);
 void            kz_actions_set_bookmark_for_action   (KzWindow      *kz,
