@@ -23,25 +23,20 @@
 #include <kz-entry.h>
 #include <glib/gi18n.h>
 
-#include "gobject-utils.h"
-
 enum {
   PROP_0,
   PROP_TEXT
 };
 
-static void       kz_entry_action_init         (KzEntryAction *action);
-static void       kz_entry_action_class_init   (KzEntryActionClass *class);
-
-static void       kz_entry_action_dispose      (GObject *object);
-static void       kz_entry_action_set_property (GObject      *object,
-						 guint         prop_id,
-						 const GValue *value,
-						 GParamSpec   *pspec);
-static void       kz_entry_action_get_property (GObject      *object,
-						 guint         prop_id,
-						 GValue       *value,
-						 GParamSpec   *pspec);
+static void       dispose      (GObject *object);
+static void       set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec);
+static void       get_property (GObject      *object,
+                                guint         prop_id,
+                                GValue       *value,
+                                GParamSpec   *pspec);
 
 static GtkWidget *create_menu_item              (GtkAction *action);
 static GtkWidget *create_tool_item              (GtkAction *action);
@@ -54,11 +49,7 @@ static void       disconnect_proxy              (GtkAction *action,
 static GtkEntry  *kz_entry_action_real_get_entry_widget (KzEntryAction *action,
 							  GtkWidget      *proxy);
 
-static GtkActionClass *parent_class = NULL;
-
-KZ_OBJECT_GET_TYPE(kz_entry_action, "KzEntryAction", KzEntryAction,
-		   kz_entry_action_class_init, kz_entry_action_init,
-		   GTK_TYPE_ACTION)
+G_DEFINE_TYPE(KzEntryAction, kz_entry_action, GTK_TYPE_ACTION)
 
 static void
 kz_entry_action_class_init (KzEntryActionClass *klass)
@@ -66,13 +57,12 @@ kz_entry_action_class_init (KzEntryActionClass *klass)
 	GObjectClass *object_class;
 	GtkActionClass *action_class;
 
-	parent_class = g_type_class_peek_parent(klass);
 	object_class = G_OBJECT_CLASS(klass);
 	action_class = GTK_ACTION_CLASS(klass);
 
-	object_class->dispose      = kz_entry_action_dispose;
-	object_class->set_property = kz_entry_action_set_property;
-	object_class->get_property = kz_entry_action_get_property;
+	object_class->dispose      = dispose;
+	object_class->set_property = set_property;
+	object_class->get_property = get_property;
 
 	action_class->create_menu_item = create_menu_item;
 	action_class->create_tool_item = create_tool_item;
@@ -101,7 +91,7 @@ kz_entry_action_init (KzEntryAction *action)
 
 
 static void
-kz_entry_action_dispose (GObject *object)
+dispose (GObject *object)
 {
 	KzEntryAction *action = KZ_ENTRY_ACTION(object);
 
@@ -109,16 +99,16 @@ kz_entry_action_dispose (GObject *object)
 		g_free (action->text);
 	action->text = NULL;
 
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(object);
+	if (G_OBJECT_CLASS (kz_entry_action_parent_class)->dispose)
+		G_OBJECT_CLASS (kz_entry_action_parent_class)->dispose(object);
 }
 
 
 static void
-kz_entry_action_set_property (GObject      *object,
-			       guint         prop_id,
-			       const GValue *value,
-			       GParamSpec   *pspec)
+set_property (GObject      *object,
+              guint         prop_id,
+              const GValue *value,
+              GParamSpec   *pspec)
 {
 	KzEntryAction *action;
 
@@ -138,10 +128,10 @@ kz_entry_action_set_property (GObject      *object,
 
 
 static void
-kz_entry_action_get_property (GObject    *object,
-			       guint       prop_id,
-			       GValue     *value,
-			       GParamSpec *pspec)
+get_property (GObject    *object,
+              guint       prop_id,
+              GValue     *value,
+              GParamSpec *pspec)
 {
 	KzEntryAction *action;
 
@@ -158,7 +148,7 @@ kz_entry_action_get_property (GObject    *object,
 	}
 }
 
-
+#if 0
 static gboolean
 cb_menu_item_entry_button_press (GtkWidget *widget)
 {
@@ -192,7 +182,7 @@ cb_menu_item_entry_leave (GtkWidget *widget, GdkEventCrossing *event)
 
 	return TRUE;
 }
-
+#endif
 
 static GtkWidget *
 create_menu_item (GtkAction *action)
@@ -204,19 +194,19 @@ create_menu_item (GtkAction *action)
 	entry = kz_entry_new();
 	gtk_widget_show(entry);
 	gtk_container_add(GTK_CONTAINER(widget), entry);
-
-	g_signal_connect_after(G_OBJECT(widget), "button-press-event",
+#if 0
+	g_signal_connect_after(widget, "button-press-event",
 			       G_CALLBACK(cb_menu_item_entry_button_press),
 			       NULL);
-	g_signal_connect_after(G_OBJECT(widget), "button-release-event",
+	g_signal_connect_after(widget, "button-release-event",
 			       G_CALLBACK(cb_menu_item_entry_button_release),
 			       NULL);
 
-	g_signal_connect(G_OBJECT(widget), "enter-notify-event",
+	g_signal_connect(widget, "enter-notify-event",
 			 G_CALLBACK(cb_menu_item_entry_enter), NULL);
-	g_signal_connect(G_OBJECT(widget), "leave-notify-event",
+	g_signal_connect(widget, "leave-notify-event",
 			 G_CALLBACK(cb_menu_item_entry_leave), NULL);
-
+#endif
 	return widget;
 }
 
@@ -227,19 +217,16 @@ cb_entry_changed(GtkEditable *editable, GtkAction *action)
 	GtkEntry *entry;
 	const gchar *text;
 
-	g_return_if_fail(GTK_IS_ENTRY(editable));
-
 	entry = GTK_ENTRY(editable);
 	text = gtk_entry_get_text(entry);
 
-	g_object_set(G_OBJECT(action), "text", text, NULL);
+	g_object_set(action, "text", text, NULL);
 }
 
 
 static void
 cb_entry_activate(GtkEntry *entry, GtkAction *action)
 {
-	g_return_if_fail(GTK_IS_ENTRY(entry));
 	g_return_if_fail(KZ_IS_ENTRY_ACTION(action));
 
 	gtk_action_activate(action);
@@ -270,7 +257,7 @@ create_tool_item (GtkAction *action)
 {
 	GtkWidget *widget, *entry;
 
-	widget = (*GTK_ACTION_CLASS(parent_class)->create_tool_item) (action);
+	widget = (*GTK_ACTION_CLASS(kz_entry_action_parent_class)->create_tool_item) (action);
 	gtk_tool_item_set_expand (GTK_TOOL_ITEM(widget), TRUE);
 
 	entry = kz_entry_new();
@@ -378,7 +365,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		if (entry_action->text)
 			gtk_entry_set_text(entry, entry_action->text);
 	}
-	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
+	GTK_ACTION_CLASS (kz_entry_action_parent_class)->connect_proxy (action, proxy);
 }
 
 
@@ -400,7 +387,7 @@ disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 	}
 
 	/* other signals will be remove by parent */
-	GTK_ACTION_CLASS (parent_class)->disconnect_proxy(action, proxy);
+	GTK_ACTION_CLASS (kz_entry_action_parent_class)->disconnect_proxy(action, proxy);
 }
 
 
