@@ -127,7 +127,7 @@ dispose (GObject *object)
 
 	if (action->completion_regex)
 	{
-		g_regex_free(action->completion_regex);
+		g_regex_unref(action->completion_regex);
 		action->completion_regex = NULL;
 	}
 
@@ -296,7 +296,7 @@ completion_func (GtkEntryCompletion *completion,
 	if (action->completion_previous_key &&
 	    strcmp(action->completion_previous_key, key))
 	{
-		g_regex_free(action->completion_regex);
+		g_regex_unref(action->completion_regex);
 		action->completion_regex = NULL;
 
 		g_free(action->completion_previous_key);
@@ -338,7 +338,8 @@ completion_func (GtkEntryCompletion *completion,
 	}
 
 	if (action->completion_regex &&
-	    g_regex_match(action->completion_regex, case_normalized_string, 0))
+	    g_regex_match(action->completion_regex, case_normalized_string, 0,
+			  NULL))
 		ret = TRUE;
 
 	g_free(item);
