@@ -146,9 +146,9 @@ dispose (GObject *obj)
 
 	if (action->kz)
 	{
-		kz_profile_set_save_each_time(KZ_GET_GLOBAL_PROFILE, FALSE);
+		kz_profile_set_auto_save(KZ_GET_GLOBAL_PROFILE, FALSE);
 		kz_location_entry_action_store_history(action);
-		kz_profile_set_save_each_time(KZ_GET_GLOBAL_PROFILE, TRUE);
+		kz_profile_set_auto_save(KZ_GET_GLOBAL_PROFILE, TRUE);
 		g_object_unref(action->kz);
 		action->kz = NULL;
 	}
@@ -340,9 +340,9 @@ kz_location_entry_action_new (KzWindow *kz)
 			      "stock_id",    GTK_STOCK_NEW,
 			      "kz-window",   kz,
 			      NULL);
-	kz_profile_set_save_each_time(KZ_GET_GLOBAL_PROFILE, FALSE);
+	kz_profile_set_auto_save(KZ_GET_GLOBAL_PROFILE, FALSE);
 	kz_location_entry_action_restore_history(action);
-	kz_profile_set_save_each_time(KZ_GET_GLOBAL_PROFILE, TRUE);
+	kz_profile_set_auto_save(KZ_GET_GLOBAL_PROFILE, TRUE);
 
 	return action;
 }
