@@ -309,7 +309,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 				 G_CALLBACK(cb_entry_key_press), action);
 		g_signal_connect(entry, "populate-popup",
 				 G_CALLBACK(cb_entry_populate_popup), action);
-		g_signal_connect(entry, "icon-pressed",
+		g_signal_connect(entry, "icon-press",
 				 G_CALLBACK(cb_entry_icon_pressed), action);
 	
 		g_signal_connect(bookmark, "notify",
@@ -492,8 +492,7 @@ cb_bookmark_notify (GObject *object,
 
 			if (KZ_IS_ENTRY(entry))
 			{
-				kz_entry_set_backtext(KZ_ENTRY(entry),
-						      title);
+				kz_entry_set_background_text(KZ_ENTRY(entry), title);
 			}
 		}
 		g_free(title);
@@ -875,25 +874,24 @@ update_icon_and_bgtext(KzSmartBookmarkAction *action)
 
 		if (KZ_IS_ENTRY(entry))
 		{
-			kz_entry_set_backtext(KZ_ENTRY(entry),
-					      title);
+			kz_entry_set_background_text(KZ_ENTRY(entry), title);
 			kz_entry_set_arrow(KZ_ENTRY(entry),
 					   is_folder);
-	
-			if (favicon)
-			{
-				kz_entry_set_icon_from_pixbuf(KZ_ENTRY(entry),
-							      favicon);
-				g_object_unref(favicon);
-			}
-			else
-			{
-				kz_entry_set_icon_from_stock(KZ_ENTRY(entry),
-					     		     stock_id,
-				     			     GTK_ICON_SIZE_MENU);
-			}
-			gtk_widget_queue_resize(GTK_WIDGET(entry));
 		}
+		if (favicon)
+		{
+			gtk_entry_set_icon_from_pixbuf(entry,
+						       GTK_ENTRY_ICON_PRIMARY,
+						       favicon);
+			g_object_unref(favicon);
+		}
+		else
+		{
+			gtk_entry_set_icon_from_stock(entry,
+						      GTK_ENTRY_ICON_PRIMARY,
+						      stock_id);
+		}
+		gtk_widget_queue_resize(GTK_WIDGET(entry));
 	}
 }
 
