@@ -202,24 +202,25 @@ static void
 cb_spin_value_changed (GtkSpinButton *spin, KzZoomAction *action)
 {
 	gint ratio;
-	GtkWidget *embed;
+	KzWeb *web;
 	KzZoomActionPrivate *priv = KZ_ZOOM_ACTION_GET_PRIVATE (action);
 
 	if (!priv->spin || !GTK_WIDGET_VISIBLE(priv->spin))
 		return;
 
+	web = KZ_WINDOW_CURRENT_WEB(priv->kz);
+	if (!web)
+		return;
+	
 	ratio = gtk_spin_button_get_value_as_int (spin);
 	
-	embed = KZ_WINDOW_CURRENT_PAGE(priv->kz);
-	
-	if (KZ_EMBED(embed))
-		kz_embed_zoom_set(KZ_EMBED(embed), ratio, TRUE);
+	kz_web_zoom(web, (gdouble)ratio / 100);
 }
 
 
 void
 kz_zoom_action_set_ratio (KzZoomAction *action,
-			  gint ratio) 
+			  gdouble ratio) 
 {
 	GtkWidget *spin;
 	KzZoomActionPrivate *priv;
@@ -233,24 +234,24 @@ kz_zoom_action_set_ratio (KzZoomAction *action,
 	spin = priv->spin;
 	g_signal_handlers_block_by_func(spin,
 			G_CALLBACK(cb_spin_value_changed), action);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), ratio);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), ratio * 100);
 	g_signal_handlers_unblock_by_func(spin,
 			G_CALLBACK(cb_spin_value_changed), action);
 }
 
-gint
+gdouble
 kz_zoom_action_get_ratio (KzZoomAction *action)
 {
 	gint ratio;
 	KzZoomActionPrivate *priv;
 	
-	g_return_val_if_fail(KZ_IS_ZOOM_ACTION(action), 100);
+	g_return_val_if_fail(KZ_IS_ZOOM_ACTION(action), 1.0);
 	
 	priv = KZ_ZOOM_ACTION_GET_PRIVATE (action);
 	
 	if (!priv->spin || !GTK_WIDGET_VISIBLE(priv->spin))
-		return 100;
-	ratio = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(priv->spin));
+		return 1.0;
+	ratio = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(priv->spin)) / 100;
     
 	return ratio;
 }
