@@ -49,7 +49,7 @@ GType         kz_zoom_action_get_type  (void) G_GNUC_CONST;
 KzZoomAction *kz_zoom_action_new       (KzWindow   *kz);
 
 void	      kz_zoom_action_set_ratio (KzZoomAction *action,
-					gint ratio);
-gint	      kz_zoom_action_get_ratio (KzZoomAction *action);
+					gdouble ratio);
+gdouble	      kz_zoom_action_get_ratio (KzZoomAction *action);
 
 #endif /* KZ_ZOOM_ACTION_H */
