@@ -35,6 +35,7 @@
 #include "kz-bookmark-item.h"
 #include "kz-bookmark.h"
 #include "kz-bookmark-file.h"
+#include "kz-bookmark-separator.h"
 #include "kz-bookmark-utils.h"
 #include "kz-actions.h"
 #include "kz-notebook.h"
@@ -132,8 +133,8 @@ kz_bookmark_bar_class_init (KzBookmarkBarClass *klass)
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	gobject_class = (GObjectClass *) klass;
-	widget_class  = (GtkWidgetClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	widget_class  = GTK_WIDGET_CLASS(klass);
 
 	/* GObject class */
 	gobject_class->constructor  = constructor;
@@ -337,26 +338,24 @@ remove_item (GtkWidget *child, gpointer data)
 	gtk_container_remove(GTK_CONTAINER(child->parent), child);
 }
 
+static void
+insert_tool_item (KzBookmark *bookmark, KzBookmarkBar *bar)
+{
+	GtkToolItem *toolitem;
+	
+	toolitem = create_tool_item(bar, bookmark);
+	gtk_toolbar_insert(GTK_TOOLBAR(bar->toolbar),
+			   toolitem, -1);
+}
 
 static void
 kz_bookmark_bar_refresh_all (KzBookmarkBar *bar)
 {
-	GList *children, *node;
-
 	gtk_container_foreach (GTK_CONTAINER (GTK_TOOLBAR(bar->toolbar)),
 			       remove_item, NULL);
 
-	children = kz_bookmark_get_children(bar->folder);
-	for (node = children; node; node = g_list_next(node))
-	{
-		GtkToolItem *toolitem;
-		KzBookmark *bookmark = node->data;
-		
-		toolitem = create_tool_item(bar, bookmark);
-		gtk_toolbar_insert(GTK_TOOLBAR(bar->toolbar),
-				   toolitem, -1);
-	}
-	g_list_free(children);
+	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bar->folder),
+					 (GFunc)insert_tool_item, bar);
 
 	gtk_widget_queue_resize(GTK_WIDGET(bar));
 }
@@ -401,9 +400,9 @@ drag_data_received (GtkWidget *widget,
                     guint time)
 {
 	KzBookmarkBar *bar;
-	KzBookmark *bookmark = NULL, *sibling, *parent;
+	KzBookmark *bookmark = NULL, *sibling;
+        KzBookmarkFolder *parent;
 	GtkWidget *src_widget;
-	GList *children;
 	gint index;
 	gboolean moved = FALSE, success = FALSE;
 
@@ -411,22 +410,20 @@ drag_data_received (GtkWidget *widget,
 	index = gtk_toolbar_get_drop_index(GTK_TOOLBAR(bar->toolbar),
 					   x, y);
 
-	children = kz_bookmark_get_children(bar->folder);
-	sibling = g_list_nth_data(children, index);
-	g_list_free(children);
+	sibling = kz_bookmark_folder_get_nth_child(KZ_BOOKMARK_FOLDER(bar->folder), index);
 
 	switch (info)
 	{
 	 case TARGET_GTK_NOTEBOOK_TAB:
 	 {
-	 	KzEmbed *kzembed;
+	 	KzWeb *kzweb;
 		const gchar *uri, *title;
 		src_widget = gtk_drag_get_source_widget(context);
 		if (!KZ_IS_NOTEBOOK(src_widget))
 			break;
-		kzembed = KZ_EMBED(kz_notebook_get_dragged_page(KZ_NOTEBOOK(src_widget)));
-		uri   = kz_embed_get_location(kzembed);
-		title = kz_embed_get_title(kzembed);
+		kzweb = KZ_WEB(kz_notebook_get_dragged_page(KZ_NOTEBOOK(src_widget)));
+		uri   = kz_web_get_location(kzweb);
+		title = kz_web_get_title(kzweb);
 		bookmark = KZ_BOOKMARK(kz_bookmark_new_with_attrs(title, uri, NULL));
 	 	break;
 	 }
@@ -445,7 +442,7 @@ drag_data_received (GtkWidget *widget,
 	
 		g_object_ref(bookmark);
 		parent = kz_bookmark_get_parent(bookmark);
-		kz_bookmark_remove(parent, bookmark);
+		kz_bookmark_folder_remove(parent, bookmark);
 		moved = TRUE;
 	 	break;
 	 case TARGET_NETSCAPE_URL:
@@ -460,16 +457,15 @@ drag_data_received (GtkWidget *widget,
 	{
 		if (sibling)
 		{
-			kz_bookmark_insert_before(bar->folder,
+			kz_bookmark_folder_insert_before(KZ_BOOKMARK_FOLDER(bar->folder),
 						  bookmark,
 						  sibling);
 		}
 		else
 		{
-			kz_bookmark_append(bar->folder, bookmark);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bar->folder), bookmark);
 		}
 		g_object_unref(bookmark);
-		kz_bookmark_file_save(KZ_BOOKMARK_FILE(bar->folder));
 		success = TRUE;
 	}
 
@@ -546,18 +542,13 @@ cb_bookmark_list_insert_child  (KzBookmark *folder,
 				KzBookmarkBar *bar)
 {
 	gint index = -1;
-	GList *children;
 	GtkToolItem *toolitem;
 	
 	g_return_if_fail(KZ_IS_BOOKMARK(child));
 	g_return_if_fail(KZ_IS_BOOKMARK_BAR(bar));
 
 	if (sibling)
-	{
-		children = kz_bookmark_get_children(folder);
-		index = g_list_index(children, sibling); 
-		g_list_free(children);
-	}
+		index = kz_bookmark_folder_get_child_index(KZ_BOOKMARK_FOLDER(folder), sibling);
 
 	toolitem = create_tool_item(bar, child);
 	
@@ -572,17 +563,13 @@ cb_bookmark_list_remove_child  (KzBookmark *folder,
 				KzBookmarkBar *bar)
 {
 	gint index;
-	GList *children;
 	GtkToolItem *toolitem;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(child));
 	g_return_if_fail(KZ_IS_BOOKMARK_BAR(bar));
 
-	children = kz_bookmark_get_children(folder);
-	index = g_list_index(children, child); 
-	g_list_free(children);
-
-	if (index == -1) return;
+	index = kz_bookmark_folder_get_child_index(KZ_BOOKMARK_FOLDER(folder), child);
+	if (index < 0) return;
 	
 	toolitem = gtk_toolbar_get_nth_item(GTK_TOOLBAR(bar->toolbar),
 					    index);
