@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-base-menu-item.c 3265 2007-10-04 04:41:08Z ikezoe $
+ *  $Id: kz-bookmark-base-menu-item.c 3675 2009-02-14 01:40:31Z ikezoe $
  */
 
 #include "kz-bookmark-base-menu-item.h"
@@ -59,9 +59,8 @@ kz_bookmark_base_menu_item_class_init (KzBookmarkBaseMenuItemClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class   = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
-	/* GObject signals */
 	gobject_class->dispose      = dispose;
 	gobject_class->set_property = set_property;
 	gobject_class->get_property = get_property;
