@@ -37,15 +37,12 @@ G_BEGIN_DECLS
 
 typedef struct _KzBookmarkEditWin      KzBookmarkEditWin;
 typedef struct _KzBookmarkEditWinClass KzBookmarkEditWinClass;
-typedef struct _KzBookmarkEditWinPriv  KzBookmarkEditWinPriv;
 
 struct _KzBookmarkEditWin
 {
 	GtkDialog dialog;
 
 	GtkWidget *bookmark_edit;
-
-	KzBookmarkEditWinPriv *priv;
 };
 
 struct _KzBookmarkEditWinClass
