@@ -18,11 +18,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-edit.c 2637 2006-12-02 07:30:49Z ikezoe $
+ *  $Id: kz-bookmark-edit.c 3756 2009-02-22 11:44:12Z ikezoe $
  */
 
 #include <glib/gi18n.h>
 #include "kz-bookmark-edit.h"
+#include "kz-bookmark-separator.h"
 #include "kz-smart-bookmark.h"
 #include "gtk-utils.h"
 #include "kz-thumbnail.h"
@@ -88,7 +89,7 @@ kz_bookmark_edit_class_init (KzBookmarkEditClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *)   klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->dispose = dispose;
 }
@@ -347,11 +348,9 @@ dispose (GObject *object)
 		edit->smart->store = NULL;
 	}
 
-	if (edit->smart)
-	{
-		g_free(edit->smart);
-		edit->smart = NULL;
-	}
+	g_free(edit->smart);
+	edit->smart = NULL;
+
 	if (G_OBJECT_CLASS (kz_bookmark_edit_parent_class)->dispose)
 		G_OBJECT_CLASS (kz_bookmark_edit_parent_class)->dispose(object);
 }
@@ -429,7 +428,7 @@ static void
 kz_bookmark_edit_set_sensitive (KzBookmarkEdit *edit)
 {
 	gboolean can_edit_title = TRUE;
-	gboolean can_edit_uri = TRUE;
+	gboolean can_edit_uri = FALSE;
 	gboolean can_edit_desc = TRUE;
 	gboolean can_edit_location = FALSE;
 	gboolean can_edit_interval = FALSE;
@@ -442,19 +441,15 @@ kz_bookmark_edit_set_sensitive (KzBookmarkEdit *edit)
 	    kz_bookmark_is_separator(edit->bookmark))
 	{
 		can_edit_title    = FALSE;
-		can_edit_uri      = FALSE;
 		can_edit_desc     = FALSE;
 		can_edit_location = FALSE;
 		can_edit_interval = FALSE;
 	}
-	else if (edit->bookmark && kz_bookmark_is_pure_folder(edit->bookmark))
-	{
-		can_edit_uri = FALSE;
-	}
 	if (edit->bookmark && KZ_IS_BOOKMARK_FILE(edit->bookmark))
 	{
 		can_edit_location = TRUE;
 		can_edit_interval = TRUE;
+		can_edit_uri = TRUE;
 	}
 
 	gtk_editable_set_editable(GTK_EDITABLE(edit->title_entry),
@@ -490,7 +485,7 @@ kz_bookmark_edit_set_sensitive (KzBookmarkEdit *edit)
 	else
 	{
 		if (edit->bookmark &&
-		    kz_bookmark_is_pure_folder(edit->bookmark))
+		    KZ_IS_BOOKMARK_FOLDER(edit->bookmark))
 		{
 			gtk_widget_hide(edit->uri_label);
 			gtk_widget_hide(edit->uri_entry);
