@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-editor.c 3081 2007-03-28 03:53:51Z kous $
+ *  $Id: kz-bookmark-editor.c 3675 2009-02-14 01:40:31Z ikezoe $
  */
 
 #include "kz-bookmark-editor.h"
@@ -32,6 +32,7 @@
 #include "kz-icons.h"
 #include "kz-bookmarks-view.h"
 #include "kz-bookmark-file.h"
+#include "kz-bookmark-separator.h"
 
 
 enum {
@@ -82,8 +83,8 @@ kz_bookmark_editor_class_init (KzBookmarkEditorClass *klass)
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	gobject_class = (GObjectClass *)   klass;
-	widget_class  = (GtkWidgetClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	widget_class  = GTK_WIDGET_CLASS(klass);
 
 	gobject_class->dispose      = dispose;
 	gobject_class->set_property = set_property;
@@ -109,7 +110,7 @@ kz_bookmark_editor_class_init (KzBookmarkEditorClass *klass)
 	GtkAction *action;						\
 	action = gtk_action_group_get_action((editor)->action_group,	\
 					     (name));			\
-	g_object_set(action, "sensitive", (sensitive), NULL);			\
+	gtk_action_set_sensitive(action, (sensitive));			\
 }
 
 
@@ -125,7 +126,7 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	GError *err = NULL;
 	gchar *ui_file_name;
 
-	g_object_set(G_OBJECT(editor), "allow-shrink", TRUE, NULL);
+	g_object_set(editor, "allow-shrink", TRUE, NULL);
 
 	gtk_window_set_title(GTK_WINDOW(editor),
 			     _("- Kazehakase Bookmark Editor -"));
