@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-file.c 3198 2007-04-15 12:42:36Z kous $
+ *  $Id: kz-bookmark-file.c 3756 2009-02-22 11:44:12Z ikezoe $
  */
 
 #include "kz-bookmark-file.h"
@@ -29,9 +29,11 @@
 #include "glib-utils.h"
 #include "utils.h"
 
-#include "kz-io.h"
 #include "kz-xmlrpc.h"
+#include "kz-downloader.h"
+#include "kz-marshalers.h"
 
+#include "kz-bookmark-separator.h"
 /* bookmark_file file types */
 #include "kz-xbel.h"
 #include "kz-rss.h"
@@ -52,6 +54,7 @@ enum {
 
 enum {
 	PROP_0,
+	PROP_DOCUMENT_TITLE,
 	PROP_BOOKMARK_FILE_LOCATION,
 	PROP_FILE_TYPE,
 	PROP_INTERVAL,
@@ -62,45 +65,55 @@ enum {
 	PROP_PREVIOUS_LAST_MODIFIED
 };
 
+#define BUFFER_SIZE 4096
+
+typedef struct _KzBookmarkFilePrivate	KzBookmarkFilePrivate;
+struct _KzBookmarkFilePrivate
+{
+        KzDownloader *downloader;
+};
+#define KZ_BOOKMARK_FILE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_BOOKMARK_FILE, KzBookmarkFilePrivate))
 
 #define ADD_BOOKMARK_FILE_TYPE(func)				\
 {									\
 	KzBookmarkFileType *type;					\
 	gint i;								\
 	for (i = 0; (type = func(i)); i++)				\
-		file_types = g_list_append(file_types, type);		\
+	file_types = g_list_append(file_types, type);		\
 }
 
 static void dispose      (GObject             *object);
 static void set_property (GObject             *object,
-	                  guint                prop_id,
-	                  const GValue        *value,
-	                  GParamSpec          *pspec);
+		guint                prop_id,
+		const GValue        *value,
+		GParamSpec          *pspec);
 static void get_property (GObject             *object,
-                          guint                prop_id,
-                          GValue              *value,
-                          GParamSpec          *pspec);
+		guint                prop_id,
+		GValue              *value,
+		GParamSpec          *pspec);
+
+static gboolean update (KzBookmarkFile *bookmark);
+
+static void connect_downloader_signals 	  (KzBookmarkFile *bookmark_file,
+					   KzDownloader *downloader);
+static void disconnect_downloader_signals (KzBookmarkFile *bookmark_file,
+					   KzDownloader *downloader);
 
-static void kz_bookmark_file_set_state    (KzBookmarkFile      *bookmark_file,
-					   KzBookmarkFileState  state);
 static void kz_bookmark_file_set_editable (KzBookmarkFile      *bookmark_file,
-					   gboolean             editable);
+		gboolean             editable);
 
 static gboolean kz_bookmark_file_is_loading_all_children (KzBookmark *bookmark);
 
-static void     cb_io_load_complete       (KzIO *io,
-					   GError *error,
-					   KzBookmarkFile *bookmark_file);
-static gboolean idle_load_complete        (gpointer data);
 
 static KzBookmarkFileType *
-            kz_bookmark_file_detect_file_type  (KzBookmarkFile *bookmark_file,
-					        gchar          *buf);
+kz_bookmark_file_detect_file_type  (KzBookmarkFile *bookmark_file,
+				    const gchar    *buf);
 
 static gint kz_bookmark_file_signals[LAST_SIGNAL] = {0};
 
 static GList *file_types = NULL;
 
+static GQuark document_title_quark    = 0;
 static GQuark location_quark          = 0;
 static GQuark file_type_quark         = 0;
 static GQuark interval_quark          = 0;
@@ -109,13 +122,11 @@ static GQuark xmlrpc_user_quark       = 0;
 static GQuark xmlrpc_pass_quark       = 0;
 static GQuark timer_quark             = 0;
 static GQuark state_quark             = 0;
-static GQuark io_quark                = 0;
-static GQuark io_signal_id_quark      = 0;
 static GQuark p_last_modified_quark   = 0;
 
-G_DEFINE_TYPE(KzBookmarkFile, kz_bookmark_file, KZ_TYPE_BOOKMARK)
+G_DEFINE_TYPE(KzBookmarkFile, kz_bookmark_file, KZ_TYPE_BOOKMARK_FOLDER)
 
-static void
+	static void
 kz_bookmark_file_class_init (KzBookmarkFileClass *klass)
 {
 	GObjectClass *object_class;
@@ -130,7 +141,7 @@ kz_bookmark_file_class_init (KzBookmarkFileClass *klass)
 		ADD_BOOKMARK_FILE_TYPE(kz_hinadi_get_file_types);
 	}
 
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->dispose      = dispose;
 	object_class->set_property = set_property;
@@ -140,208 +151,207 @@ kz_bookmark_file_class_init (KzBookmarkFileClass *klass)
 	klass->load_completed     = NULL;
 	klass->save_start         = NULL;
 	klass->save_completed     = NULL;
-	klass->update		  = NULL;
+	klass->update		  = update;
 
 	g_object_class_install_property(
-		object_class,
-		 PROP_BOOKMARK_FILE_LOCATION,
-		 g_param_spec_string(
-			 "location",
-			 _("Location of bookmark_file file itself"),
-			 _("The location of the bookmark_file file itself"),
-			 NULL,
-			 G_PARAM_READWRITE));
+			object_class,
+			PROP_DOCUMENT_TITLE,
+			g_param_spec_string(
+				"document-title",
+				_("Original document title"),
+				_("The original document title of the link"),
+				NULL,
+				G_PARAM_READWRITE));
 
 	g_object_class_install_property(
-		object_class,
-		 PROP_FILE_TYPE,
-		 g_param_spec_string(
-			 "file-type",
-			 _("File type"),
-			 _("File type of the bookmark_file file"),
-			 NULL,
-			 G_PARAM_READWRITE));
+			object_class,
+			PROP_BOOKMARK_FILE_LOCATION,
+			g_param_spec_string(
+				"location",
+				_("Location of bookmark_file file itself"),
+				_("The location of the bookmark_file file itself"),
+				NULL,
+				G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_object_class_install_property(
-		object_class,
-		 PROP_INTERVAL,
-		 g_param_spec_uint(
-			 "interval",
-			 _("Interval"),
-			 _("Update interval"),
-			 0,
-			 G_MAXUINT,
-			 0,
-			 G_PARAM_READWRITE));
+			object_class,
+			PROP_FILE_TYPE,
+			g_param_spec_string(
+				"file-type",
+				_("File type"),
+				_("File type of the bookmark_file file"),
+				NULL,
+				G_PARAM_READWRITE));
 
 	g_object_class_install_property(
-		object_class,
-		 PROP_EDITABLE,
-		 g_param_spec_boolean(
-			 "editable",
-			 _("Editable"),
-			 _("Whether attributes of the bookmark_file is writable or not"),
-			 TRUE,
-			 G_PARAM_READWRITE));
-	
+			object_class,
+			PROP_INTERVAL,
+			g_param_spec_uint(
+				"interval",
+				_("Interval"),
+				_("Update interval"),
+				0,
+				G_MAXUINT,
+				0,
+				G_PARAM_READWRITE));
+
 	g_object_class_install_property(
-		object_class,
-		 PROP_XMLRPC,
-		 g_param_spec_string(
-			 "xmlrpc",
-			 _("XMLRPC"),
-			 _("The interface address of the XML-RPC for shared bookmark_file"),
-			 NULL,
-			 G_PARAM_READWRITE));
+			object_class,
+			PROP_EDITABLE,
+			g_param_spec_boolean(
+				"editable",
+				_("Editable"),
+				_("Whether attributes of the bookmark_file is writable or not"),
+				TRUE,
+				G_PARAM_READWRITE));
+
 	g_object_class_install_property(
-		object_class,
-		 PROP_XMLRPC_USER,
-		 g_param_spec_string(
-			 "xmlrpc-user",
-			 _("XMLRPC User"),
-			 _("The user name for XMLRPC"),
-			 NULL,
-			 G_PARAM_READWRITE));
+			object_class,
+			PROP_XMLRPC,
+			g_param_spec_string(
+				"xmlrpc",
+				_("XMLRPC"),
+				_("The interface address of the XML-RPC for shared bookmark_file"),
+				NULL,
+				G_PARAM_READWRITE));
 	g_object_class_install_property(
-		object_class,
-		 PROP_XMLRPC_PASS,
-		 g_param_spec_string(
-			 "xmlrpc-pass",
-			 _("XMLRPC Password"),
-			 _("The password for XMLRPC"),
-			 NULL,
-			 G_PARAM_READWRITE));
-	
+			object_class,
+			PROP_XMLRPC_USER,
+			g_param_spec_string(
+				"xmlrpc-user",
+				_("XMLRPC User"),
+				_("The user name for XMLRPC"),
+				NULL,
+				G_PARAM_READWRITE));
 	g_object_class_install_property(
-		object_class,
-		 PROP_PREVIOUS_LAST_MODIFIED,
-		 g_param_spec_uint(
-			 "previous-last-modified",
-			 _("Previous Last Modified"),
-			 _("The time of previous last modified"),
-			 0,
-			 G_MAXUINT,
-			 0,
-			 G_PARAM_READWRITE));
+			object_class,
+			PROP_XMLRPC_PASS,
+			g_param_spec_string(
+				"xmlrpc-pass",
+				_("XMLRPC Password"),
+				_("The password for XMLRPC"),
+				NULL,
+				G_PARAM_READWRITE));
+
+	g_object_class_install_property(
+			object_class,
+			PROP_PREVIOUS_LAST_MODIFIED,
+			g_param_spec_uint(
+				"previous-last-modified",
+				_("Previous Last Modified"),
+				_("The time of previous last modified"),
+				0,
+				G_MAXUINT,
+				0,
+				G_PARAM_READWRITE));
 
 	kz_bookmark_file_signals[LOAD_START_SIGNAL]
 		= g_signal_new("load_start",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkFileClass,
-						load_start),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (KzBookmarkFileClass,
+					load_start),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 
 	kz_bookmark_file_signals[LOAD_COMPLETED_SIGNAL]
 		= g_signal_new("load_completed",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkFileClass,
-						load_completed),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (KzBookmarkFileClass,
+					load_completed),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 
 	kz_bookmark_file_signals[SAVE_START_SIGNAL]
 		= g_signal_new("save_start",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkFileClass,
-						save_start),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (KzBookmarkFileClass,
+					save_start),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 
 	kz_bookmark_file_signals[SAVE_COMPLETED_SIGNAL]
 		= g_signal_new("save_completed",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkFileClass,
-						save_completed),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (KzBookmarkFileClass,
+					save_completed),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 
 	kz_bookmark_file_signals[ERROR_SIGNAL]
 		= g_signal_new("error",
-			       G_TYPE_FROM_CLASS(klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkFileClass,
-						error),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__STRING,
-			       G_TYPE_NONE, 1, G_TYPE_STRING);
+				G_TYPE_FROM_CLASS(klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (KzBookmarkFileClass,
+					error),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__STRING,
+				G_TYPE_NONE, 1, G_TYPE_STRING);
 
 	kz_bookmark_file_signals[UPDATE_SIGNAL]
 		= g_signal_new("update",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkFileClass,
-						update),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (KzBookmarkFileClass,
+					update),
+				NULL, NULL,
+				_kz_marshal_BOOLEAN__VOID,
+				G_TYPE_BOOLEAN, 0);
 
+	document_title_quark  = g_quark_from_string("KzBookmarkFile::DocumentTitle");
 	location_quark        = g_quark_from_string("KzBookmarkFile::BookmarkFileLocation");
 	file_type_quark       = g_quark_from_string("KzBookmarkFile::FileType");
 	interval_quark        = g_quark_from_string("KzBookmarkFile::Interval");
 	timer_quark           = g_quark_from_string("KzBookmarkFile::Timer");
 	state_quark           = g_quark_from_string("KzBookmarkFile::State");
-	io_quark              = g_quark_from_string("KzBookmarkFile::KzIO");
-	io_signal_id_quark    = g_quark_from_string("KzBookmarkFile::KzIOSignalID");
 	xmlrpc_quark          = g_quark_from_string("KzBookmarkFile::XMLRPC");
 	xmlrpc_user_quark     = g_quark_from_string("KzBookmarkFile::XMLRPCUser");
 	xmlrpc_pass_quark     = g_quark_from_string("KzBookmarkFile::XMLRPCPassword");
 	p_last_modified_quark = g_quark_from_string("KzBookmarkFile::PreviousLastModified");
+
+	g_type_class_add_private(object_class, sizeof(KzBookmarkFilePrivate));
 }
 
 
 static void
 kz_bookmark_file_init (KzBookmarkFile *bookmark_file)
 {
-}
+	KzBookmarkFilePrivate *priv = KZ_BOOKMARK_FILE_GET_PRIVATE(bookmark_file);
 
+	priv->downloader = NULL;
+}
 
 static void
 dispose (GObject *object)
 {
-	guint timer_id, io_id;
-	KzIO *io;
+	guint timer_id;
+	KzBookmarkFilePrivate *priv = KZ_BOOKMARK_FILE_GET_PRIVATE(object);
 	
+	kz_bookmark_file_set_state(KZ_BOOKMARK_FILE(object), KZ_BOOKMARK_FILE_STATE_DISPOSING);
+
 	timer_id = GPOINTER_TO_UINT(g_object_get_qdata(object, timer_quark));
 	if (timer_id)
 		g_source_remove(timer_id);
 	timer_id = 0;
 	g_object_set_qdata(object, timer_quark, GUINT_TO_POINTER(timer_id));
 
-	/* dispose KzIO object if exists */
-	/* FIXME! we need more robust way */
-	io = g_object_get_qdata(object, io_quark);
-	if (io)
+	if (priv->downloader)
 	{
-		kz_io_stop(io);
-	}
+		g_object_unref(priv->downloader);
+		priv->downloader = NULL;
+        }
 
-	io_id = GPOINTER_TO_UINT(g_object_get_qdata(object, io_signal_id_quark));
-	if (io_id)
-	{
-		/* wait until disconnecting (i.e. loading stops  )*/
-		while (g_signal_handler_is_connected(io, io_id))
-			gtk_main_iteration();
-	}
-	g_object_set_qdata(object, io_quark, NULL);
-
-	io_id = 0;
-	g_object_set_qdata(object, io_signal_id_quark,
-			   GUINT_TO_POINTER(io_id));
-
-	if (G_OBJECT_CLASS (kz_bookmark_file_parent_class)->dispose)
-		G_OBJECT_CLASS (kz_bookmark_file_parent_class)->dispose(object);
+	if (G_OBJECT_CLASS(kz_bookmark_file_parent_class)->dispose)
+		G_OBJECT_CLASS(kz_bookmark_file_parent_class)->dispose(object);
 }
 
-
 #define CHANGE_STR(obj, quark, value) \
 { \
 	g_object_set_qdata_full((obj), (quark), (value), \
@@ -357,12 +367,13 @@ set_property (GObject *object,
 	KzBookmarkFile *bookmark_file = KZ_BOOKMARK_FILE(object);
 
 	switch (prop_id) {
-	case PROP_BOOKMARK_FILE_LOCATION:
-	{
-		CHANGE_STR(object, location_quark,
+	case PROP_DOCUMENT_TITLE:
+		CHANGE_STR(object, document_title_quark,
 			   g_value_dup_string(value));
 		break;
-	}
+	case PROP_BOOKMARK_FILE_LOCATION:
+                CHANGE_STR(object, location_quark, g_value_dup_string(value));
+		break;
 	case PROP_FILE_TYPE:
 	{
 		gchar *str = g_value_dup_string(value);
@@ -441,7 +452,6 @@ set_property (GObject *object,
 	}
 }
 
-
 static void
 get_property (GObject *object,
               guint prop_id,
@@ -452,6 +462,10 @@ get_property (GObject *object,
 	gchar *str;
 
 	switch (prop_id) {
+	case PROP_DOCUMENT_TITLE:
+		str = g_object_get_qdata(object, document_title_quark);
+		g_value_set_string(value, str);
+		break;
 	case PROP_BOOKMARK_FILE_LOCATION:
 		str = g_object_get_qdata(object, location_quark);
 		g_value_set_string(value, str);
@@ -501,6 +515,13 @@ get_property (GObject *object,
 	}
 }
 
+static gboolean
+update (KzBookmarkFile *bookmark)
+{
+	kz_bookmark_file_save(bookmark);
+
+	return TRUE;
+}
 
 KzBookmarkFile *
 kz_bookmark_file_new (const gchar *location,
@@ -512,7 +533,6 @@ kz_bookmark_file_new (const gchar *location,
 	location = location ? location : "";
 
 	bookmark_file = g_object_new(KZ_TYPE_BOOKMARK_FILE,
-				     "type",                   KZ_BOOKMARK_FOLDER,
 				     "location",               location,
 				     "title",                  title,
 				     "file-type",              file_type,
@@ -539,7 +559,6 @@ kz_bookmark_file_create_new (const gchar *location,
 	}
 
 	bookmark_file = g_object_new(KZ_TYPE_BOOKMARK_FILE,
-				     "type",      KZ_BOOKMARK_FOLDER,
 				     "location",  location,
 				     "title",     title,
 				     "file-type", file_type,
@@ -557,6 +576,14 @@ kz_bookmark_file_create_new (const gchar *location,
 }
 
 
+const gchar *
+kz_bookmark_file_get_document_title (KzBookmarkFile *bookmark_file)
+{
+	g_return_val_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file), NULL);
+	return g_object_get_qdata(G_OBJECT(bookmark_file), document_title_quark);
+}
+
+
 const gchar *
 kz_bookmark_file_get_location (KzBookmarkFile *bookmark_file)
 {
@@ -628,12 +655,21 @@ kz_bookmark_file_is_editable (KzBookmarkFile *bookmark_file)
 }
 
 
+void
+kz_bookmark_file_set_document_title (KzBookmarkFile *bookmark_file,
+				     const gchar *document_title)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
+	g_object_set(bookmark_file, "document-title", document_title, NULL);
+}
+
+
 void
 kz_bookmark_file_set_location (KzBookmarkFile *bookmark_file,
 			       const gchar *location)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "location", location, NULL);
+	g_object_set(bookmark_file, "location", location, NULL);
 }
 
 
@@ -642,7 +678,7 @@ kz_bookmark_file_set_file_type (KzBookmarkFile *bookmark_file,
 			        const gchar *file_type)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "file-type", file_type, NULL);
+	g_object_set(bookmark_file, "file-type", file_type, NULL);
 }
 
 
@@ -650,7 +686,7 @@ void
 kz_bookmark_file_set_interval (KzBookmarkFile *bookmark_file, guint interval)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "interval", interval, NULL);
+	g_object_set(bookmark_file, "interval", interval, NULL);
 }
 
 void
@@ -658,7 +694,7 @@ kz_bookmark_file_set_xmlrpc (KzBookmarkFile *bookmark_file,
 		             const gchar *xmlrpc)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "xmlrpc", xmlrpc, NULL);
+	g_object_set(bookmark_file, "xmlrpc", xmlrpc, NULL);
 }
 
 void
@@ -666,7 +702,7 @@ kz_bookmark_file_set_xmlrpc_user (KzBookmarkFile *bookmark_file,
 		                  const gchar *user)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "xmlrpc-user", user, NULL);
+	g_object_set(bookmark_file, "xmlrpc-user", user, NULL);
 }
 
 void
@@ -674,7 +710,7 @@ kz_bookmark_file_set_xmlrpc_pass (KzBookmarkFile *bookmark_file,
 		                  const gchar *pass)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "xmlrpc-pass", pass, NULL);
+	g_object_set(bookmark_file, "xmlrpc-pass", pass, NULL);
 }
 
 
@@ -682,7 +718,7 @@ static void
 kz_bookmark_file_set_editable (KzBookmarkFile *bookmark_file, gboolean editable)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-	g_object_set(G_OBJECT(bookmark_file), "editable", editable, NULL);
+	g_object_set(bookmark_file, "editable", editable, NULL);
 }
 
 
@@ -691,13 +727,88 @@ kz_bookmark_file_set_editable (KzBookmarkFile *bookmark_file, gboolean editable)
  *                        load/save boomark file                            *
  *                                                                           *
  *****************************************************************************/
+
+static void
+parse_file_contents (KzBookmarkFile *bookmark_file, const gchar *contents, gsize length)
+{
+	KzBookmarkFileType *type;
+
+	type = kz_bookmark_file_detect_file_type(bookmark_file, contents);
+	if (!type)
+		/* use default? */;
+	if (type && type->from_string)
+	{
+		if (!kz_bookmark_file_get_file_type(bookmark_file))
+			kz_bookmark_file_set_file_type(bookmark_file, type->file_type);
+		type->from_string(bookmark_file, contents, length, NULL);
+	}
+
+}
+
+static void 
+cb_downloader_start (KzDownloader *downloader, gpointer data)
+{
+	g_signal_emit(data,
+		      kz_bookmark_file_signals[LOAD_START_SIGNAL],
+		      0);
+}
+
+static void 
+cb_downloader_completed (KzDownloader *downloader, gpointer data)
+{
+	KzBookmarkFile *bookmark_file = KZ_BOOKMARK_FILE(data);
+
+	parse_file_contents(bookmark_file,
+			    kz_downloader_get_buffer(downloader),
+			    kz_downloader_get_current_progress(downloader));
+	kz_bookmark_file_set_state(bookmark_file, KZ_BOOKMARK_FILE_STATE_NORMAL);
+	g_signal_emit(bookmark_file,
+		      kz_bookmark_file_signals[LOAD_COMPLETED_SIGNAL],
+		      0);
+	disconnect_downloader_signals(bookmark_file, downloader);
+}
+
+static void 
+cb_downloader_error (KzDownloader *downloader, const gchar *message, gpointer data)
+{
+	g_signal_emit(data,
+		      kz_bookmark_file_signals[ERROR_SIGNAL],
+		      0, message);
+	disconnect_downloader_signals(KZ_BOOKMARK_FILE(data), downloader);
+}
+
+static void
+connect_downloader_signals (KzBookmarkFile *bookmark_file, KzDownloader *downloader)
+{
+	g_signal_connect(downloader, "start",
+			 G_CALLBACK(cb_downloader_start), bookmark_file);
+	g_signal_connect(downloader, "completed",
+			 G_CALLBACK(cb_downloader_completed), bookmark_file);
+	g_signal_connect(downloader, "error",
+			 G_CALLBACK(cb_downloader_error), bookmark_file);
+}
+
+static void
+disconnect_downloader_signals (KzBookmarkFile *bookmark_file, KzDownloader *downloader)
+{
+	g_signal_handlers_disconnect_by_func(downloader,
+					     G_CALLBACK(cb_downloader_start),
+					     bookmark_file);
+	g_signal_handlers_disconnect_by_func(downloader,
+					     G_CALLBACK(cb_downloader_completed),
+					     bookmark_file);
+	g_signal_handlers_disconnect_by_func(downloader,
+					     G_CALLBACK(cb_downloader_error),
+					     bookmark_file);
+}
+
 gboolean
 kz_bookmark_file_load_start (KzBookmarkFile *bookmark_file)
 {
-	KzIO *io;
 	const gchar *uri;
-	guint id, last_mod;
+	guint last_mod;
 	KzBookmarkFileState state;
+	KzBookmarkFilePrivate *priv = KZ_BOOKMARK_FILE_GET_PRIVATE(bookmark_file);
 
 	g_return_val_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file), FALSE);
 
@@ -717,22 +828,13 @@ kz_bookmark_file_load_start (KzBookmarkFile *bookmark_file)
 	last_mod = kz_bookmark_get_last_modified(KZ_BOOKMARK(bookmark_file));
 	g_object_set(bookmark_file, "previous-last-modified", last_mod, NULL);
 
-	g_signal_emit(bookmark_file,
-		      kz_bookmark_file_signals[LOAD_START_SIGNAL],
-		      0);
-
-	kz_bookmark_remove_all(KZ_BOOKMARK(bookmark_file));
-
-	io = kz_io_new(uri);
-	id = g_signal_connect(io, "io_completed",
-			      G_CALLBACK(cb_io_load_complete),
-			      bookmark_file);
-
-	g_object_set_qdata(G_OBJECT(bookmark_file), io_quark, io);
-	g_object_set_qdata(G_OBJECT(bookmark_file), io_signal_id_quark,
-			   GUINT_TO_POINTER(id));
-
-	kz_io_load_to_buffer(io);
+	kz_bookmark_folder_remove_all(KZ_BOOKMARK_FOLDER(bookmark_file));
+        
+	if (priv->downloader)
+		g_object_unref(priv->downloader);
+	priv->downloader = kz_downloader_new(uri);
+	connect_downloader_signals(bookmark_file, priv->downloader);
+	kz_downloader_to_buffer(priv->downloader);
 
 	return TRUE;
 }
@@ -741,7 +843,9 @@ kz_bookmark_file_load_start (KzBookmarkFile *bookmark_file)
 void
 kz_bookmark_file_load_stop (KzBookmarkFile *bookmark_file)
 {
-	g_warning("kz_bookmark_file_load_stop() is not implemented yet.");
+	KzBookmarkFilePrivate *priv = KZ_BOOKMARK_FILE_GET_PRIVATE(bookmark_file);
+	if (priv->downloader)
+		kz_downloader_cancel(priv->downloader);
 }
 
 
@@ -756,7 +860,6 @@ kz_bookmark_file_save_start (KzBookmarkFile *bookmark_file)
 void
 kz_bookmark_file_save_stop (KzBookmarkFile *bookmark_file)
 {
-	g_warning("kz_bookmark_file_save_stop() is not implemented yet.");
 }
 
 void
@@ -769,23 +872,13 @@ kz_bookmark_file_load (KzBookmarkFile *bookmark_file)
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
 
 	file = kz_bookmark_file_get_location(bookmark_file);
+	if (!file || !*file)
+		return;
 
-	if (file && *file)
+	if (g_file_get_contents(file, &str, &length, NULL))
 	{
-		if (g_file_get_contents(file, &str, &length, NULL))
-		{
-			KzBookmarkFileType *type;
-			type = kz_bookmark_file_detect_file_type(bookmark_file, str);
-			if (!type)
-				/* use default? */;
-			if (type && type->from_string)
-			{
-				if (!kz_bookmark_file_get_file_type(bookmark_file))
-					kz_bookmark_file_set_file_type(bookmark_file, type->file_type);
-				type->from_string(bookmark_file, str, length, NULL);
-			}
-			g_free(str);
-		}
+		parse_file_contents(bookmark_file, str, length);
+		g_free(str);
 	}
 }
 
@@ -794,12 +887,14 @@ void
 kz_bookmark_file_save (KzBookmarkFile *bookmark_file)
 {
 	KzBookmarkFileType *type;
-	KzIO *io;
 	const gchar *file;
 	gchar *str;
 
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
 
+	if (!kz_bookmark_file_is_editable(bookmark_file))
+		return;
+
 	type = kz_bookmark_file_detect_file_type(bookmark_file, NULL);
 	if (!type || !type->to_string) return;
 
@@ -808,88 +903,21 @@ kz_bookmark_file_save (KzBookmarkFile *bookmark_file)
 
 	if (file && *file && str && *str)
 	{
-		io = kz_io_new(file);
-		kz_io_write(io, str);
-		g_object_unref(io);
-		g_free(str);
+		GError *error = NULL;
+
+		/* workaround */
+		g_file_set_contents(file, str, -1, &error);
+		if (error) 
+		{
+			g_warning("%s", error->message);
+			g_error_free(error);
+		}
 	}
+	g_free(str);
 }
 
-
-static gboolean
-idle_load_complete (gpointer data)
-{
-	KzIO *io = data;
-
-	g_object_unref(G_OBJECT(io));
-
-	return FALSE;
-}
-
-
-static void
-cb_io_load_complete (KzIO *io, GError *error, KzBookmarkFile *bookmark_file)
-{	
-	KzBookmarkFileType *type;
-	const gchar *buf;
-	guint size;
-
-	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
-
-	if (error) goto ERROR;
-
-	buf  = kz_io_get_buffer(io);
-	size = kz_io_get_loaded_size(io);
-
-	if (!buf || size == 0) goto ERROR;
-
-	type = kz_bookmark_file_detect_file_type(bookmark_file, (gchar *)buf);
-	if (!type)
-		/* use default? */;
-	if (type && type->from_string)
-	{
-		if (!kz_bookmark_file_get_file_type(bookmark_file))
-			kz_bookmark_file_set_file_type(bookmark_file, type->file_type);
-		type->from_string(bookmark_file, buf, size, NULL);
-	}
-
-ERROR:
-	g_signal_handlers_disconnect_by_func
-		(io,
-		 G_CALLBACK(cb_io_load_complete),
-		 bookmark_file);
-	kz_bookmark_file_set_state(bookmark_file, KZ_BOOKMARK_FILE_STATE_NORMAL);
-	g_object_set_qdata(G_OBJECT(bookmark_file), io_quark, NULL);
-	g_object_set_qdata(G_OBJECT(bookmark_file), io_signal_id_quark,
-			   GPOINTER_TO_UINT(0));
-
-	if (error)
-	{
-		gchar *message;
-		if (error->message)
-			message = g_strdup_printf("Load failed: %s",
-						  error->message);
-		else
-			message = g_strdup("Load failed");
-
-		g_signal_emit(bookmark_file,
-			      kz_bookmark_file_signals[ERROR_SIGNAL],
-			      0,
-			      message);
-		g_free(message);
-	}
-	else
-	{
-		g_signal_emit(bookmark_file,
-			      kz_bookmark_file_signals[LOAD_COMPLETED_SIGNAL],
-			      0);
-	}
-	g_idle_add(idle_load_complete, io);
-}
-
-
 static KzBookmarkFileType *
-kz_bookmark_file_detect_file_type (KzBookmarkFile *bookmark_file, gchar *buf)
+kz_bookmark_file_detect_file_type (KzBookmarkFile *bookmark_file, const gchar *buf)
 {
 	GList *node;
 	const gchar *type_str;
@@ -927,17 +955,13 @@ kz_bookmark_file_detect_file_type (KzBookmarkFile *bookmark_file, gchar *buf)
 }
 
 
-static void
+void
 kz_bookmark_file_set_state (KzBookmarkFile *bookmark_file, KzBookmarkFileState state)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK_FILE(bookmark_file));
 
-	if (state == KZ_BOOKMARK_FILE_STATE_LOADING)
-		g_object_set_qdata(G_OBJECT(bookmark_file), state_quark,
-				   GINT_TO_POINTER(state));
-	else
-		g_object_set_qdata(G_OBJECT(bookmark_file), state_quark,
-				   GINT_TO_POINTER(KZ_BOOKMARK_FILE_STATE_NORMAL));
+	g_object_set_qdata(G_OBJECT(bookmark_file), state_quark,
+			   GINT_TO_POINTER(state));
 }
 
 
@@ -959,13 +983,14 @@ kz_bookmark_file_is_loading_all_children (KzBookmark *bookmark)
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
 	
 	/* check loading state */
-	children = kz_bookmark_get_children(bookmark);
+	children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(bookmark));
 
 	node = children;
 	while (node)
 	{
 		KzBookmark *child = node->data;
-		if (kz_bookmark_has_children(child))
+		if (KZ_IS_BOOKMARK_FOLDER(child) &&
+                    kz_bookmark_folder_has_children(KZ_BOOKMARK_FOLDER(child)))
 		{
 			ret = kz_bookmark_file_is_loading_all_children(child);
 			if (ret) break;
@@ -1152,7 +1177,7 @@ kz_bookmark_file_is_update (KzBookmarkFile *bookmark_file)
 	last_time = kz_bookmark_file_get_previous_last_modified(bookmark_file);
 	time = kz_bookmark_get_last_modified(KZ_BOOKMARK(bookmark_file));
 
-	return last_time < time ? TRUE : FALSE;
+	return (last_time < time);
 }
 
 void
