@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-folder-menu-item.c 2638 2006-12-02 09:22:27Z ikezoe $
+ *  $Id: kz-bookmark-folder-menu-item.c 3675 2009-02-14 01:40:31Z ikezoe $
  */
 
 #include "kz-bookmark-folder-menu-item.h"
@@ -48,7 +48,7 @@ kz_bookmark_folder_menu_item_class_init (KzBookmarkFolderMenuItemClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class   = (GObjectClass *) klass;
+	gobject_class   = G_OBJECT_CLASS(klass);
 
 	/* GObject signals */
 	gobject_class->constructor  = constructor;
