@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-item.c 3321 2007-12-15 10:56:19Z kous $
+ *  $Id: kz-bookmark-item.c 3822 2009-09-04 00:28:36Z ikezoe $
  */
 
 #include "kz-bookmark-item.h"
@@ -132,9 +132,9 @@ kz_bookmark_item_class_init (KzBookmarkItemClass *klass)
 	GtkWidgetClass *widget_class;
 	GtkToolItemClass *tool_item_class;
 
-	gobject_class   = (GObjectClass *) klass;
-	widget_class    = (GtkWidgetClass *) klass;
-	tool_item_class = (GtkToolItemClass *) klass;
+	gobject_class   = G_OBJECT_CLASS(klass);
+	widget_class    = GTK_WIDGET_CLASS(klass);
+	tool_item_class = GTK_TOOL_ITEM_CLASS(klass);
 
 	/* GObject signals */
 	gobject_class->dispose      = dispose;
@@ -151,21 +151,21 @@ kz_bookmark_item_class_init (KzBookmarkItemClass *klass)
 	g_object_class_install_property
 		(gobject_class,
 		 PROP_BOOKMARK,
-		 g_param_spec_object ("bookmark",
-				      _("Bookmark"),
-				      _("The boomark to show"),
-				      KZ_TYPE_BOOKMARK,
-				      G_PARAM_READWRITE |
-				      G_PARAM_CONSTRUCT_ONLY));
+		 g_param_spec_object("bookmark",
+				     _("Bookmark"),
+				     _("The boomark to show"),
+				     KZ_TYPE_BOOKMARK,
+				     G_PARAM_READWRITE |
+				     G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property
 		(gobject_class,
 		 PROP_KZ_WINDOW,
-		 g_param_spec_object ("kz-window",
-				      _("KzWindow"),
-				      _("The KzWindow to add a home button"),
-				      KZ_TYPE_WINDOW,
-				      G_PARAM_READWRITE |
-				      G_PARAM_CONSTRUCT_ONLY));
+		 g_param_spec_object("kz-window",
+				     _("KzWindow"),
+				     _("The KzWindow to add a home button"),
+				     KZ_TYPE_WINDOW,
+				     G_PARAM_READWRITE |
+				     G_PARAM_CONSTRUCT_ONLY));
 }
 
 
@@ -260,10 +260,10 @@ constructor (GType                  type,
 	gtk_container_add(GTK_CONTAINER(button), image);
 	gtk_widget_show(image);
 	gtk_widget_show(button);
-	style = gtk_rc_style_new ();
+	style = gtk_rc_style_new();
 	style->xthickness = style->ythickness = 0;
-	gtk_widget_modify_style (button, style);
-	g_object_unref (style);
+	gtk_widget_modify_style(button, style);
+	g_object_unref(style);
 
 	if (is_file)
 	{
@@ -291,11 +291,11 @@ constructor (GType                  type,
 	gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
 	g_signal_connect(eventbox, "button_release_event",
 			 G_CALLBACK(cb_bookmark_menu_release), item);
-	gtk_widget_show (eventbox);
+	gtk_widget_show(eventbox);
 
 	item->label = gtk_label_new(kz_bookmark_get_title(item->bookmark));
 	gtk_container_add(GTK_CONTAINER(eventbox), item->label);
-	gtk_widget_show (item->label);
+	gtk_widget_show(item->label);
 
 	gtk_drag_source_set(GTK_WIDGET(item),
 			    GDK_BUTTON1_MASK |
@@ -320,8 +320,8 @@ dispose (GObject *object)
 {
 	KzBookmarkItem *menu;
 
-	if (G_OBJECT_CLASS (kz_bookmark_item_parent_class)->dispose)
-		G_OBJECT_CLASS (kz_bookmark_item_parent_class)->dispose(object);
+	if (G_OBJECT_CLASS(kz_bookmark_item_parent_class)->dispose)
+		G_OBJECT_CLASS(kz_bookmark_item_parent_class)->dispose(object);
 
 	menu = KZ_BOOKMARK_ITEM(object);
 
@@ -461,9 +461,7 @@ kz_bookmark_item_reset_submenu (KzBookmarkItem *menu)
 
 	submenu = kz_bookmark_menu_create_submenu(menu->bookmark, menu->kz);
 	if (menu->sub_menu)
-	{
 		gtk_widget_destroy(menu->sub_menu);
-	}
 
 	menu->sub_menu = submenu;
 }
@@ -661,7 +659,8 @@ cb_drag_data_received (GtkWidget        *widget,
 		       gpointer          data)
 {
 	KzBookmarkItem *item;
-	KzBookmark *bookmark = NULL, *parent, *file;
+	KzBookmark *bookmark = NULL, *file;
+        KzBookmarkFolder *parent;
 	GtkWidget *src_widget;
 	gboolean success = FALSE, moved = FALSE;
 
@@ -670,14 +669,14 @@ cb_drag_data_received (GtkWidget        *widget,
 	{
 	 case TARGET_GTK_NOTEBOOK_TAB:
 	 {
-	 	KzEmbed *kzembed;
+	 	KzWeb *kzweb;
 		const gchar *uri, *title;
 		src_widget = gtk_drag_get_source_widget(context);
 		if (!KZ_IS_NOTEBOOK(src_widget))
 			break;
-		kzembed = KZ_EMBED(kz_notebook_get_dragged_page(KZ_NOTEBOOK(src_widget)));
-		uri   = kz_embed_get_location(kzembed);
-		title = kz_embed_get_title(kzembed);
+		kzweb = KZ_WEB(kz_notebook_get_dragged_page(KZ_NOTEBOOK(src_widget)));
+		uri   = kz_web_get_location(kzweb);
+		title = kz_web_get_title(kzweb);
 		bookmark = KZ_BOOKMARK(kz_bookmark_new_with_attrs(title, uri, NULL));
 	 	break;
 	 }
@@ -690,7 +689,7 @@ cb_drag_data_received (GtkWidget        *widget,
 		bookmark = KZ_BOOKMARK_ITEM(src_widget)->bookmark;
 		g_object_ref(bookmark);
 		parent = kz_bookmark_get_parent(bookmark);
-		kz_bookmark_remove(parent, bookmark);
+		kz_bookmark_folder_remove(parent, bookmark);
 		moved = TRUE;
 	 	break;
 	 case TARGET_NETSCAPE_URL:
@@ -703,7 +702,7 @@ cb_drag_data_received (GtkWidget        *widget,
 
 	if (bookmark)
 	{
-		kz_bookmark_append(item->bookmark, bookmark);
+		kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(item->bookmark), bookmark);
 		if (KZ_IS_BOOKMARK_FILE(item->bookmark))
 			file = item->bookmark;
 		else
@@ -716,10 +715,6 @@ cb_drag_data_received (GtkWidget        *widget,
 					NULL,
 					bookmark);
 		}
-		else
-		{
-			kz_bookmark_file_save(KZ_BOOKMARK_FILE(file));
-		}
 		g_object_unref(bookmark);
 		success = TRUE;
 	}
