@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu-item-list.h 2253 2005-07-20 03:47:43Z ikezoe $
+ *  $Id: kz-bookmark-menu-item-list.h 3769 2009-03-16 22:57:11Z ikezoe $
  */
 
 #ifndef __KZ_BOOKMARK_MENU_ITEM_LIST_H__
@@ -45,7 +45,7 @@ struct _KzBookmarkMenuItemList
 
 	GtkMenuShell *menu_shell;
 
-	KzBookmark *folder;
+	KzBookmarkFolder *folder;
 	KzWindow   *kz;
 
 	guint initial_children;
