@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu.c 2296 2005-08-17 10:38:05Z ikezoe $
+ *  $Id: kz-bookmark-menu.c 3627 2009-02-12 03:06:11Z ikezoe $
  */
 
 #include "kz-bookmark-menu.h"
@@ -27,6 +27,7 @@
 
 #include "kazehakase.h"
 #include "utils.h"
+#include "kz-bookmark-separator.h"
 #include "kz-bookmark-menu-item-list.h"
 #include "kz-bookmark-menu-item.h"
 #include "kz-bookmark-folder-menu-item.h"
@@ -75,7 +76,7 @@ kz_bookmark_menu_append_menuitems (GtkMenuShell *shell, KzWindow *kz,
 	kz_bookmark_menu_item_list_new(shell, kz, folder);
 
 	/* children */
-	children = kz_bookmark_get_children(folder);
+	children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(folder));
 	for (node = children; node; node = g_list_next(node))
 	{
 		KzBookmark *child = node->data;
