@@ -7,11 +7,6 @@
 #include <glib-object.h>
 
 G_BEGIN_DECLS
-/* enumerations from "kz-bookmark.h" */
-GType kz_bookmark_type_get_type (void);
-#define KZ_TYPE_BOOKMARK_TYPE (kz_bookmark_type_get_type())
-GType kz_bookmark_flag_get_type (void);
-#define KZ_TYPE_BOOKMARK_FLAG (kz_bookmark_flag_get_type())
 /* enumerations from "kz-bookmark-file.h" */
 GType kz_bookmark_file_flag_get_type (void);
 #define KZ_TYPE_BOOKMARK_FILE_FLAG (kz_bookmark_file_flag_get_type())
