@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-view.c 2714 2006-12-31 01:59:41Z ikezoe $
+ *  $Id: kz-bookmarks-view.c 3699 2009-02-15 03:52:37Z ikezoe $
  */
 
 #include "kz-bookmarks-view.h"
@@ -26,6 +26,7 @@
 
 #include "kazehakase.h"
 #include "kz-bookmark-file.h"
+#include "kz-bookmark-separator.h"
 #include "kz-smart-bookmark.h"
 #include "kz-favicon.h"
 #include "kz-icons.h"
@@ -61,10 +62,10 @@ struct _KzBookmarksViewPrivate
 
 static void     dispose    (GObject              *object);
 
-static void     connect_bookmark_signals     (KzBookmarksView *view,
-					      KzBookmark *bookmark);
-static void     disconnect_bookmark_signals  (KzBookmarksView *view,
-					      KzBookmark *bookmark);
+static void     connect_bookmark_signals     (KzBookmark *bookmark,
+					      KzBookmarksView *view);
+static void     disconnect_bookmark_signals  (KzBookmark *bookmark,
+					      KzBookmarksView *view);
 
 static GtkTreePath *find_row                 (GtkTreeModel  *model,
 					      KzBookmark    *bookmark);
@@ -119,7 +120,7 @@ kz_bookmarks_view_class_init (KzBookmarksViewClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *)   klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->dispose      = dispose;
 
@@ -147,7 +148,7 @@ dispose (GObject *object)
 
 	if (view->root_folder)
 	{
-		disconnect_bookmark_signals(view, view->root_folder);
+		disconnect_bookmark_signals(KZ_BOOKMARK(view->root_folder), view);
 		g_object_unref(view->root_folder);
 		view->root_folder = NULL;
 	}
@@ -261,7 +262,7 @@ kz_bookmarks_view_set_root_folder (KzBookmarksView *view,
 
 	if (view->root_folder)
 	{
-		disconnect_bookmark_signals(view, view->root_folder);
+		disconnect_bookmark_signals(KZ_BOOKMARK(view->root_folder), view);
 		g_object_unref(view->root_folder);
 		view->root_folder = NULL;
 	}
@@ -275,7 +276,7 @@ kz_bookmarks_view_set_root_folder (KzBookmarksView *view,
 
 	view->root_folder = top_folder;
 	g_object_ref(view->root_folder);
-	connect_bookmark_signals(view, view->root_folder);
+	connect_bookmark_signals(KZ_BOOKMARK(view->root_folder), view);
 
 	if (include_top)
 	{
@@ -286,7 +287,7 @@ kz_bookmarks_view_set_root_folder (KzBookmarksView *view,
 	{
 		GList *children, *node;
 
-		children = kz_bookmark_get_children(top_folder);
+		children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(top_folder));
 		for (node = children; node; node = g_list_next(node))
 		{
 			insert_bookmark(view, folder_only,
@@ -391,7 +392,7 @@ find_folder_sibling (KzBookmark *parent, KzBookmark *sibling)
 
 	if (!sibling) return NULL;
 
-	children = kz_bookmark_get_children(parent);
+	children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(parent));
 
 	node = g_list_find(children, sibling);
 	g_return_val_if_fail(node, NULL);
@@ -432,7 +433,7 @@ needs_refresh (KzBookmarksView *view, KzBookmark *bookmark)
 
 	for (parent = bookmark;
 	     parent;
-	     parent = kz_bookmark_get_parent(parent))
+	     parent = KZ_BOOKMARK(kz_bookmark_get_parent(parent)))
 	{
 		if (parent == view->root_folder)
 			return TRUE;
@@ -505,12 +506,12 @@ insert_bookmark(KzBookmarksView *view,
 			? KZ_STOCK_REMOTE_BOOKMARK : KZ_STOCK_FOLDER;
 		image = gtk_image_new_from_stock(icon,
 						 GTK_ICON_SIZE_MENU);
-		g_object_ref(G_OBJECT(image));
-		g_object_ref_sink(G_OBJECT(image));
+		g_object_ref(image);
+		g_object_ref_sink(image);
 		favicon = gtk_widget_render_icon(image, icon,
 						 GTK_ICON_SIZE_MENU,
 						 NULL);
-		g_object_unref(G_OBJECT(image));
+		g_object_unref(image);
 	}
 	else if (KZ_IS_SMART_BOOKMARK(bookmark))
 	{
@@ -518,8 +519,8 @@ insert_bookmark(KzBookmarksView *view,
 
 		image = gtk_image_new_from_stock(KZ_STOCK_SMART_BOOKMARK,
 						 GTK_ICON_SIZE_MENU);
-		g_object_ref(G_OBJECT(image));
-		g_object_ref_sink(G_OBJECT(image));
+		g_object_ref(image);
+		g_object_ref_sink(image);
 		favicon = gtk_widget_render_icon(image, KZ_STOCK_SMART_BOOKMARK,
 						 GTK_ICON_SIZE_MENU,
 						 NULL);
@@ -541,8 +542,8 @@ insert_bookmark(KzBookmarksView *view,
 			GtkWidget *image;
 			image = gtk_image_new_from_stock(KZ_STOCK_BOOKMARK,
 							 GTK_ICON_SIZE_MENU);
-			g_object_ref(G_OBJECT(image));
-			g_object_ref_sink(G_OBJECT(image));
+			g_object_ref(image);
+			g_object_ref_sink(image);
 			favicon = gtk_widget_render_icon(image, KZ_STOCK_BOOKMARK,
 							 GTK_ICON_SIZE_MENU,
 							 NULL);
@@ -562,8 +563,8 @@ insert_bookmark(KzBookmarksView *view,
 			title_is_editable    = FALSE;
 			location_is_editable = FALSE;
 		}
-		else if (!KZ_IS_BOOKMARK_FILE(bookmark) &&
-			 kz_bookmark_is_pure_folder(bookmark))
+		else if (KZ_IS_BOOKMARK_FOLDER(bookmark) &&
+		         !KZ_IS_BOOKMARK_FILE(bookmark))
 		{
 			location_is_editable = FALSE;
 		}
@@ -595,7 +596,7 @@ insert_bookmark(KzBookmarksView *view,
 	{
 		GList *children, *node;
 
-		children = kz_bookmark_get_children(bookmark);
+		children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(bookmark));
 		for (node = children; node; node = g_list_next(node))
 		{
 			if (!KZ_IS_BOOKMARK(node->data))
@@ -610,7 +611,8 @@ insert_bookmark(KzBookmarksView *view,
 	}
 
 	/* expand */
-	if (!kz_bookmark_get_folded(bookmark))
+	if (KZ_IS_BOOKMARK_FOLDER(bookmark) &&
+	    !kz_bookmark_folder_get_folded(KZ_BOOKMARK_FOLDER(bookmark)))
 	{
 		path = gtk_tree_model_get_path(model, &iter);
 		gtk_tree_view_expand_row(GTK_TREE_VIEW(view), path, FALSE);
@@ -656,7 +658,7 @@ find_next_current_folder (KzBookmark *bookmark)
 	     next;
 	     next = kz_bookmark_next(next))
 	{
-		if (kz_bookmark_is_folder(next))
+		if (KZ_IS_BOOKMARK_FOLDER(next))
 			return next;
 	}
 
@@ -664,11 +666,11 @@ find_next_current_folder (KzBookmark *bookmark)
 	     next;
 	     next = kz_bookmark_prev(next))
 	{
-		if (kz_bookmark_is_folder(next))
+		if (KZ_IS_BOOKMARK_FOLDER(next))
 			return next;
 	}
 
-	return kz_bookmark_get_parent(bookmark);
+	return KZ_BOOKMARK(kz_bookmark_get_parent(bookmark));
 }
 
 
@@ -795,7 +797,7 @@ cb_bookmark_insert_child (KzBookmark *bookmark,
 		return;
 
 	priv = KZ_BOOKMARKS_VIEW_GET_PRIVATE (view);
-	connect_bookmark_signals (view, child);
+	connect_bookmark_signals (KZ_BOOKMARK(child), view);
 
 	if (!priv->folder_only || kz_bookmark_is_folder(child))
 		insert_bookmark(view, priv->folder_only,
@@ -807,7 +809,7 @@ static void
 cb_bookmark_remove_child (KzBookmark *bookmark, KzBookmark *child,
 			  KzBookmarksView *view)
 {
-	disconnect_bookmark_signals (view, child);
+	disconnect_bookmark_signals(KZ_BOOKMARK(child), view);
 
 	/* set selection */
 	ensure_cursor(view, child);
@@ -830,53 +832,48 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec,
 
 
 static void
-connect_bookmark_signals (KzBookmarksView *view,
-			  KzBookmark *bookmark)
+connect_bookmark_signals (KzBookmark *bookmark,
+			  KzBookmarksView *view)
 {
-	GList *children, *node;
-
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 
-	g_signal_connect_after(bookmark, "insert-child",
-			       G_CALLBACK(cb_bookmark_insert_child), view);
-	g_signal_connect(bookmark, "remove-child",
-			 G_CALLBACK(cb_bookmark_remove_child), view);
 	g_signal_connect(bookmark, "notify",
 			 G_CALLBACK(cb_bookmark_notify), view);
 
 	if (!kz_bookmark_is_folder(bookmark)) return;
 
-	children = kz_bookmark_get_children(bookmark);
-	for (node = children; node; node = g_list_next(node))
-		connect_bookmark_signals (view, node->data);
-	g_list_free(children);
+	g_signal_connect_after(bookmark, "insert-child",
+			       G_CALLBACK(cb_bookmark_insert_child), view);
+	g_signal_connect(bookmark, "remove-child",
+			 G_CALLBACK(cb_bookmark_remove_child), view);
+
+	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bookmark),
+					 (GFunc)connect_bookmark_signals,
+					 view);
 }
 
 
 static void
-disconnect_bookmark_signals (KzBookmarksView *view,
-			     KzBookmark *bookmark)
+disconnect_bookmark_signals (KzBookmark *bookmark,
+			     KzBookmarksView *view)
 {
-	GList *children, *node;
-
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 
+	g_signal_handlers_disconnect_by_func
+		(bookmark,
+		 G_CALLBACK(cb_bookmark_notify), view);
+
+	if (!kz_bookmark_is_folder(bookmark)) return;
+
 	g_signal_handlers_disconnect_by_func
 		(bookmark,
 		 G_CALLBACK(cb_bookmark_insert_child), view);
 	g_signal_handlers_disconnect_by_func
 		(bookmark,
 		 G_CALLBACK(cb_bookmark_remove_child), view);
-	g_signal_handlers_disconnect_by_func
-		(bookmark,
-		 G_CALLBACK(cb_bookmark_notify), view);
-
-	if (!kz_bookmark_is_folder(bookmark)) return;
-
-	children = kz_bookmark_get_children(bookmark);
-	for (node = children; node; node = g_list_next(node))
-		disconnect_bookmark_signals (view, node->data);
-	g_list_free(children);
+	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bookmark),
+					 (GFunc)disconnect_bookmark_signals,
+					 view);
 }
 
 
@@ -1036,7 +1033,8 @@ cb_drag_data_received (GtkWidget *widget,
 	GtkTreeIter src_iter, dest_iter;
 	GtkTreeViewDropPosition pos;
 	GtkWidget *src_widget;
-	KzBookmark *src, *dest, *parent;
+	KzBookmark *src, *dest;
+        KzBookmarkFolder *parent;
 	KzBookmark *file;
 	gboolean success = FALSE;
 
@@ -1080,12 +1078,12 @@ cb_drag_data_received (GtkWidget *widget,
 		parent = kz_bookmark_get_parent(src);
 		if (!parent) goto ERROR;
 		g_object_ref(src);
-		kz_bookmark_remove(parent, src);
+		kz_bookmark_folder_remove(parent, src);
 		
 		if (KZ_IS_BOOKMARK_FILE(parent))
-			file = parent;
+			file = KZ_BOOKMARK(parent);
 		else
-			file = kz_bookmark_get_parent_file(parent);
+			file = kz_bookmark_get_parent_file(KZ_BOOKMARK(parent));
 	
 		if (kz_bookmark_file_has_xmlrpc(KZ_BOOKMARK_FILE(file)))
 		{
@@ -1098,7 +1096,7 @@ cb_drag_data_received (GtkWidget *widget,
 		    kz_bookmark_is_folder(dest))
 		{
 			/* move the src bookmark into the dest folder */
-			parent = dest;
+			parent = KZ_BOOKMARK_FOLDER(dest);
 			dest = NULL;
 		}
 		else
@@ -1111,7 +1109,7 @@ cb_drag_data_received (GtkWidget *widget,
 			{
 				GList *children, *node;
 
-				children = kz_bookmark_get_children(parent);
+				children = kz_bookmark_folder_get_children(parent);
 				node = g_list_find(children, dest);
 				node = g_list_next(node);
 				if (node)
@@ -1124,17 +1122,17 @@ cb_drag_data_received (GtkWidget *widget,
 
 		if (!parent) goto ERROR;
 
-		kz_bookmark_insert_before(parent, src, dest);
+		kz_bookmark_folder_insert_before(parent, src, dest);
 		
 		if (KZ_IS_BOOKMARK_FILE(parent))
-			file = parent;
+			file = KZ_BOOKMARK(parent);
 		else
-			file = kz_bookmark_get_parent_file(parent);
+			file = kz_bookmark_get_parent_file(KZ_BOOKMARK(parent));
 	
 		if (kz_bookmark_file_has_xmlrpc(KZ_BOOKMARK_FILE(file)))
 		{
 			kz_bookmark_file_xmlrpc_insert(KZ_BOOKMARK_FILE(file),
-						       parent,
+						       KZ_BOOKMARK(parent),
 						       dest,
 						       src);
 		}
