@@ -23,6 +23,7 @@
 #include <time.h>
 
 #include "kz-lirs.h"
+#include "kz-bookmark-folder.h"
 
 static gboolean kz_lirs_is_supported      (KzBookmarkFile  *bookmark,
 					   const gchar     *buf);
@@ -181,8 +182,8 @@ kz_lirs_parse_from_string (KzBookmarkFile *bookmark,
 			kz_bookmark_set_link(KZ_BOOKMARK(bookmark), elements[8]);
 		
 		g_strfreev(elements);
-		kz_bookmark_append(KZ_BOOKMARK(bookmark), item);
-		g_object_unref(G_OBJECT(item));
+		kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), item);
+		g_object_unref(item);
 
 		line_number++;
 	}
@@ -190,8 +191,8 @@ kz_lirs_parse_from_string (KzBookmarkFile *bookmark,
 	g_strfreev(lines);
 
 	/* sorting */
-	kz_bookmark_sort(KZ_BOOKMARK(bookmark),
-			 KZ_BOOKMARK_SORT_LAST_MODIFIED);
+	kz_bookmark_folder_sort(KZ_BOOKMARK_FOLDER(bookmark),
+			        KZ_BOOKMARK_FOLDER_SORT_LAST_MODIFIED);
 
 	return TRUE;
 }
