@@ -21,6 +21,7 @@
  */
 
 #include "kz-nsbookmark.h"
+#include "kz-bookmark-separator.h"
 
 #include <string.h>
 #include <utils.h>
@@ -72,12 +73,12 @@ static void
 kz_nsbookmark_init (KzBookmark *bookmark)
 {
 	bookmark->flags |= KZ_BOOKMARK_FILE_FLAG;
-	g_object_set(G_OBJECT(bookmark),
+	g_object_set(bookmark,
 		     "type", KZ_BOOKMARK_PURE_FOLDER,
 		     NULL);
-	g_signal_connect(G_OBJECT(bookmark), "notify",
+	g_signal_connect(bookmark, "notify",
 			 G_CALLBACK(kz_nsbookmark_notify), NULL);
-	g_signal_connect(G_OBJECT(bookmark), "insert-child",
+	g_signal_connect(bookmark, "insert-child",
 			 G_CALLBACK(kz_nsbookmark_insert_child), NULL);
 }
 */
@@ -112,19 +113,17 @@ kz_nsbookmark_parse_bookmark (KzBookmark *bookmark,
 		if (!strncasecmp(pos, "<H3", 3))
 		{
 			gchar *folder_title;
-			KzBookmark *folder = kz_bookmark_pure_folder_new();
+			KzBookmark *folder;
 
 			/* folder title */
 			folder_title = xml_get_content(pos);
+			folder = kz_bookmark_folder_new(folder_title);
 			if (folder_title)
-			{
-				kz_bookmark_set_title(folder, folder_title);
 				g_free(folder_title);
-			}
 			pos += 3;
 			pos = kz_nsbookmark_parse_bookmark(folder, pos);
 
-			kz_bookmark_append(bookmark, folder);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), folder);
 			g_object_unref(folder);
 		}
 		else if (!strncasecmp(pos, "<A HREF=", 8))
@@ -144,7 +143,7 @@ kz_nsbookmark_parse_bookmark (KzBookmark *bookmark,
 				g_free(title);
 			}
 
-			kz_bookmark_append(bookmark, child);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), child);
 			g_object_unref(child);
 		}
 		else if (!strncasecmp(pos, "<DL>", 4))
@@ -156,7 +155,7 @@ kz_nsbookmark_parse_bookmark (KzBookmark *bookmark,
 			/* separator */
 			KzBookmark *separator = kz_bookmark_separator_new();
 
-			kz_bookmark_append(bookmark, separator);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), separator);
 			g_object_unref(separator);
 		}
 		else if (!strncasecmp(pos, "</DL>", 4))
