@@ -18,13 +18,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-proxy-folder.h 1837 2004-12-08 13:37:26Z ikezoe $
  */
 
 #ifndef __KZ_PROXY_FOLDER_H__
 #define __KZ_PROXY_FOLDER_H__
 
-#include "kz-bookmark.h"
+#include "kz-bookmark-folder.h"
 
 G_BEGIN_DECLS
 
@@ -42,16 +41,16 @@ typedef struct _KzProxyFolderClass    KzProxyFolderClass;
 
 struct _KzProxyFolder
 {
-	KzBookmark parent;
+	KzBookmarkFolder parent;
 };
 
 struct _KzProxyFolderClass
 {
-	KzBookmarkClass parent_class;
+	KzBookmarkFolderClass parent_class;
 };
 
 GType	     kz_proxy_folder_get_type		(void) G_GNUC_CONST;
-KzBookmark  *kz_proxy_folder_new		(KzBookmark *folder);
+KzBookmark  *kz_proxy_folder_new		(KzBookmarkFolder *folder);
 
 KzBookmark  *kz_proxy_folder_get_original_bookmark (KzProxyFolder *proxy,
 						    KzBookmark    *bookmark);
