@@ -31,6 +31,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
+#include "kz-bookmark-file.h"
 
 static gboolean kz_rss_is_supported      (KzBookmarkFile *bookmark,
 					  const gchar    *buffer);
@@ -178,8 +179,8 @@ parse_item_node (KzXMLNode *parent,
 		}
 	}
 
-	kz_bookmark_append(bookmark, child);
-	g_object_unref(G_OBJECT(child));
+	kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), child);
+	g_object_unref(child);
 }
 
 
@@ -202,7 +203,7 @@ parse_channel_node (KzXMLNode *parent,
 		if (kz_xml_node_name_is(node, "title"))
 		{
 			gchar *title = kz_xml_node_to_str(node);
-			kz_bookmark_set_document_title(bookmark, title);
+			kz_bookmark_file_set_document_title(KZ_BOOKMARK_FILE(bookmark), title);
 			g_free(title);
 		}
 		else if (kz_xml_node_name_is(node, "link"))
