@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-session.c 3133 2007-04-04 03:03:55Z makeinu $
+ *  $Id: kz-session.c 3705 2009-02-15 23:23:55Z ikezoe $
  */
 
 #include "kz-session.h"
@@ -36,7 +36,7 @@ struct _KzSessionPrivate
 
 #define KZ_SESSION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_SESSION, KzSessionPrivate))
 
-static void kz_bookmark_file_update  (KzBookmarkFile *bookmark);
+static gboolean kz_bookmark_file_update  (KzBookmarkFile *bookmark);
 
 static void cb_profile_changed (KzProfile *profile,
 				const gchar *section, const gchar *key,
@@ -51,8 +51,8 @@ kz_session_class_init (KzSessionClass *klass)
 	KzBookmarkFileClass *bookmark_file_class;
 	GObjectClass *object_class;
 
-	object_class = (GObjectClass *) klass;
-	bookmark_file_class = (KzBookmarkFileClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
+	bookmark_file_class = KZ_BOOKMARK_FILE_CLASS(klass);
 
 	bookmark_file_class->update  = kz_bookmark_file_update;
 
@@ -140,27 +140,24 @@ cb_profile_changed (KzProfile *profile,
 		    const gchar *old_value,
 		    KzSession *session)
 {
-	KzSessionPrivate *priv = KZ_SESSION_GET_PRIVATE(session);
-
-	if (key && strcmp(key, "save"))
+	if (key && !strcmp(key, "save"))
 	{
-		gboolean auto_save;
+		KzSessionPrivate *priv = KZ_SESSION_GET_PRIVATE(session);
+
 		kz_profile_get_value(profile, "Session", "save",
-				     &auto_save, sizeof(gboolean),
+				     &priv->auto_save, sizeof(gboolean),
 				     KZ_PROFILE_VALUE_TYPE_BOOL);
-		priv->auto_save = auto_save;
 	}
 }
 
-static void
+static gboolean
 kz_bookmark_file_update (KzBookmarkFile *bookmark)
 {
 	KzSessionPrivate *priv = KZ_SESSION_GET_PRIVATE(bookmark);
 
-	if (priv->frozen)
-		return;
+	if (priv->frozen || !priv->auto_save)
+		return TRUE;
 
-	if (priv->auto_save)
-		kz_bookmark_file_save(bookmark);
+        return KZ_BOOKMARK_FILE_CLASS(kz_session_parent_class)->update(bookmark);
 }
 
