@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xbel.c 3267 2007-10-04 05:17:39Z ikezoe $
+ *  $Id: kz-xbel.c 3841 2009-09-11 03:37:19Z ikezoe $
  */
 
 #include "kazehakase.h"
@@ -25,6 +25,8 @@
 #include "kz-xml.h"
 #include "utils.h"
 #include "kz-smart-bookmark.h"
+#include "kz-bookmark-separator.h"
+#include "kz-bookmark-folder.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -51,11 +53,13 @@ static void     kz_xbel_connect_signals    (KzBookmark  *bookmark,
 					    gpointer     data);
 static void     kz_xbel_disconnect_signals (KzBookmark  *bookmark,
 					    gpointer     data);
-static void     cb_bookmark_insert_child   (KzBookmark  *bookmark,
+static void     cb_bookmark_folder_insert_child
+					   (KzBookmarkFolder *folder,
 					    KzBookmark  *child,
 					    KzBookmark  *sibling,
 					    gpointer     data);
-static void     cb_bookmark_remove_child   (KzBookmark  *bookmark,
+static void     cb_bookmark_folder_remove_child
+					   (KzBookmarkFolder *folder,
 					    KzBookmark  *child,
 					    gpointer     data);
 static void     cb_bookmark_notify         (GObject     *object,
@@ -142,16 +146,12 @@ kz_xbel_init (KzBookmarkFile *bookmark)
 	g_object_set_qdata_full(G_OBJECT(bookmark), xml_quark,
 				xml, (GDestroyNotify) g_object_unref);
 
-	g_object_set(G_OBJECT(bookmark),
-		     "type", KZ_BOOKMARK_PURE_FOLDER,
-		     NULL);
-
 	g_signal_connect(bookmark, "notify",
 			 G_CALLBACK(kz_xbel_notify), xml);
 	g_signal_connect_after(bookmark, "insert-child",
-			       G_CALLBACK(cb_bookmark_insert_child), bookmark);
+			       G_CALLBACK(cb_bookmark_folder_insert_child), bookmark);
 	g_signal_connect_after(bookmark, "remove-child",
-			       G_CALLBACK(cb_bookmark_remove_child), bookmark);
+			       G_CALLBACK(cb_bookmark_folder_remove_child), bookmark);
 }
 
 
@@ -234,14 +234,16 @@ kz_xbel_notify (GObject *object, GParamSpec *pspec, KzXML *xml)
 	KzBookmark *bookmark;
 	KzXMLNode *node;
 	const gchar *prop;
-        GValue value = { 0 };
+	GValue value = { 0 };
+	gboolean updated;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(object));
 	g_return_if_fail(KZ_IS_XML(xml));
 
 	bookmark = KZ_BOOKMARK(object);
 
-	if (BOOKMARK_IS_BUILDING(bookmark)) return;
+	if (BOOKMARK_IS_BUILDING(bookmark))
+		return;
 
 	node = kz_xml_get_root_element(xml);
 	if (!node) return;
@@ -266,7 +268,7 @@ kz_xbel_notify (GObject *object, GParamSpec *pspec, KzXML *xml)
 	g_value_unset(&value);
 
 	/* emit update signal */
-	g_signal_emit_by_name(object, "update", 0);
+	g_signal_emit_by_name(object, "update", &updated);
 }
 
 
@@ -387,24 +389,24 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 			else if (!strcmp(attr->name, "kz:current_position"))
 			{
 				guint pos = (guint)atoi(attr->value);
-				kz_bookmark_set_current(bookmark, pos);
+				kz_bookmark_folder_set_current_position(KZ_BOOKMARK_FOLDER(bookmark), pos);
 			}
 			else if (!strcmp(attr->name, "kz:lock"))
 			{
 				if (!strcmp(attr->value, "yes"))
-					kz_bookmark_set_lock(bookmark, TRUE);
+					kz_bookmark_folder_set_lock(KZ_BOOKMARK_FOLDER(bookmark), TRUE);
 			}
 			else if (!strcmp(attr->name, "kz:auto_refresh"))
 			{
 				if (!strcmp(attr->value, "yes"))
-					kz_bookmark_set_auto_refresh(bookmark, TRUE);
+					kz_bookmark_folder_set_auto_refresh(KZ_BOOKMARK_FOLDER(bookmark), TRUE);
 			}
 			else if (!strcmp(attr->name, "kz:javascript"))
 			{
 				if (!strcmp(attr->value, "yes"))
-					kz_bookmark_set_javascript(bookmark, TRUE);
+					kz_bookmark_folder_set_javascript(KZ_BOOKMARK_FOLDER(bookmark), TRUE);
 				if (!strcmp(attr->value, "no"))
-					kz_bookmark_set_javascript(bookmark, FALSE);
+					kz_bookmark_folder_set_javascript(KZ_BOOKMARK_FOLDER(bookmark), FALSE);
 			}
 		}
 
@@ -593,12 +595,12 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			}
 			else
 			{
-				folder = kz_bookmark_pure_folder_new();
+				folder = kz_bookmark_folder_new(NULL);
 			}
 
 			folded_str = kz_xml_node_get_attr(node, "folded");
 			folded = folded_str && !strcmp(folded_str, "yes");
-			kz_bookmark_set_folded(folder, folded);
+			kz_bookmark_folder_set_folded(KZ_BOOKMARK_FOLDER(folder), folded);
 
 			id = kz_xml_node_get_attr(node, "id");
 			if (id)
@@ -607,7 +609,7 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			BOOKMARK_SET_BUILDING(folder);
 			g_object_set_qdata(G_OBJECT(folder),
 					   node_quark, node);
-			kz_bookmark_append(bookmark, folder);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), folder);
 			parse_child_node(folder, node);
 			BOOKMARK_UNSET_BUILDING(folder);
 
@@ -662,7 +664,7 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			 * because smart bookmark action needs bookmark title 
 			 * in child's nodes.
 			 */
-			kz_bookmark_append(bookmark, child_bookmark);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), child_bookmark);
 			BOOKMARK_UNSET_BUILDING(bookmark);
 			g_object_unref(child_bookmark);
 		}
@@ -673,7 +675,7 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			BOOKMARK_SET_BUILDING(separator);
 			g_object_set_qdata(G_OBJECT(separator),
 					   node_quark, node);
-			kz_bookmark_append(bookmark, separator);
+			kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(bookmark), separator);
 			BOOKMARK_UNSET_BUILDING(separator);
 			g_object_unref(separator);
 		}
@@ -1037,10 +1039,11 @@ create_xml_node (KzBookmark *bookmark)
 	else if (kz_bookmark_is_folder(bookmark))
 	{
 		KzXMLNode *metanode;
-		gint pos = kz_bookmark_get_current(bookmark);
-		gboolean lock = kz_bookmark_get_lock(bookmark);
-		gboolean auto_refresh = kz_bookmark_get_auto_refresh(bookmark);
-		gboolean javascript = kz_bookmark_get_javascript(bookmark);
+                KzBookmarkFolder *folder = KZ_BOOKMARK_FOLDER(bookmark);
+		gint pos = kz_bookmark_folder_get_current_position(folder);
+		gboolean lock = kz_bookmark_folder_get_lock(folder);
+		gboolean auto_refresh = kz_bookmark_folder_get_auto_refresh(folder);
+		gboolean javascript = kz_bookmark_folder_get_javascript(folder);
 		/*
 		 * <folder>\n
 		 * <title/>
@@ -1177,24 +1180,15 @@ kz_xbel_connect_signals (KzBookmark *bookmark, gpointer data)
 	if (!kz_bookmark_is_folder(bookmark)) return;
 
 	g_signal_connect_after(bookmark, "insert-child",
-			       G_CALLBACK(cb_bookmark_insert_child),
+			       G_CALLBACK(cb_bookmark_folder_insert_child),
 			       data);
 	g_signal_connect_after(bookmark, "remove-child",
-			       G_CALLBACK(cb_bookmark_remove_child),
+			       G_CALLBACK(cb_bookmark_folder_remove_child),
 			       data);
 
 	/* for children */
-	{
-		GList *node, *children;
-
-		children = kz_bookmark_get_children(bookmark);
-		for (node = children; node; node = g_list_next(node))
-		{
-			KzBookmark *child = node->data;
-			kz_xbel_connect_signals(child, data);
-		}
-		g_list_free(children);
-	}
+	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bookmark),
+					 (GFunc)kz_xbel_connect_signals, data);
 }
 
 
@@ -1210,45 +1204,46 @@ kz_xbel_disconnect_signals (KzBookmark *bookmark, gpointer data)
 
 	g_signal_handlers_disconnect_by_func
 		(bookmark,
-		 G_CALLBACK(cb_bookmark_insert_child), data);
+		 G_CALLBACK(cb_bookmark_folder_insert_child), data);
 	g_signal_handlers_disconnect_by_func
 		(bookmark,
-		 G_CALLBACK(cb_bookmark_remove_child), data);
+		 G_CALLBACK(cb_bookmark_folder_remove_child), data);
 
 	/* for children */
-	{
-		GList *node, *children;
-
-		children = kz_bookmark_get_children(bookmark);
-		for (node = children; node; node = g_list_next(node))
-		{
-			KzBookmark *child = node->data;
-			kz_xbel_disconnect_signals(child, data);
-		}
-		g_list_free(children);
-	}
+	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bookmark),
+					 (GFunc)kz_xbel_disconnect_signals, data);
 }
 
 
 static void
-cb_bookmark_insert_child (KzBookmark *bookmark,
-			  KzBookmark *child, KzBookmark *sibling,
-			  gpointer data)
+cb_bookmark_folder_insert_child (KzBookmarkFolder *folder,
+				 KzBookmark *child, KzBookmark *sibling,
+				 gpointer data)
 {
-	kz_xbel_insert_xml_node(child, bookmark, sibling);
+	gboolean updated;
+
+	kz_xbel_insert_xml_node(child, KZ_BOOKMARK(folder), sibling);
 	kz_xbel_connect_signals(child, data);
 
-	g_signal_emit_by_name(G_OBJECT(data), "update", 0);
+	if (kz_bookmark_file_get_state(KZ_BOOKMARK_FILE(data)) !=
+	    KZ_BOOKMARK_FILE_STATE_NORMAL)
+		return;
+	g_signal_emit_by_name(data, "update", &updated);
 }
 
 
 static void
-cb_bookmark_remove_child (KzBookmark *bookmark, KzBookmark *child, gpointer data)
+cb_bookmark_folder_remove_child (KzBookmarkFolder *folder, KzBookmark *child, gpointer data)
 {
+	gboolean updated;
+
 	kz_xbel_disconnect_signals(child, data);
 	kz_xbel_remove_xml_node(child);
 
-	g_signal_emit_by_name(G_OBJECT(data), "update", 0);
+	if (kz_bookmark_file_get_state(KZ_BOOKMARK_FILE(data)) !=
+	    KZ_BOOKMARK_FILE_STATE_NORMAL)
+		return;
+	g_signal_emit_by_name(data, "update", &updated);
 }
 
 
@@ -1345,12 +1340,14 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec, gpointer data)
 	KzBookmark *bookmark;
 	KzXMLNode *node;
 	const gchar *prop;
-        GValue value = { 0 };
+	GValue value = { 0 };
+	gboolean updated;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(object));
 	bookmark = KZ_BOOKMARK(object);
 
-	if (BOOKMARK_IS_BUILDING(bookmark)) return;
+	if (BOOKMARK_IS_BUILDING(bookmark))
+		return;
 
 	node = g_object_get_qdata(G_OBJECT(bookmark), node_quark);
 	g_return_if_fail(node);
@@ -1464,7 +1461,7 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec, gpointer data)
 		kz_xml_node_set_attr(metanode, "kz:xmlrpc_user", xmlrpc);
 		g_free(xmlrpc);
 	}
-	else if (!strcmp(prop, "current"))
+	else if (!strcmp(prop, "current-position"))
 	{
 		KzXMLNode *metanode;
 		guint pos = g_value_get_uint(&value);
@@ -1482,11 +1479,8 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec, gpointer data)
 		KzXMLNode *metanode;
 		const gchar *str;
 		gboolean lock = g_value_get_boolean(&value);
-		
-		if (lock)
-			str = "yes";
-		else
-			str = "no";
+
+		str = lock ? "yes" : "no";
 
 		metanode = xml_node_find_metadata_node(node);
 	
@@ -1501,10 +1495,7 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec, gpointer data)
 		const gchar *str;
 		gboolean auto_refresh = g_value_get_boolean(&value);
 		
-		if (auto_refresh)
-			str = "yes";
-		else
-			str = "no";
+		str = auto_refresh ? "yes" : "no";
 
 		metanode = xml_node_find_metadata_node(node);
 	
@@ -1513,16 +1504,13 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec, gpointer data)
 
 		kz_xml_node_set_attr(metanode, "kz:auto_refresh", str);
 	}
-	else if (!strcmp(prop, "javascript"))
+	else if (!strcmp(prop, "enable-javascript"))
 	{
 		KzXMLNode *metanode;
 		const gchar *str;
 		gboolean javascript = g_value_get_boolean(&value);
 		
-		if (javascript)
-			str = "yes";
-		else
-			str = "no";
+		str = javascript ? "yes" : "no";
 
 		metanode = xml_node_find_metadata_node(node);
 	
@@ -1532,7 +1520,7 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec, gpointer data)
 		kz_xml_node_set_attr(metanode, "kz:javascript", str);
 	}
 	g_value_unset(&value);
-	g_signal_emit_by_name(G_OBJECT(data), "update", 0);
+	g_signal_emit_by_name(data, "update", &updated);
 }
 
 
@@ -1553,6 +1541,11 @@ kz_xbel_save (KzXBEL *xbel)
 }
 #endif
 
+static void
+insert_bookmark (KzBookmark *bookmark, KzBookmark *parent)
+{
+	kz_xbel_insert_xml_node (bookmark, parent, NULL);
+}
 
 static void
 kz_xbel_insert_xml_node (KzBookmark *bookmark,
@@ -1601,15 +1594,8 @@ kz_xbel_insert_xml_node (KzBookmark *bookmark,
 
 	if (kz_bookmark_is_folder(bookmark) && !KZ_IS_BOOKMARK_FILE(bookmark))
 	{
-		GList *node, *children;
-
-		children = kz_bookmark_get_children(bookmark);
-		for (node = children; node; node = g_list_next(node))
-		{
-			KzBookmark *child = node->data;
-			kz_xbel_insert_xml_node (child, bookmark, NULL);
-		}
-		g_list_free(children);
+		kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bookmark),
+						 (GFunc)insert_bookmark, bookmark);
 	}
 }
 
@@ -1623,15 +1609,8 @@ kz_xbel_remove_xml_node (KzBookmark *bookmark)
 
 	if (kz_bookmark_is_folder(bookmark))
 	{
-		GList *node, *children;
-
-		children = kz_bookmark_get_children(bookmark);
-		for (node = children; node; node = g_list_next(node))
-		{
-			KzBookmark *child = node->data;
-			kz_xbel_remove_xml_node (child);
-		}
-		g_list_free(children);
+		kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(bookmark),
+						 (GFunc)kz_xbel_remove_xml_node, NULL);	
 	}
 
 	node = g_object_get_qdata(G_OBJECT(bookmark), node_quark);
