@@ -21,11 +21,11 @@
 #ifndef __KZ_ABOUT_DIALOG_H__
 #define __KZ_ABOUT_DIALOG_H__
 
-#include "kz-window.h"
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-void kz_about_dialog_new (KzWindow *kz);
+void kz_about_dialog_new (GtkWindow *parent_window);
 
 G_END_DECLS
 
