@@ -32,7 +32,7 @@
 enum {
 	PROP_0,
 	PROP_PARENT_WINDOW,
-	PROP_KZ_EMBED,
+	PROP_KZ_WEB,
 	PROP_SELECTED_ONLY
 };
 
@@ -87,8 +87,8 @@ kz_links_dialog_class_init (KzLinksDialogClass *klass)
 	GObjectClass *gobject_class;
 	GtkDialogClass *dialog_class;
 
-	gobject_class = (GObjectClass *) klass;
-	dialog_class  = (GtkDialogClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	dialog_class  = GTK_DIALOG_CLASS(klass);
 
 	gobject_class->constructor  = constructor;
 	gobject_class->dispose      = dispose;
@@ -109,11 +109,11 @@ kz_links_dialog_class_init (KzLinksDialogClass *klass)
 				     G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property
 		(gobject_class,
-		 PROP_KZ_EMBED,
-		 g_param_spec_object("kz-embed",
-				     _("KzEmbed"),
-				     _("The embed widget to extract links"),
-				     KZ_TYPE_EMBED,
+		 PROP_KZ_WEB,
+		 g_param_spec_object("kz-web",
+				     _("KzWeb"),
+				     _("The web widget to extract links"),
+				     KZ_TYPE_WEB,
 				     G_PARAM_READWRITE |
 				     G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property
@@ -138,7 +138,7 @@ kz_links_dialog_init (KzLinksDialog *kzlinks)
 	GtkTreeSelection *selection;
 
 	kzlinks->parent_window = NULL;
-	kzlinks->kzembed       = NULL;
+	kzlinks->kzweb       = NULL;
 	kzlinks->selected_only = FALSE;
 
 	main_vbox = GTK_DIALOG(kzlinks)->vbox;
@@ -231,16 +231,16 @@ constructor (GType                  type,
 
 	g_signal_connect(kzlinks->parent_window, "destroy",
 			 G_CALLBACK(cb_parent_destroy), kzlinks);
-	g_signal_connect(kzlinks->kzembed, "destroy",
+	g_signal_connect(kzlinks->kzweb, "destroy",
 			 G_CALLBACK(cb_parent_destroy), kzlinks);
 
 	g_snprintf(buf, sizeof(buf), _("Extracted links  - %s"),
-		   kz_embed_get_title(kzlinks->kzembed));
+		   kz_web_get_title(kzlinks->kzweb));
 	gtk_window_set_title(GTK_WINDOW(kzlinks), buf);
 	gtk_window_set_transient_for(GTK_WINDOW(kzlinks),
 	    			     GTK_WINDOW(kzlinks->parent_window));
 
-	result = kz_embed_get_links(kzlinks->kzembed, &list,
+	result = kz_web_get_links(kzlinks->kzweb, &list,
 				    kzlinks->selected_only);
 	if (!result || !list) return object;
 
@@ -285,14 +285,14 @@ dispose (GObject *object)
 		kzlinks->parent_window = NULL;
 	}
 
-	if (kzlinks->kzembed)
+	if (kzlinks->kzweb)
 	{
 		g_signal_handlers_disconnect_by_func
-			(kzlinks->kzembed,
+			(kzlinks->kzweb,
 			 G_CALLBACK(cb_parent_destroy),
 			 kzlinks);
-		g_object_unref(kzlinks->kzembed);
-		kzlinks->kzembed = NULL;
+		g_object_unref(kzlinks->kzweb);
+		kzlinks->kzweb = NULL;
 	}
 
 	if (kzlinks->list_store)
@@ -319,8 +319,8 @@ set_property (GObject         *object,
 	case PROP_PARENT_WINDOW:
 		kzlinks->parent_window = g_object_ref(g_value_get_object(value));
 		break;
-	case PROP_KZ_EMBED:
-		kzlinks->kzembed = g_object_ref(g_value_get_object(value));
+	case PROP_KZ_WEB:
+		kzlinks->kzweb = g_object_ref(g_value_get_object(value));
 		break;
 	case PROP_SELECTED_ONLY:
 		kzlinks->selected_only = g_value_get_boolean(value);
@@ -345,8 +345,8 @@ get_property (GObject         *object,
 	case PROP_PARENT_WINDOW:
 		g_value_set_object(value, kzlinks->parent_window);
 		break;
-	case PROP_KZ_EMBED:
-		g_value_set_object(value, kzlinks->kzembed);
+	case PROP_KZ_WEB:
+		g_value_set_object(value, kzlinks->kzweb);
 		break;
 	case PROP_SELECTED_ONLY:
 		g_value_set_boolean(value, kzlinks->selected_only);
@@ -359,13 +359,13 @@ get_property (GObject         *object,
 
 
 GtkWidget *
-kz_links_dialog_new (GtkWindow *parent_window, KzEmbed *kzembed, gboolean selected_only)
+kz_links_dialog_new (GtkWindow *parent_window, KzWeb *kzweb, gboolean selected_only)
 {
 	GtkWidget *widget;
 
 	widget = GTK_WIDGET(g_object_new(KZ_TYPE_LINKS_DIALOG,
 					 "parent-window", parent_window,
-					 "kz-embed",      kzembed,
+					 "kz-web",      kzweb,
 					 "selected-only", selected_only,
 					 NULL));
 
@@ -495,7 +495,7 @@ response (GtkDialog *dialog, gint arg)
 	g_return_if_fail (KZ_IS_LINKS_DIALOG(dialog));
 
 	kzlinks = KZ_LINKS_DIALOG(dialog);
-	if (!kzlinks->parent_window || !kzlinks->kzembed) return;
+	if (!kzlinks->parent_window || !kzlinks->kzweb) return;
 
 	switch (arg) {
 	case GTK_RESPONSE_YES:
@@ -540,7 +540,7 @@ tree_sel_open_selected (GtkTreeModel *model,
 	if (url && *url)
 	{
 		kz_window_open_new_tab_with_parent(KZ_WINDOW(kzlinks->parent_window),
-						   url, GTK_WIDGET(kzlinks->kzembed));
+						   url, GTK_WIDGET(kzlinks->kzweb));
 	}
 
 	g_free(url);
