@@ -23,7 +23,7 @@
 
 #include <gtk/gtk.h>
 
-#include "kz-embed.h"
+#include "kz-web.h"
 
 G_BEGIN_DECLS
 
@@ -43,7 +43,7 @@ struct _KzLinksDialog
 	GtkDialog parent;
 
 	GtkWindow    *parent_window;
-	KzEmbed      *kzembed;
+	KzWeb      *kzweb;
 
 	GtkWidget    *scrolled_window;
 	GtkTreeView  *tree_view;
@@ -59,7 +59,7 @@ struct _KzLinksDialogClass
 
 GType            kz_links_dialog_get_type          (void) G_GNUC_CONST;
 GtkWidget       *kz_links_dialog_new               (GtkWindow  *parent_window,
-						    KzEmbed    *kzembed,
+						    KzWeb    *kzweb,
 						    gboolean    selected_only);
 
 #endif /* __KZ_LINKS_DIALOG_H__ */
