@@ -67,8 +67,8 @@ kz_password_manager_dialog_class_init (KzPasswordManagerDialogClass *klass)
 	GObjectClass *gobject_class;
 	GtkDialogClass *dialog_class;
 
-	gobject_class = (GObjectClass *) klass;
-	dialog_class  = (GtkDialogClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	dialog_class  = GTK_DIALOG_CLASS(klass);
 
 	gobject_class->constructor  = constructor;
 	gobject_class->dispose      = dispose;
@@ -182,7 +182,7 @@ constructor (GType                  type,
 	GObjectClass *klass = G_OBJECT_CLASS(kz_password_manager_dialog_parent_class);
 	GList *list = NULL, *node;
 	gboolean result;
-	KzEmbedPrefs *embed_prefs = KZ_EMBED_PREFS(kz_embed_new("gecko"));
+	KzEmbedPrefs *embed_prefs;
 
 	object = klass->constructor(type, n_props, props);
 
@@ -194,6 +194,10 @@ constructor (GType                  type,
 	    			     GTK_WINDOW(password_manager->parent_window));
 
 #warning FIXME! KzProfile should handle password.
+	embed_prefs = KZ_EMBED_PREFS(kz_embed_new("gecko", "gecko"));
+        if (!embed_prefs)
+                return object;
+
 	result = kz_embed_prefs_get_passwords(embed_prefs, &list);
 	gtk_widget_destroy(GTK_WIDGET(embed_prefs));
 
@@ -320,10 +324,14 @@ cb_remove_button_clicked (GtkWidget *widget,
 	GtkTreeSelection *selection;
 	GtkTreeModel *model = NULL;
 	GList *list, *node, *pass = NULL;
-	KzEmbedPrefs *embed_prefs = KZ_EMBED_PREFS(kz_embed_new("gecko"));
+	KzEmbedPrefs *embed_prefs;
 
 	g_return_if_fail(KZ_IS_PASSWORD_MANAGER_DIALOG(password_manager));
 
+	embed_prefs = KZ_EMBED_PREFS(kz_embed_new("gecko", "gecko"));
+        if (!embed_prefs)
+                return;
+
 	selection = gtk_tree_view_get_selection(password_manager->tree_view);
 	
 	list = gtk_tree_selection_get_selected_rows(selection,
