@@ -263,7 +263,7 @@ constructor (GType                  type,
 	     GObjectConstructParam *props)
 {
 	GtkWidget *widget;
-        KzEmbed *embed;
+	KzWeb *web;
 	KzPrintDialog *print;
 	GObject *object;
 	GObjectClass *klass = G_OBJECT_CLASS(kz_print_dialog_parent_class);
@@ -278,13 +278,13 @@ constructor (GType                  type,
 	gtk_window_set_title(GTK_WINDOW(print), _("Print"));
 	gtk_window_set_transient_for(GTK_WINDOW(print), GTK_WINDOW(print->parent_window));
 
-	embed = KZ_WINDOW_CURRENT_EMBED(KZ_WINDOW(print->parent_window));
+	web = KZ_WINDOW_CURRENT_WEB(KZ_WINDOW(print->parent_window));
 
-	if (embed)
+	if (web)
 	{
 		GList *printer_list, *node;
 
-		printer_list = kz_embed_get_printer_list(embed);
+		printer_list = kz_web_get_printer_list(web);
 
 		for (node = g_list_first(printer_list); node; node = g_list_next(node))
 		{
