@@ -23,7 +23,6 @@
 
 #include <gtk/gtk.h>
 
-#include "kz-window.h"
 #include "kz-embed.h"
 
 G_BEGIN_DECLS
@@ -60,7 +59,7 @@ struct _KzPrintDialog
 {
 	GtkDialog parent;
 
-	KzWindow     *kz;
+	GtkWindow    *parent_window;
 	PrintInfo    *info;
 };
 
@@ -70,7 +69,7 @@ struct _KzPrintDialogClass
 };
 
 GType            kz_print_dialog_get_type (void) G_GNUC_CONST;
-GtkWidget       *kz_print_dialog_new      (KzWindow *kz);
+GtkWidget       *kz_print_dialog_new      (GtkWindow *parent_window);
 
 PrintInfo	*kz_print_dialog_get_print_info (KzPrintDialog *print);
 
