@@ -122,7 +122,7 @@ kz_prompt_dialog_class_init (KzPromptDialogClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->constructor  = constructor;
 	gobject_class->dispose      = dispose;
