@@ -122,6 +122,7 @@ static GObject *constructor  (GType type,
                               guint n_props,
                               GObjectConstructParam *props);
 static void     dispose      (GObject *object);
+static void     finalize     (GObject *object);
 static void     set_property (GObject *object,
                               guint prop_id,
                               const GValue *value,
@@ -154,11 +155,11 @@ kz_app_class_init (KzAppClass *klass)
 {
 	GObjectClass *object_class;
 
-	kz_app_parent_class = g_type_class_peek_parent(klass);
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->constructor  = constructor;
 	object_class->dispose	   = dispose;
+	object_class->finalize	   = finalize;
 	object_class->set_property = set_property;
 	object_class->get_property = get_property;
 
@@ -172,7 +173,7 @@ kz_app_class_init (KzAppClass *klass)
 			       NULL, NULL,
 			       _kz_marshal_VOID__STRING_ENUM,
 			       G_TYPE_NONE, 2,
-			       G_TYPE_STRING, EGG_TYPE_PIXBUF_THUMB_SIZE);
+			       G_TYPE_STRING, EGG_TYPE_PIXBUF_THUMBNAIL_SIZE);
 
 	g_object_class_install_property(
 		object_class,
@@ -215,7 +216,7 @@ constructor (GType                  type,
 	}
 	else
 	{
-		object = g_object_ref(G_OBJECT(the_kz_app));
+		object = g_object_ref(the_kz_app);
 	}
 	return object;
 }
@@ -318,7 +319,7 @@ kz_app_init_path (KzApp *app)
 						   "etc", PACKAGE, NULL);
 
 	priv->system_embed_module_dir = g_build_filename(priv->system_lib_dir,
-							 "embed", NULL);
+							 "web", NULL);
 	priv->system_search_module_dir = g_build_filename(priv->system_lib_dir,
 							  "search", NULL);
 
@@ -326,7 +327,7 @@ kz_app_init_path (KzApp *app)
 	priv->system_ruby_ext_dir = g_build_filename(priv->system_ext_dir,
 						     "ruby", NULL);
 #else
-	priv->system_data_dir = g_strdup(DATADIR);
+	priv->system_data_dir = g_strdup(SYSDATADIR);
 	priv->system_kz_data_dir = g_strdup(KZ_DATADIR);
 	priv->system_config_dir = g_strdup(KZ_SYSCONFDIR);
 	priv->system_lib_dir = g_strdup(KZ_LIBDIR);
@@ -378,31 +379,6 @@ kz_app_dispose_path (GObject *app)
 	g_free(priv->system_ext_dir);
 	g_free(priv->system_ruby_ext_dir);
 	g_free(priv->system_ruby_ext_data_dir);
-
-	priv->user_dir = NULL;
-	priv->history_dir = NULL;
-	priv->history_time_stamp = NULL;
-	priv->popup_dir = NULL;
-	priv->favicon_dir = NULL;
-	priv->smartbookmark_history_dir = NULL;
-	priv->form_data_dir = NULL;
-	priv->clips_dir = NULL;
-	priv->thumbnails_dir = NULL;
-
-	priv->system_data_dir = NULL;
-	priv->system_kz_data_dir = NULL;
-	priv->system_lib_dir = NULL;
-	priv->system_config_dir = NULL;
-
-	priv->system_icons_dir = NULL;
-	priv->system_pixmaps_dir = NULL;
-
-	priv->system_embed_module_dir = NULL;
-	priv->system_search_module_dir = NULL;
-
-	priv->system_ext_dir = NULL;
-	priv->system_ruby_ext_dir = NULL;
-	priv->system_ruby_ext_data_dir = NULL;
 }
 
 #define DEFINE_PATH_GETTER(name)			\
@@ -565,6 +541,40 @@ clean_history_cache (void)
 
 static void
 dispose (GObject *object)
+{
+	KzAppPrivate *priv = KZ_APP_GET_PRIVATE(object);
+
+	if (priv->bookmarks)
+	{
+		kz_root_bookmark_save_all(priv->bookmarks);
+		g_object_unref(priv->bookmarks);
+		priv->bookmarks = NULL;
+	}
+
+	if (priv->profile)
+	{
+		kz_profile_close(priv->profile);
+		priv->profile   = NULL;
+	}
+
+	if (priv->proxy)
+	{
+		kz_profile_close(priv->proxy);
+		priv->proxy     = NULL;
+	}
+
+	if (priv->favicon)
+	{
+		g_object_unref(priv->favicon);
+		priv->favicon   = NULL;
+	}
+
+	if (G_OBJECT_CLASS (kz_app_parent_class)->dispose)
+		G_OBJECT_CLASS (kz_app_parent_class)->dispose(object);
+}
+
+static void
+finalize (GObject *object)
 {
 	gchar *accel_prefs_file;
 	KzAppPrivate *priv = KZ_APP_GET_PRIVATE(object);
@@ -577,28 +587,13 @@ dispose (GObject *object)
 	gtk_accel_map_save(accel_prefs_file);
 	g_free(accel_prefs_file);
 
-	kz_root_bookmark_save_all(priv->bookmarks);
-	g_object_unref(G_OBJECT(priv->bookmarks));
-
-	kz_profile_close(priv->profile);
-	kz_profile_close(priv->proxy);
-
-	if (priv->favicon)
-		g_object_unref(priv->favicon);
-
-	if (priv->argv)
-		g_strfreev(priv->argv);
-
 #ifdef USE_SSL
 	gnutls_global_deinit();
 #endif
 
 #ifdef USE_MIGEMO
 	if (priv->migemo)
-	{
 		kz_migemo_free(priv->migemo);
-		priv->migemo = NULL;
-	}
 #endif
 
 #ifdef HAVE_LIBSM
@@ -606,16 +601,11 @@ dispose (GObject *object)
 		SmcCloseConnection ((SmcConn) priv->smc_conn, 0, NULL);
 #endif /* HAVE_LIBSM */
 
-	priv->profile   = NULL;
-	priv->proxy     = NULL;
-	priv->bookmarks = NULL;
-	priv->favicon   = NULL;
-	priv->argv	= NULL;
-
+	g_strfreev(priv->argv);
 	kz_app_dispose_path(object);
 
-	if (G_OBJECT_CLASS (kz_app_parent_class)->dispose)
-		G_OBJECT_CLASS (kz_app_parent_class)->dispose(object);
+	if (G_OBJECT_CLASS (kz_app_parent_class)->finalize)
+		G_OBJECT_CLASS (kz_app_parent_class)->finalize(object);
 }
 
 KzApp *
@@ -674,14 +664,6 @@ kz_app_quit_dependencies (KzApp *app)
 	kz_search_exit();
 }
 
-static void
-cb_window_map (KzWindow *kz, GtkAction *action)
-{
-	gtk_action_activate(action);
-	g_signal_handlers_disconnect_by_func
-		(kz, G_CALLBACK(cb_window_map), action);
-}
-
 void
 kz_app_save_session (KzApp *app)
 {
@@ -711,8 +693,8 @@ kz_app_restore_session (KzApp *app)
 	for (node = priv->window_list; node; node = g_list_next(node))
 	{
 		KzWindow *kz = KZ_WINDOW(node->data);
-		kz_bookmark_remove(priv->bookmarks->current_session,
-				   kz->tabs);
+		kz_bookmark_folder_remove(KZ_BOOKMARK_FOLDER(priv->bookmarks->current_session),
+				  	  KZ_BOOKMARK(kz->tabs));
 		kz_window_close_all_tab(kz);
 	}
 
@@ -721,7 +703,7 @@ kz_app_restore_session (KzApp *app)
 	if (g_file_test(location, G_FILE_TEST_EXISTS))
 		kz_bookmark_file_load(KZ_BOOKMARK_FILE(session));
 
-	list = kz_bookmark_get_children(session);
+	list = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(session));
 
 	window_node = priv->window_list;
 	/* windows */
@@ -770,8 +752,8 @@ kz_app_create_new_window (KzApp *app, const gchar *uri)
 
 	if (!kz_session_is_frozen(KZ_SESSION(priv->bookmarks->current_session)))
 	{
-		kz_bookmark_append(priv->bookmarks->current_session,
-				   KZ_WINDOW(window)->tabs);
+		kz_bookmark_folder_append(KZ_BOOKMARK_FOLDER(priv->bookmarks->current_session),
+				   	  KZ_BOOKMARK(KZ_WINDOW(window)->tabs));
 	}
 
 	g_signal_connect(window, "destroy",
@@ -901,7 +883,7 @@ kz_app_get_window_list (KzApp *app)
 }
 
 GtkWidget *
-kz_app_get_window_from_tab (KzApp *app, GtkWidget *tab_widget)
+kz_app_get_window_from_web (KzApp *app, KzWeb *web)
 {
 	KzAppPrivate *priv = KZ_APP_GET_PRIVATE(app);
 	GList *node;
@@ -909,12 +891,12 @@ kz_app_get_window_from_tab (KzApp *app, GtkWidget *tab_widget)
 	for (node = (GList*)priv->window_list; node; node = g_list_next (node))
 	{
 		KzWindow *kz = node->data;
-		GtkWidget *label;
+		KzTabLabel *label;
 
 		if (!KZ_IS_WINDOW(kz)) continue;
 
-		label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-						   tab_widget);
+		label = kz_notebook_get_tab_label(KZ_NOTEBOOK(kz->notebook),
+						  web);
 		if (label)
 			return GTK_WIDGET(kz);
 	}
@@ -970,8 +952,8 @@ cb_destroy_window (GtkObject *object, gpointer data)
 	else
 	{
 		if (kz->tabs)
-			kz_bookmark_remove(priv->bookmarks->current_session,
-					   kz->tabs);
+			kz_bookmark_folder_remove(KZ_BOOKMARK_FOLDER(priv->bookmarks->current_session),
+					   	  KZ_BOOKMARK(kz->tabs));
 	}
 }
 
@@ -988,7 +970,7 @@ kz_app_create_thumbnail (KzApp *app,
 			 GdkPixbuf *pixbuf,
 			 const gchar *uri,
 			 time_t mtime,
-			 EggPixbufThumbSize size)
+			 EggPixbufThumbnailSize size)
 {
 	GError *error = NULL;
 
@@ -998,8 +980,9 @@ kz_app_create_thumbnail (KzApp *app,
 	if (uri[0] == '\0')
 		return FALSE;
 
-	if (!egg_pixbuf_add_thumbnail_data(pixbuf, uri, mtime, size))
-		return FALSE;
+	egg_pixbuf_set_thumbnail_uri(pixbuf, uri);
+        egg_pixbuf_set_thumbnail_mtime(pixbuf, mtime);
+        egg_pixbuf_set_thumbnail_size(pixbuf, size);
 
 	if (!egg_pixbuf_save_thumbnail(pixbuf, &error, NULL))
 	{
