@@ -81,10 +81,8 @@ kz_download_box_class_init (KzDownloadBoxClass *klass)
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	kz_download_box_parent_class = g_type_class_peek_parent (klass);
-
-	gobject_class = (GObjectClass *) klass;
-	widget_class  = (GtkWidgetClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	widget_class  = GTK_WIDGET_CLASS(klass);
 
 	/* GObject signals */
 	gobject_class->dispose      = dispose;
@@ -299,10 +297,10 @@ cb_icon_clicked (GtkWidget *event_box,
 		break;
 	case 3:
 		menu = gtk_menu_new();
-		g_signal_connect(G_OBJECT(icon->event_box), "destroy",
+		g_signal_connect(icon->event_box, "destroy",
 				 G_CALLBACK(cb_download_icon_destroy), menu);
 		menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_CANCEL, NULL);
-		g_signal_connect(G_OBJECT(menuitem), "activate",
+		g_signal_connect(menuitem, "activate",
 				 G_CALLBACK(cb_cancel_menu_activate), icon);
 		gtk_menu_shell_append(GTK_MENU_SHELL (menu), menuitem);
 		gtk_widget_show (menuitem);
@@ -337,16 +335,16 @@ cb_add_item (KzDownloaderGroup *dlgrp,
 
 	/* create semitransparent icon image */
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-	icon_set = gtk_style_lookup_icon_set (GTK_WIDGET(dlbox)->style,
-					      GTK_STOCK_SAVE);
+	icon_set = gtk_style_lookup_icon_set(GTK_WIDGET(dlbox)->style,
+					     GTK_STOCK_SAVE);
 
 	if (icon_set != NULL)
 	{
-		orig_pixbuf = gtk_icon_set_render_icon (
+		orig_pixbuf = gtk_icon_set_render_icon(
 				icon_set,
 				icon->image->style,
-				gtk_widget_get_direction (icon->image),
-				GTK_WIDGET_STATE (icon->image),
+				gtk_widget_get_direction(icon->image),
+				GTK_WIDGET_STATE(icon->image),
 				GTK_ICON_SIZE_MENU,
 				icon->image,
 				"download icon");
@@ -505,11 +503,10 @@ cb_progress_item (KzDownloaderGroup *dlgrp,
 		gtk_image_set_from_pixbuf(GTK_IMAGE(icon->image),
 					  orig_pixbuf);
 		g_object_unref(pixbuf);
-		if(direction)
-			g_free(direction);
+                g_free(direction);
 	}
 	percent_str = g_strdup_printf("%s:%02d%%",
-				      dl->filename,
+				      kz_downloader_get_filename(dl),
 				      percent);
 	gtk_widget_set_tooltip_text(icon->event_box, percent_str);
 	g_free(percent_str);
