@@ -28,11 +28,11 @@
 G_BEGIN_DECLS
 
 #define KZ_TYPE_DOWNLOAD_BOX		(kz_download_box_get_type ())
-#define KZ_DOWNLOAD_BOX(obj)		(GTK_CHECK_CAST ((obj), KZ_TYPE_DOWNLOAD_BOX, KzDownloadBox))
-#define KZ_DOWNLOAD_BOX_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), KZ_TYPE_DOWNLOAD_BOX, KzDownloadBoxClass))
-#define KZ_IS_DOWNLOAD_BOX(obj)		(GTK_CHECK_TYPE ((obj), KZ_TYPE_DOWNLOAD_BOX))
-#define KZ_IS_DOWNLOAD_BOX_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), KZ_TYPE_DOWNLOAD_BOX))
-#define KZ_DOWNLOAD_BOX_GET_CLASS(obj)	(GTK_CHECK_GET_CLASS ((obj), KZ_TYPE_DOWNLOAD_BOX, KzDownloadBoxClass))
+#define KZ_DOWNLOAD_BOX(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), KZ_TYPE_DOWNLOAD_BOX, KzDownloadBox))
+#define KZ_DOWNLOAD_BOX_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), KZ_TYPE_DOWNLOAD_BOX, KzDownloadBoxClass))
+#define KZ_IS_DOWNLOAD_BOX(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KZ_TYPE_DOWNLOAD_BOX))
+#define KZ_IS_DOWNLOAD_BOX_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), KZ_TYPE_DOWNLOAD_BOX))
+#define KZ_DOWNLOAD_BOX_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), KZ_TYPE_DOWNLOAD_BOX, KzDownloadBoxClass))
 
 typedef struct _KzDownloadBox      KzDownloadBox;
 typedef struct _KzDownloadBoxClass KzDownloadBoxClass;
