@@ -54,7 +54,7 @@ static void disconnect_signals             (KzDownloaderGroup *dlgrp,
 					    KzDownloader      *dl);
 
 static void cb_download_complete (KzDownloader *dl, KzDownloaderGroup *dlgrp);
-static void cb_download_error    (KzDownloader *dl, KzDownloaderGroup *dlgrp);
+static void cb_download_error    (KzDownloader *dl, const gchar *error, KzDownloaderGroup *dlgrp);
 static void cb_download_progress (KzDownloader *dl, KzDownloaderGroup *dlgrp);
 
 static gint kz_downloader_group_signals[LAST_SIGNAL] = {0};
@@ -66,9 +66,7 @@ kz_downloader_group_class_init (KzDownloaderGroupClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	kz_downloader_group_parent_class = g_type_class_peek_parent (klass);
-
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	/* GObject class */
 	gobject_class->dispose      = dispose;
@@ -260,7 +258,7 @@ cb_download_complete (KzDownloader *dl, KzDownloaderGroup *dlgrp)
 
 
 static void
-cb_download_error (KzDownloader *dl, KzDownloaderGroup *dlgrp)
+cb_download_error (KzDownloader *dl, const gchar *error, KzDownloaderGroup *dlgrp)
 {
 	g_return_if_fail(KZ_IS_DOWNLOADER_GROUP(dlgrp));
 
@@ -294,16 +292,15 @@ kz_downloader_group_add_item (KzDownloaderGroup *dlgrp,
 
 	basename = g_path_get_basename(uri);
 	dl = kz_downloader_new_with_filename(uri, basename);
+	g_free(basename);
 	if (!dl)
 		return;
 
 	list = g_slist_append(dlgrp->items, dl);
-	g_object_set(G_OBJECT(dlgrp), "kz-downloader-items", list, NULL);
+	g_object_set(dlgrp, "kz-downloader-items", list, NULL);
 	connect_signals(dlgrp, dl);
 
 	kz_downloader_to_file(dl);
-
-	g_free(basename);
 }
 
 
@@ -323,7 +320,7 @@ kz_downloader_group_add_item_with_filename (KzDownloaderGroup *dlgrp,
 		return;
 
 	list = g_slist_append(dlgrp->items, dl);
-	g_object_set(G_OBJECT(dlgrp), "kz-downloader-items", list, NULL);
+	g_object_set(dlgrp, "kz-downloader-items", list, NULL);
 	connect_signals(dlgrp, dl);
 	
 	kz_downloader_to_file(dl);
@@ -338,9 +335,9 @@ kz_downloader_group_add_downloader (KzDownloaderGroup *dlgrp,
 	g_return_if_fail(KZ_IS_DOWNLOADER_GROUP(dlgrp));
 	g_return_if_fail(KZ_IS_DOWNLOADER(dl));
 
-	g_object_ref(G_OBJECT(dl));
+	g_object_ref(dl);
 	list = g_slist_append(dlgrp->items, KZ_DOWNLOADER(dl));
-	g_object_set(G_OBJECT(dlgrp), "kz-downloader-items", list, NULL);
+	g_object_set(dlgrp, "kz-downloader-items", list, NULL);
 
 	connect_signals(dlgrp, KZ_DOWNLOADER(dl));
 }
@@ -349,9 +346,7 @@ kz_downloader_group_add_downloader (KzDownloaderGroup *dlgrp,
 static gboolean
 idle_remove_item(gpointer data)
 {
-	KzDownloader *dl = data;
-
-	g_object_unref(G_OBJECT(dl));
+	g_object_unref(data);
 
 	return FALSE;
 }
@@ -367,6 +362,6 @@ kz_downloader_group_remove_item (KzDownloaderGroup *dlgrp, KzDownloader *dl)
 		      kz_downloader_group_signals[REMOVE_SIGNAL],
 		      0, dl);
 	list = g_slist_remove(dlgrp->items, dl);
-	g_object_set(G_OBJECT(dlgrp), "kz-downloader-items", list, NULL);
+	g_object_set(dlgrp, "kz-downloader-items", list, NULL);
 	g_idle_add(idle_remove_item, dl);
 }
