@@ -22,6 +22,7 @@
 #include "kz-embed.h"
 #include "kz-module.h"
 #include "kz-marshalers.h"
+#include <glib/gi18n.h>
 
 static GList *embeds = NULL;
 static gboolean loaded = FALSE;
@@ -123,14 +124,14 @@ _kz_embed_new (KzModule *module, const gchar *name,
 }
 
 GtkWidget *
-kz_embed_new (const gchar *name)
+kz_embed_new (const gchar *module_name, const gchar *engine_name)
 {
 	KzModule *module;
 
-	module = kz_module_find(embeds, name);
+	module = kz_module_find(embeds, module_name);
 	if (!module)
 	{
-		module = kz_module_load_module(_kz_embed_module_dir(), name);
+		module = kz_module_load_module(_kz_embed_module_dir(), module_name);
 		if (!module)
 			return NULL;
 #warning FIXME: workaround for kz-gecko-embed.cpp.
@@ -138,7 +139,9 @@ kz_embed_new (const gchar *name)
 		embeds = g_list_prepend(embeds, module);
 	}
 
-	return _kz_embed_new(module, name, NULL);
+	return _kz_embed_new(module, module_name,
+			     "engine-name", engine_name,
+			     NULL);
 }
 
 
@@ -169,8 +172,6 @@ enum {
 static void kz_embed_base_init (gpointer g_class);
 static gint kz_embed_signals[LAST_SIGNAL] = {0};
 
-static gchar *kz_embed_get_up_location(KzEmbed *kzembed);
-
 GType
 kz_embed_get_type (void)
 {
@@ -191,220 +192,220 @@ kz_embed_get_type (void)
 		type = g_type_register_static(G_TYPE_INTERFACE, "KzEmbed",
 					      &info, 0);
 		g_type_interface_add_prerequisite(type,
-						  G_TYPE_OBJECT);
+						  GTK_TYPE_WIDGET);
 	}
 	return type;
 }
 
 static void
-kz_embed_base_init (gpointer g_class)
+kz_embed_base_init (gpointer g_iface)
 {
 	static gboolean initialized = FALSE;
 
-	if (initialized) return;
+	if (!initialized)
+	{
+		initialized = TRUE;
 
-	kz_embed_signals[LINK_MESSAGE_SIGNAL]
-		= g_signal_new ("kz-link-message",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, link_message),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[LINK_MESSAGE_SIGNAL]
+			= g_signal_new ("kz-link-message",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, link_message),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__STRING,
+					G_TYPE_NONE, 1,
+					G_TYPE_STRING);
 
-	kz_embed_signals[JS_STATUS_SIGNAL]
-		= g_signal_new ("kz-js-status",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, js_status),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[JS_STATUS_SIGNAL]
+			= g_signal_new ("kz-js-status",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, js_status),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__VOID,
+					G_TYPE_NONE, 0);
 
-	kz_embed_signals[LOCATION_SIGNAL]
-		= g_signal_new ("kz-location",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, location),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[LOCATION_SIGNAL]
+			= g_signal_new ("kz-location",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, location),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__STRING,
+					G_TYPE_NONE, 1,
+					G_TYPE_STRING);
 
-	kz_embed_signals[TITLE_SIGNAL]
-		= g_signal_new ("kz-title",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, title),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[TITLE_SIGNAL]
+			= g_signal_new ("kz-title",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, title),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__STRING,
+					G_TYPE_NONE, 1,
+					G_TYPE_STRING);
 
-	kz_embed_signals[PROGRESS_SIGNAL]
-		= g_signal_new ("kz-progress",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, progress),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[PROGRESS_SIGNAL]
+			= g_signal_new ("kz-progress",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, progress),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__DOUBLE,
+					G_TYPE_NONE, 1,
+					G_TYPE_DOUBLE);
 
-	kz_embed_signals[NET_START_SIGNAL]
-		= g_signal_new ("kz-net-start",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, net_start),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[NET_START_SIGNAL]
+			= g_signal_new ("kz-net-start",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, net_start),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__VOID,
+					G_TYPE_NONE, 0);
 
-	kz_embed_signals[NET_STOP_SIGNAL]
-		= g_signal_new ("kz-net-stop",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, net_stop),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[NET_STOP_SIGNAL]
+			= g_signal_new ("kz-net-stop",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, net_stop),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__VOID,
+					G_TYPE_NONE, 0);
 
-	kz_embed_signals[NEW_WINDOW_SIGNAL]
-		= g_signal_new ("kz-new-window",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, new_window),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__POINTER,
-				G_TYPE_NONE, 1, G_TYPE_POINTER);
+		kz_embed_signals[NEW_WINDOW_SIGNAL]
+			= g_signal_new ("kz-new-window",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, new_window),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__POINTER,
+					G_TYPE_NONE, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[OPEN_URI_SIGNAL]
-		= g_signal_new ("kz-open-uri",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, open_uri),
-				NULL, NULL,
-				_kz_marshal_INT__STRING,
-				G_TYPE_INT, 1, G_TYPE_STRING);
+		kz_embed_signals[OPEN_URI_SIGNAL]
+			= g_signal_new ("kz-open-uri",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, open_uri),
+					NULL, NULL,
+					_kz_marshal_INT__STRING,
+					G_TYPE_INT, 1, G_TYPE_STRING);
 
-	kz_embed_signals[SIZE_TO_SIGNAL]
-		= g_signal_new ("kz-size-to",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, size_to),
-				NULL, NULL,
-				_kz_marshal_VOID__INT_INT,
-				G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
+		kz_embed_signals[SIZE_TO_SIGNAL]
+			= g_signal_new ("kz-size-to",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, size_to),
+					NULL, NULL,
+					_kz_marshal_VOID__INT_INT,
+					G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
-	kz_embed_signals[DOM_KEY_DOWN_SIGNAL]
-		= g_signal_new ("kz-dom-key-down",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_key_down),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_KEY_DOWN_SIGNAL]
+			= g_signal_new ("kz-dom-key-down",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_key_down),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_KEY_PRESS_SIGNAL]
-		= g_signal_new ("kz-dom-key-press",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_key_press),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_KEY_PRESS_SIGNAL]
+			= g_signal_new ("kz-dom-key-press",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_key_press),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_KEY_UP_SIGNAL]
-		= g_signal_new ("kz-dom-key-up",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_key_up),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_KEY_UP_SIGNAL]
+			= g_signal_new ("kz-dom-key-up",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_key_up),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_MOUSE_DOWN_SIGNAL]
-		= g_signal_new ("kz-dom-mouse-down",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_down),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_MOUSE_DOWN_SIGNAL]
+			= g_signal_new ("kz-dom-mouse-down",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_down),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_MOUSE_UP_SIGNAL]
-		= g_signal_new ("kz-dom-mouse-up",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_up),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_MOUSE_UP_SIGNAL]
+			= g_signal_new ("kz-dom-mouse-up",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_up),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_MOUSE_CLICK_SIGNAL]
-		= g_signal_new ("kz-dom-mouse-click",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_click),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_MOUSE_CLICK_SIGNAL]
+			= g_signal_new ("kz-dom-mouse-click",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_click),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_MOUSE_DBL_CLICK_SIGNAL]
-		= g_signal_new ("kz-dom-mouse-dbl-click",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_dbl_click),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_MOUSE_DBL_CLICK_SIGNAL]
+			= g_signal_new ("kz-dom-mouse-dbl-click",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_dbl_click),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_MOUSE_OVER_SIGNAL]
-		= g_signal_new ("kz-dom-mouse-over",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_over),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
+		kz_embed_signals[DOM_MOUSE_OVER_SIGNAL]
+			= g_signal_new ("kz-dom-mouse-over",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_over),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-	kz_embed_signals[DOM_MOUSE_OUT_SIGNAL]
-		= g_signal_new ("kz-dom-mouse-out",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_out),
-				NULL, NULL,
-				_kz_marshal_INT__POINTER,
-				G_TYPE_INT, 1, G_TYPE_POINTER);
-	
-	kz_embed_signals[SELECTION_SIGNAL]
-		= g_signal_new ("kz-selection",
-				KZ_TYPE_EMBED,
-				G_SIGNAL_RUN_FIRST,
-				G_STRUCT_OFFSET(KzEmbedIFace, selection),
-				NULL, NULL,
-				g_cclosure_marshal_VOID__VOID,
-				G_TYPE_NONE, 0);
+		kz_embed_signals[DOM_MOUSE_OUT_SIGNAL]
+			= g_signal_new ("kz-dom-mouse-out",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_LAST,
+					G_STRUCT_OFFSET(KzEmbedIFace, dom_mouse_out),
+					NULL, NULL,
+					_kz_marshal_BOOLEAN__POINTER,
+					G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 
-
-	initialized = TRUE;
+		kz_embed_signals[SELECTION_SIGNAL]
+			= g_signal_new ("kz-selection",
+					KZ_TYPE_EMBED,
+					G_SIGNAL_RUN_FIRST,
+					G_STRUCT_OFFSET(KzEmbedIFace, selection),
+					NULL, NULL,
+					g_cclosure_marshal_VOID__VOID,
+					G_TYPE_NONE, 0);
+		g_object_interface_install_property(
+			g_iface,
+			g_param_spec_string("engine-name",
+					    _("Engine Name"),
+					    _("Engine Name"),
+					    NULL,
+					    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	}
 }
 
 
 void
-kz_embed_load_url (KzEmbed *kzembed, const gchar *url)
+kz_embed_load_uri (KzEmbed *kzembed, const gchar *uri)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 
-	if (kz_embed_get_lock(kzembed))
-	{
-		KzEmbed *newembed = NULL;
-		g_signal_emit_by_name(kzembed, 
-				      "new-window",
-				      &newembed, 0);
-		KZ_EMBED_GET_IFACE(kzembed)->load_url(newembed, url);
-	}
-	else
-	{
-		KZ_EMBED_GET_IFACE(kzembed)->load_url(kzembed, url);
-	}
+	KZ_EMBED_GET_IFACE(kzembed)->load_uri(kzembed, uri);
 }
 
 
@@ -412,69 +413,21 @@ void
 kz_embed_view_source (KzEmbed *kzembed, const gchar *url)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->view_source);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->view_source)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->view_source(kzembed, url);
 }
 
 
-gboolean
-kz_embed_is_loading (KzEmbed *kzembed)
+void
+kz_embed_view_current_page_source_in_new (KzEmbed *kzembed, KzEmbed *new_embed)
 {
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->is_loading, FALSE);
+	g_return_if_fail(KZ_IS_EMBED(kzembed));
+	if (!KZ_EMBED_GET_IFACE(kzembed)->view_current_page_source_in_new)
+		return;
 
-	return KZ_EMBED_GET_IFACE(kzembed)->is_loading(kzembed);
-}
-
-
-const gchar *
-kz_embed_get_title (KzEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_title, NULL);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->get_title(kzembed);
-}
-
-
-const gchar *
-kz_embed_get_location (KzEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_location, NULL);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->get_location(kzembed);
-}
-
-
-gchar *
-kz_embed_ensure_title (KzEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->ensure_title, NULL);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->ensure_title(kzembed);
-}
-
-
-gchar *
-kz_embed_get_link_message (KzEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_link_message, NULL);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->get_link_message(kzembed);
-}
-
-
-gdouble
-kz_embed_get_progress (KzEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), 0.0);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_progress, 0.0);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->get_progress(kzembed);
+	KZ_EMBED_GET_IFACE(kzembed)->view_current_page_source_in_new(kzembed, new_embed);
 }
 
 
@@ -482,7 +435,8 @@ gboolean
 kz_embed_can_cut_selection (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_cut_selection, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->can_cut_selection)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->can_cut_selection(kzembed);
 }
@@ -492,7 +446,8 @@ gboolean
 kz_embed_can_copy_selection (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_copy_selection, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->can_copy_selection)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->can_copy_selection(kzembed);
 }
@@ -502,7 +457,8 @@ gboolean
 kz_embed_can_paste (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_paste, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->can_paste)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->can_paste(kzembed);
 }
@@ -512,7 +468,8 @@ void
 kz_embed_cut_selection (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->cut_selection);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->cut_selection)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->cut_selection(kzembed);
 }
@@ -522,7 +479,8 @@ void
 kz_embed_copy_selection (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->copy_selection);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->copy_selection)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->copy_selection(kzembed);
 }
@@ -532,7 +490,8 @@ void
 kz_embed_paste (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->paste);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->paste)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->paste(kzembed);
 }
@@ -542,7 +501,8 @@ void
 kz_embed_select_all (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->select_all);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->select_all)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->select_all(kzembed);
 }
@@ -552,17 +512,19 @@ gchar *
 kz_embed_get_selection_string (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_selection_string, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_selection_string)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_selection_string(kzembed);
 }
 
 
 gboolean
-kz_embed_find (KzEmbed *kzembed, const char *keyword, gboolean backward)
+kz_embed_find (KzEmbed *kzembed, const gchar *keyword, gboolean backward)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->find, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->find)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->find(kzembed, keyword, backward);
 }
@@ -570,11 +532,12 @@ kz_embed_find (KzEmbed *kzembed, const char *keyword, gboolean backward)
 
 gboolean
 kz_embed_incremental_search (KzEmbed *kzembed,
-			     const char *keyword,
+			     const gchar *keyword,
 			     gboolean backward)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->incremental_search, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->incremental_search)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->incremental_search(kzembed,
 							       keyword,
@@ -586,7 +549,8 @@ gboolean
 kz_embed_selection_is_collapsed (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), TRUE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->selection_is_collapsed, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->selection_is_collapsed)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->selection_is_collapsed(kzembed);
 }
@@ -600,7 +564,8 @@ kz_embed_get_links (KzEmbed *kzembed,
 	g_return_val_if_fail(list, FALSE);
 	*list = NULL;
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_links, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_links)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_links(kzembed,
 						      list,
@@ -615,7 +580,8 @@ kz_embed_get_dest_anchors (KzEmbed *kzembed,
 	g_return_val_if_fail(list, FALSE);
 	*list = NULL;
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_links, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_dest_anchors)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_dest_anchors(kzembed,
 							     list);
@@ -623,29 +589,29 @@ kz_embed_get_dest_anchors (KzEmbed *kzembed,
 
 
 void
-kz_embed_copy_page (KzEmbed *kzembed, KzEmbed *dkzembed, KzEmbedCopyType type)
+kz_embed_copy_page (KzEmbed *src_embed, KzEmbed *dest_embed)
 {
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->copy_page);
+	GList *history = NULL;
+	guint current_position = 0;
 
-	KZ_EMBED_GET_IFACE(kzembed)->copy_page(kzembed, dkzembed, type);
-}
+	g_return_if_fail(KZ_IS_EMBED(src_embed));
+	g_return_if_fail(KZ_IS_EMBED(dest_embed));
 
+	if (G_OBJECT_TYPE(src_embed) == G_OBJECT_TYPE(dest_embed) &&
+	    KZ_EMBED_GET_IFACE(src_embed)->copy_page)
+	{
+		KZ_EMBED_GET_IFACE(src_embed)->copy_page(src_embed, dest_embed);
+		return;
+	}
 
-gboolean
-kz_embed_shistory_copy (KzEmbed *source,
-			KzEmbed *dest,
-			gboolean back_history,
-			gboolean forward_history,
-			gboolean set_current)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(source), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(source)->shistory_copy, FALSE);
+	if (!KZ_EMBED_GET_IFACE(src_embed)->get_history)
+		return;
+	if (!KZ_EMBED_GET_IFACE(dest_embed)->set_history)
+		return;
 
-	return KZ_EMBED_GET_IFACE(source)->shistory_copy(source, dest,
-							 back_history,
-							 forward_history,
-							 set_current);
+	kz_embed_get_history(src_embed, &history, &current_position);
+	kz_embed_set_history(dest_embed, history, current_position);
+	kz_site_list_free(history);
 }
 
 
@@ -654,7 +620,8 @@ kz_embed_shistory_get_pos (KzEmbed *kzembed,
 			   gint *pos, gint *count)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), 0);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->shistory_get_pos, 0);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->shistory_get_pos)
+		return 0;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->shistory_get_pos(kzembed,
 							     pos, count);
@@ -669,7 +636,8 @@ kz_embed_shistory_get_nth (KzEmbed *kzembed,
 			   gchar **aTitle)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->shistory_get_nth);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->shistory_get_nth)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->shistory_get_nth(kzembed, nth, is_relative,
 						      aUrl, aTitle);
@@ -707,7 +675,6 @@ kz_embed_go_back (KzEmbed *kzembed)
 	KZ_EMBED_GET_IFACE(kzembed)->go_back(kzembed);
 }
 
-
 void
 kz_embed_go_forward (KzEmbed *kzembed)
 {
@@ -717,70 +684,32 @@ kz_embed_go_forward (KzEmbed *kzembed)
 	KZ_EMBED_GET_IFACE(kzembed)->go_forward(kzembed);
 }
 
-
-void
-kz_embed_go_up (KzEmbed *kzembed)
-{
-	gchar *location;
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
-
-	location = kz_embed_get_up_location(kzembed);
-	kz_embed_load_url(kzembed, location);
-	g_free(location);
-}
-
-
 gboolean
 kz_embed_can_go_back (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_go_back, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->can_go_back)
+                return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->can_go_back(kzembed);
 }
 
-
 gboolean
 kz_embed_can_go_forward (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_go_forward, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->can_go_forward)
+                return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->can_go_forward(kzembed);
 }
 
-
-gboolean
-kz_embed_can_go_up (KzEmbed *kzembed)
-{
-	gboolean ret;
-	gchar *location;
-
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-
-	location = kz_embed_get_up_location(kzembed);
-
-	if (!location) return FALSE;
-
-	/* stupid! */
-	if (strcmp(location, "http://") &&
-	    strcmp(location, "https://")  &&
-	    strcmp(location, "ftp://")  &&
-	    strcmp(location, "file://"))
-		ret = TRUE;
-	else
-		ret = FALSE;
-	g_free(location);
-
-	return ret;
-}
-
-
 gboolean
 kz_embed_can_go_nav_link (KzEmbed *kzembed, KzEmbedNavLink link)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_go_nav_link, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->can_go_nav_link)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->can_go_nav_link(kzembed, link);
 }
@@ -790,7 +719,8 @@ void
 kz_embed_go_nav_link (KzEmbed *kzembed, KzEmbedNavLink link)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->go_nav_link);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->go_nav_link)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->go_nav_link(kzembed, link);
 }
@@ -802,7 +732,8 @@ kz_embed_append_nav_link (KzEmbed *kzembed,
 			  KzNavi *navi)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->append_nav_link);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->append_nav_link)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->append_nav_link(kzembed, link, navi);
 }
@@ -827,7 +758,8 @@ kz_embed_set_nth_nav_link (KzEmbed *kzembed,
 			   guint n)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_nth_nav_link);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->set_nth_nav_link)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->set_nth_nav_link(kzembed, link, navi, n);
 }
@@ -848,7 +780,8 @@ KzNavi *
 kz_embed_get_nth_nav_link (KzEmbed *kzembed, KzEmbedNavLink link, guint n)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_nth_nav_link, NULL);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_nth_nav_link)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_nth_nav_link(kzembed, link, n);
 }
@@ -858,7 +791,8 @@ GList *
 kz_embed_get_nav_links (KzEmbed *kzembed, KzEmbedNavLink link)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_nav_links, NULL);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_nav_links)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_nav_links(kzembed, link);
 }
@@ -868,77 +802,41 @@ void
 kz_embed_go_history_index (KzEmbed *kzembed, gint index)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->go_history_index);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->go_history_index)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->go_history_index(kzembed, index);
 }
 
 
-void
-kz_embed_do_command (KzEmbed *kzembed, const char *command)
-{
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->do_command);
-
-	KZ_EMBED_GET_IFACE(kzembed)->do_command(kzembed, command);
-}
-
-
-gboolean
-kz_embed_can_do_command (KzEmbed *kzembed, const char *command)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->can_do_command, FALSE);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->can_do_command(kzembed, command);
-}
-
-
-gboolean
-kz_embed_get_lock (KzEmbed *kzembed)
-{
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_lock, FALSE);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->get_lock(kzembed);
-}
-
-
-void
-kz_embed_set_lock (KzEmbed *kzembed, gboolean lock)
-{
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_lock);
-
-	KZ_EMBED_GET_IFACE(kzembed)->set_lock(kzembed, lock);
-}
-
-
 gchar *
 kz_embed_get_body_text (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_body_text, NULL);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_body_text)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_body_text(kzembed);
 }
 
 
 void
-kz_embed_set_encoding (KzEmbed *kzembed, const char *encoding)
+kz_embed_set_encoding (KzEmbed *kzembed, const gchar *encoding)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_encoding);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->set_encoding)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->set_encoding(kzembed, encoding);
 }
 
 
 void
-kz_embed_get_encoding (KzEmbed *kzembed, char **encoding, gboolean *forced)
+kz_embed_get_encoding (KzEmbed *kzembed, gchar **encoding, gboolean *forced)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_encoding);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_encoding)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->get_encoding(kzembed, encoding, forced);
 }
@@ -948,7 +846,8 @@ void
 kz_embed_print (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->print);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->print)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->print(kzembed);
 }
@@ -958,7 +857,8 @@ void
 kz_embed_print_preview (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->print_preview);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->print_preview)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->print_preview(kzembed);
 }
@@ -968,14 +868,15 @@ GList *
 kz_embed_get_printer_list (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_printer_list, NULL);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_printer_list)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_printer_list(kzembed);
 }
 
 
 void
-kz_embed_create_thumbnail (KzEmbed *kzembed, EggPixbufThumbSize size)
+kz_embed_create_thumbnail (KzEmbed *kzembed, EggPixbufThumbnailSize size)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 
@@ -985,10 +886,11 @@ kz_embed_create_thumbnail (KzEmbed *kzembed, EggPixbufThumbSize size)
 
 
 gboolean
-kz_embed_save_with_content (KzEmbed *kzembed, const char *rawfilename)
+kz_embed_save_with_content (KzEmbed *kzembed, const gchar *rawfilename)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->save_with_content, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->save_with_content)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->save_with_content(kzembed,
 							      rawfilename);
@@ -1001,7 +903,8 @@ kz_embed_set_text_into_textarea	(KzEmbed *kzembed,
 				 const gchar *text)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_text_into_textarea, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->set_text_into_textarea)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->set_text_into_textarea(kzembed,
 								   element,
@@ -1013,7 +916,8 @@ gchar *
 kz_embed_get_text_from_textarea (KzEmbed *kzembed, gpointer element)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_text_from_textarea, NULL);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_text_from_textarea)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_text_from_textarea(kzembed,
 								   element);
@@ -1021,40 +925,44 @@ kz_embed_get_text_from_textarea (KzEmbed *kzembed, gpointer element)
 
 
 void
-kz_embed_zoom_set (KzEmbed *kzembed, gint zoom, gboolean reflow)
+kz_embed_zoom (KzEmbed *kzembed, gdouble zoom)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->zoom_set);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->zoom)
+		return;
 
-	KZ_EMBED_GET_IFACE(kzembed)->zoom_set(kzembed, zoom, reflow);
+	KZ_EMBED_GET_IFACE(kzembed)->zoom(kzembed, zoom);
 }
 
 
-gint
-kz_embed_zoom_get (KzEmbed *kzembed)
+gdouble
+kz_embed_get_zoom_ratio (KzEmbed *kzembed)
 {
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), 100);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->zoom_set, 100);
+	g_return_val_if_fail(KZ_IS_EMBED(kzembed), 1.0);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_zoom_ratio)
+		return 1.0;
 
-	return KZ_EMBED_GET_IFACE(kzembed)->zoom_get(kzembed);
+	return KZ_EMBED_GET_IFACE(kzembed)->get_zoom_ratio(kzembed);
 }
 
 
 void
-kz_embed_set_text_size (KzEmbed *kzembed, gint zoom, gboolean reflow)
+kz_embed_set_text_size (KzEmbed *kzembed, gdouble zoom)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_text_size);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->set_text_size)
+		return;
 
-	KZ_EMBED_GET_IFACE(kzembed)->set_text_size(kzembed, zoom, reflow);
+	KZ_EMBED_GET_IFACE(kzembed)->set_text_size(kzembed, zoom);
 }
 
 
-gint
+gdouble
 kz_embed_get_text_size (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), 100);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_text_size, 100);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_text_size)
+		return 100;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_text_size(kzembed);
 }
@@ -1064,7 +972,8 @@ gchar *
 kz_embed_get_html_with_contents (KzEmbed *kzembed, const gchar *storedir)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_html_with_contents, NULL);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_html_with_contents)
+		return NULL;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_html_with_contents(kzembed,
 								   storedir);
@@ -1072,21 +981,21 @@ kz_embed_get_html_with_contents (KzEmbed *kzembed, const gchar *storedir)
 
 
 void
-kz_embed_set_history (KzEmbed *kzembed, KzBookmark *history)
+kz_embed_set_history (KzEmbed *kzembed, const GList *history, guint current_position)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 	if (KZ_EMBED_GET_IFACE(kzembed)->set_history)
-		KZ_EMBED_GET_IFACE(kzembed)->set_history(kzembed, history);
+		KZ_EMBED_GET_IFACE(kzembed)->set_history(kzembed, history, current_position);
 }
 
 
 void
-kz_embed_get_history (KzEmbed *kzembed, KzBookmark *history)
+kz_embed_get_history (KzEmbed *kzembed, GList **history, guint *current_position)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 	
 	if (KZ_EMBED_GET_IFACE(kzembed)->get_history)
-		KZ_EMBED_GET_IFACE(kzembed)->get_history(kzembed, history);
+		KZ_EMBED_GET_IFACE(kzembed)->get_history(kzembed, history, current_position);
 }
 
 
@@ -1094,15 +1003,15 @@ guint
 kz_embed_get_last_modified (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), 0);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_last_modified, 0);
-
-	return KZ_EMBED_GET_IFACE(kzembed)->get_last_modified(kzembed);
+	if (KZ_EMBED_GET_IFACE(kzembed)->get_last_modified)
+		return KZ_EMBED_GET_IFACE(kzembed)->get_last_modified(kzembed);
+	return 0;
 }
 
 
 void
 kz_embed_fine_scroll (KzEmbed *kzembed, 
-		      int horiz, int vert)
+		      gint horiz, gint vert)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
 
@@ -1114,7 +1023,8 @@ void
 kz_embed_page_down (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->page_down);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->page_down)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->page_down(kzembed);
 }
@@ -1124,7 +1034,8 @@ void
 kz_embed_page_up (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->page_up);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->page_up)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->page_up(kzembed);
 }
@@ -1134,7 +1045,8 @@ void
 kz_embed_set_allow_javascript (KzEmbed *kzembed, gboolean allow)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_allow_javascript);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->set_allow_javascript)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->set_allow_javascript(kzembed, allow);
 }
@@ -1144,7 +1056,8 @@ gboolean
 kz_embed_get_allow_javascript (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_allow_javascript, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_allow_javascript)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_allow_javascript(kzembed);
 }
@@ -1154,7 +1067,8 @@ void
 kz_embed_set_allow_images (KzEmbed *kzembed, gboolean allow)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->set_allow_images);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->set_allow_images)
+		return;
 
 	KZ_EMBED_GET_IFACE(kzembed)->set_allow_images(kzembed, allow);
 }
@@ -1164,7 +1078,8 @@ gboolean
 kz_embed_get_allow_images (KzEmbed *kzembed)
 {
 	g_return_val_if_fail(KZ_IS_EMBED(kzembed), FALSE);
-	g_return_val_if_fail(KZ_EMBED_GET_IFACE(kzembed)->get_allow_images, FALSE);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->get_allow_images)
+		return FALSE;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->get_allow_images(kzembed);
 }
@@ -1173,31 +1088,9 @@ void
 kz_embed_show_page_certificate (KzEmbed *kzembed)
 {
 	g_return_if_fail(KZ_IS_EMBED(kzembed));
-	g_return_if_fail(KZ_EMBED_GET_IFACE(kzembed)->show_page_certificate);
+	if (!KZ_EMBED_GET_IFACE(kzembed)->show_page_certificate)
+		return;
 
 	return KZ_EMBED_GET_IFACE(kzembed)->show_page_certificate(kzembed);
 }
 
-static gchar *
-kz_embed_get_up_location(KzEmbed *kzembed)
-{
-	const gchar *location;
-	gchar *up_location = NULL;
-	gchar *pos, *dummy;
-	int len; 
-
-	location = kz_embed_get_location(kzembed);
-	if (!location) return NULL;
-
-	len = strlen(location);
-	if (location[len - 1] == '/')
-		dummy = g_strndup(location, len - 1);
-	else 
-		dummy = g_strndup(location, len);
-	pos =  strrchr(dummy, '/');
-	if (pos)
-		up_location = g_strndup(dummy, pos - dummy + 1);
-	g_free(dummy);
-
-	return up_location;
-}
