@@ -24,6 +24,7 @@
 #include "kz-embed-event.h"
 #include "kz-navi.h"
 #include "egg-pixbuf-thumbnail.h"
+#include "kz-site.h"
 
 #ifndef __KZ_EMBED_H__
 #define __KZ_EMBED_H__
@@ -77,22 +78,14 @@ struct _KzEmbedIFace {
 	GTypeInterface g_iface;
 
 	/* vtable */
-	void		(*load_url)		(KzEmbed       *kzembed,
-						 const gchar   *url);
+	void		(*load_uri)		(KzEmbed       *kzembed,
+						 const gchar   *uri);
 	void		(*view_source)		(KzEmbed       *kzembed,
 						 const gchar   *url);
 	void            (*view_current_page_source_in_new)
                                                 (KzEmbed       *kzembed,
 						 KzEmbed       *new_embed);
 
-	gboolean	(*is_loading)		(KzEmbed       *kzembed);
-
-	const gchar    *(*get_title)		(KzEmbed       *kzembed);
-	const gchar    *(*get_location)		(KzEmbed       *kzembed);
-	gchar	       *(*get_link_message)	(KzEmbed       *kzembed);
-
-	gdouble		(*get_progress)		(KzEmbed       *kzembed);
-
 	gboolean	(*can_cut_selection)	(KzEmbed       *kzembed);
 	gboolean	(*can_copy_selection)	(KzEmbed       *kzembed);
 	gboolean	(*can_paste)		(KzEmbed       *kzembed);
@@ -137,11 +130,9 @@ struct _KzEmbedIFace {
 	void		(*stop_load)		(KzEmbed       *kzembed);
 	void		(*go_back)		(KzEmbed       *kzembed);
 	void		(*go_forward)		(KzEmbed       *kzembed);
-	void		(*go_up)		(KzEmbed       *kzembed);
 
 	gboolean	(*can_go_back)		(KzEmbed       *kzembed);
 	gboolean	(*can_go_forward)	(KzEmbed       *kzembed);
-	gboolean	(*can_go_up)		(KzEmbed       *kzembed);
 
 	gboolean	(*can_go_nav_link)	(KzEmbed       *kzembed,
 						 KzEmbedNavLink	link);
@@ -167,19 +158,7 @@ struct _KzEmbedIFace {
 	void		(*go_history_index)	(KzEmbed       *kzembed,
 						 gint		index);
 
-	void		(*do_command)		(KzEmbed       *kzembed,
-						 const gchar   *command);
-	gboolean	(*can_do_command)	(KzEmbed       *kzembed,
-						 const gchar   *command);
-
-	gboolean	(*get_lock)		(KzEmbed       *kzembed);
-	void		(*set_lock)		(KzEmbed       *kzembed,
-						 gboolean	lock);
-
 	gchar	       *(*get_body_text)	(KzEmbed       *kzembed);
-#if 0
-	gchar	       *(*get_selection_source)	(KzEmbed       *kzembed);
-#endif
 	void		(*set_encoding)		(KzEmbed       *kzembed,
 						 const gchar   *encoding);
 	void		(*get_encoding)		(KzEmbed       *kzembed,
@@ -204,29 +183,25 @@ struct _KzEmbedIFace {
 						 gpointer	element);
 
 
-	void		(*zoom_set)		(KzEmbed       *kzembed, 
-						 gint		zoom, 
-						 gboolean	reflow);
-	gint		(*zoom_get)		(KzEmbed       *kzembed);
+	void		(*zoom) 		(KzEmbed       *kzembed, 
+						 gdouble	zoom);
+	gdouble		(*get_zoom_ratio)	(KzEmbed       *kzembed);
 	void		(*set_text_size)	(KzEmbed       *kzembed, 
-						 gint		zoom, 
-						 gboolean	reflow);
-	gint		(*get_text_size)	(KzEmbed       *kzembed);
+						 gdouble	zoom);
+	gdouble		(*get_text_size)	(KzEmbed       *kzembed);
 
 	gchar	       *(*get_html_with_contents)
 						(KzEmbed       *kzembed,
 						 const gchar   *storedir);
 
 	void		(*set_history)		(KzEmbed       *kzembed,
-						 KzBookmarkFolder *history);
+						 const GList   *history,
+						 guint		current_position);
 	void		(*get_history)		(KzEmbed       *kzembed,
-						 KzBookmarkFolder *history);
+						 GList	      **history,
+						 guint	       *current_position);
 	guint		(*get_last_modified)	(KzEmbed       *kzembed);
 
-#if 0
-	void		(*set_edit_mode)	(KzEmbed       *kzembed);
-	void		(*set_view_mode)	(KzEmbed       *kzembed);
-#endif
 	void            (*fine_scroll)          (KzEmbed       *embed,
 						 gint           horiz,
 						 gint           vert);
@@ -240,7 +215,6 @@ struct _KzEmbedIFace {
 	void		(*set_allow_images)	(KzEmbed       *kzembed,
 						 gboolean	allow);
         void            (*show_page_certificate)(KzEmbed       *kzembed);
-        const gchar    *(*get_name)             (KzEmbed       *kzembed);
 
 	/* signals */
 	void     (*link_message)        (KzEmbed      *embed,
@@ -261,23 +235,23 @@ struct _KzEmbedIFace {
 	void     (*size_to)             (KzEmbed      *embed,
 					 gint          width,
 					 gint          height);
-	gint     (*dom_key_down)        (KzEmbed      *embed,
-					 KzEmbedEvent *event);
-	gint     (*dom_key_press)       (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_key_up)          (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_mouse_down)      (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_mouse_up)        (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_mouse_click)     (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_mouse_dbl_click) (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_mouse_over)      (KzEmbed      *embed,
-					 KzEmbedEvent  event);
-	gint     (*dom_mouse_out)       (KzEmbed      *embed,
+	gboolean (*dom_key_down)        (KzEmbed      *embed,
+		    		 	 KzEmbedEvent *event);
+	gboolean (*dom_key_press)       (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_key_up)          (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_mouse_down)      (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_mouse_up)        (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_mouse_click)     (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_mouse_dbl_click) (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_mouse_over)      (KzEmbed      *embed,
+		    		 	 KzEmbedEvent  event);
+	gboolean (*dom_mouse_out)       (KzEmbed      *embed,
 					 KzEmbedEvent  event);
 	void     (*selection)           (KzEmbed      *embed);
 #if 1 /* not implemented yet */
@@ -299,24 +273,16 @@ void            kz_embed_exit			(void);
 GList	       *kz_embed_engine_names		(void);
 GList	       *kz_embed_engine_ids		(void);
 
-GtkWidget      *kz_embed_new			(const gchar   *name);
-void		kz_embed_load_url		(KzEmbed       *kzembed,
-						 const gchar   *url);
+GtkWidget      *kz_embed_new			(const gchar   *module_name,
+						 const gchar   *engine_name);
+void		kz_embed_load_uri		(KzEmbed       *kzembed,
+						 const gchar   *uri);
 void		kz_embed_view_source		(KzEmbed       *kzembed,
 						 const gchar   *url);
 void            kz_embed_view_current_page_source_in_new
                                                 (KzEmbed       *kzembed,
                                                  KzEmbed       *new_embed);
 
-gboolean	kz_embed_is_loading		(KzEmbed       *kzembed);
-
-const gchar    *kz_embed_get_title		(KzEmbed       *kzembed);
-const gchar    *kz_embed_get_location		(KzEmbed       *kzembed);
-gchar	       *kz_embed_ensure_title		(KzEmbed       *kzembed);
-gchar	       *kz_embed_get_link_message	(KzEmbed       *kzembed);
-
-gdouble		kz_embed_get_progress		(KzEmbed       *kzembed);
-
 gboolean	kz_embed_can_cut_selection	(KzEmbed       *kzembed);
 gboolean	kz_embed_can_copy_selection	(KzEmbed       *kzembed);
 gboolean	kz_embed_can_paste		(KzEmbed       *kzembed);
@@ -362,11 +328,9 @@ void		kz_embed_reload			(KzEmbed       *kzembed,
 void		kz_embed_stop_load		(KzEmbed       *kzembed);
 void		kz_embed_go_back		(KzEmbed       *kzembed);
 void		kz_embed_go_forward		(KzEmbed       *kzembed);
-void		kz_embed_go_up			(KzEmbed       *kzembed);
 
 gboolean	kz_embed_can_go_back		(KzEmbed       *kzembed);
 gboolean	kz_embed_can_go_forward		(KzEmbed       *kzembed);
-gboolean	kz_embed_can_go_up		(KzEmbed       *kzembed);
 
 gboolean	kz_embed_can_go_nav_link	(KzEmbed       *kzembed,
 						 KzEmbedNavLink	link);
@@ -391,16 +355,6 @@ GList          *kz_embed_get_nav_links		(KzEmbed       *kzembed,
 						 KzEmbedNavLink	link);
 void		kz_embed_go_history_index	(KzEmbed       *kzembed,
 						 gint		index);
-
-void		kz_embed_do_command		(KzEmbed       *kzembed,
-						 const gchar   *command);
-gboolean	kz_embed_can_do_command		(KzEmbed       *kzembed,
-						 const gchar   *command);
-
-gboolean	kz_embed_get_lock		(KzEmbed       *kzembed);
-void		kz_embed_set_lock		(KzEmbed       *kzembed,
-						 gboolean	lock);
-
 gchar	       *kz_embed_get_body_text		(KzEmbed       *kzembed);
 
 void		kz_embed_set_encoding		(KzEmbed       *kzembed,
@@ -425,23 +379,23 @@ gchar	       *kz_embed_get_text_from_textarea	(KzEmbed       *kzembed,
 						 gpointer	element);
 
 
-void		kz_embed_zoom_set		(KzEmbed       *kzembed, 
-						 gint		zoom, 
-						 gboolean	reflow);
-gint		kz_embed_zoom_get		(KzEmbed       *kzembed);
+void		kz_embed_zoom   		(KzEmbed       *kzembed, 
+						 gdouble	zoom);
+gdouble		kz_embed_get_zoom_ratio	        (KzEmbed       *kzembed);
 void		kz_embed_set_text_size		(KzEmbed       *kzembed, 
-						 gint		zoom, 
-						 gboolean	reflow);
-gint		kz_embed_get_text_size		(KzEmbed       *kzembed);
+						 gdouble	zoom);
+gdouble		kz_embed_get_text_size		(KzEmbed       *kzembed);
 
 gchar	       *kz_embed_get_html_with_contents
 						(KzEmbed       *kzembed,
 						 const gchar   *storedir);
 
 void		kz_embed_set_history		(KzEmbed       *kzembed,
-						 KzBookmarkFolder *history);
+						 const GList   *history,
+						 guint          current_position);
 void		kz_embed_get_history		(KzEmbed       *kzembed,
-						 KzBookmarkFolder *history);
+						 GList        **history,
+						 guint         *current_position);
 
 guint		kz_embed_get_last_modified	(KzEmbed       *kzembed);	
 
