@@ -35,6 +35,9 @@
 #include "kz-search.h"
 #include "kz-statusbar.h"
 #include "kz-migemo.h"
+#include "kz-site.h"
+#include "kz-password-info.h"
+#include "kz-web.h"
 #include "kz-ext.h"
 
 /* enumerations from "kz-app.h" */
@@ -310,5 +313,78 @@ kz_thumbnails_view_mode_get_type (void)
   return etype;
 }
 
+/* enumerations from "kz-web.h" */
+GType
+kz_web_load_flag_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GFlagsValue values[] = {
+      { KZ_WEB_NORMAL_FLAG, "KZ_WEB_NORMAL_FLAG", "normal-flag" },
+      { KZ_WEB_SOURCE_FLAG, "KZ_WEB_SOURCE_FLAG", "source-flag" },
+      { 0, NULL, NULL }
+    };
+    etype = g_flags_register_static ("KzWebLoadFlag", values);
+  }
+  return etype;
+}
+GType
+kz_web_reload_flag_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { KZ_WEB_RELOAD_NORMAL, "KZ_WEB_RELOAD_NORMAL", "normal" },
+      { KZ_WEB_RELOAD_BYPASS_CACHE, "KZ_WEB_RELOAD_BYPASS_CACHE", "bypass-cache" },
+      { KZ_WEB_RELOAD_BYPASS_PROXY, "KZ_WEB_RELOAD_BYPASS_PROXY", "bypass-proxy" },
+      { KZ_WEB_RELOAD_BYPASS_PROXY_AND_CACHE, "KZ_WEB_RELOAD_BYPASS_PROXY_AND_CACHE", "bypass-proxy-and-cache" },
+      { KZ_WEB_RELOAD_CHARSET_CHANGE, "KZ_WEB_RELOAD_CHARSET_CHANGE", "charset-change" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("KzWebReloadFlag", values);
+  }
+  return etype;
+}
+GType
+kz_web_go_action_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { KZ_WEB_GO_NONE, "KZ_WEB_GO_NONE", "none" },
+      { KZ_WEB_GO_BACK, "KZ_WEB_GO_BACK", "back" },
+      { KZ_WEB_GO_FORWARD, "KZ_WEB_GO_FORWARD", "forward" },
+      { KZ_WEB_GO_UP, "KZ_WEB_GO_UP", "up" },
+      { KZ_WEB_GO_INDEX, "KZ_WEB_GO_INDEX", "index" },
+      { KZ_WEB_GO_CONTENTS, "KZ_WEB_GO_CONTENTS", "contents" },
+      { KZ_WEB_GO_START, "KZ_WEB_GO_START", "start" },
+      { KZ_WEB_GO_PREV, "KZ_WEB_GO_PREV", "prev" },
+      { KZ_WEB_GO_NEXT, "KZ_WEB_GO_NEXT", "next" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("KzWebGoAction", values);
+  }
+  return etype;
+}
+GType
+kz_web_nav_link_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { KZ_WEB_LINK_NEXT, "KZ_WEB_LINK_NEXT", "next" },
+      { KZ_WEB_LINK_PREV, "KZ_WEB_LINK_PREV", "prev" },
+      { KZ_WEB_LINK_INDEX, "KZ_WEB_LINK_INDEX", "index" },
+      { KZ_WEB_LINK_CONTENTS, "KZ_WEB_LINK_CONTENTS", "contents" },
+      { KZ_WEB_LINK_START, "KZ_WEB_LINK_START", "start" },
+      { KZ_WEB_LINK_RSS, "KZ_WEB_LINK_RSS", "rss" },
+      { KZ_WEB_LINK_GUARD, "KZ_WEB_LINK_GUARD", "guard" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("KzWebNavLink", values);
+  }
+  return etype;
+}
+
 /* Generated data ends here */
 
