@@ -44,6 +44,15 @@ GType language_group_get_type (void);
 /* enumerations from "kz-thumbnails-view.h" */
 GType kz_thumbnails_view_mode_get_type (void);
 #define KZ_TYPE_THUMBNAILS_VIEW_MODE (kz_thumbnails_view_mode_get_type())
+/* enumerations from "kz-web.h" */
+GType kz_web_load_flag_get_type (void);
+#define KZ_TYPE_WEB_LOAD_FLAG (kz_web_load_flag_get_type())
+GType kz_web_reload_flag_get_type (void);
+#define KZ_TYPE_WEB_RELOAD_FLAG (kz_web_reload_flag_get_type())
+GType kz_web_go_action_get_type (void);
+#define KZ_TYPE_WEB_GO_ACTION (kz_web_go_action_get_type())
+GType kz_web_nav_link_get_type (void);
+#define KZ_TYPE_WEB_NAV_LINK (kz_web_nav_link_get_type())
 G_END_DECLS
 
 #endif /* __KZ_ENUM_TYPES_H__ */
