@@ -26,7 +26,6 @@
 
 #include "kazehakase.h"
 #include "kz-favicon.h"
-#include "kz-profile.h"
 #include "kz-downloader.h"
 #include "utils.h"
 #include "glib-utils.h"
@@ -37,9 +36,7 @@ struct _KzFaviconPrivate
 {
 	GHashTable *uri_hash;
 	GHashTable *pixbuf_hash;
-	KzProfile  *uri_list;
-
-	guint n_favicon;
+	GKeyFile *favicon_file;
 };
 #define KZ_FAVICON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_FAVICON, KzFaviconPrivate))
 
@@ -48,7 +45,7 @@ enum {
 	LAST_SIGNAL
 };
 
-#define MAX_FAVICONS 2000
+#define MAX_FAVICONS 20000
 
 typedef struct downloader_info
 {
@@ -59,13 +56,12 @@ typedef struct downloader_info
 static void dispose      (GObject *object);
 
 static void kz_favicon_load_uri_list        (KzFavicon *kzfav);
-static void kz_favicon_uri_hash_to_uri_list (gpointer key,
-					     gpointer value,
-					     KzFavicon *kzfav);
+static void kz_favicon_store_uri_list 	    (KzFavicon *kzfav);
 
 static void cb_downloader_load_complete (KzDownloader *dl,
 					 downloader_info_t *info);
 static void cb_downloader_load_error    (KzDownloader *dl,
+					 const gchar *message,
 					 downloader_info_t *info);
 
 static const gchar *kz_favicon_get_favicon_file_name (KzFavicon *kzfav,
@@ -82,8 +78,7 @@ kz_favicon_class_init (KzFaviconClass *klass)
 {
 	GObjectClass *object_class;
 
-	kz_favicon_parent_class = g_type_class_peek_parent (klass);
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->dispose  = dispose;
 	
@@ -98,23 +93,34 @@ kz_favicon_class_init (KzFaviconClass *klass)
 				g_cclosure_marshal_VOID__STRING,
 				G_TYPE_NONE, 1,
 				G_TYPE_STRING);
-	g_type_class_add_private (object_class, sizeof(KzFaviconPrivate));
+	g_type_class_add_private(object_class, sizeof(KzFaviconPrivate));
 }
 
+static gchar *
+get_favicon_filename (void)
+{
+	gchar *filename;
+	filename = g_build_filename(g_get_home_dir(),
+				    "."PACKAGE,
+				    "faviconrc",
+				    NULL);
+	return filename;
+}
 
 static void
 kz_favicon_init (KzFavicon *kzfav)
 {
-	gchar *favicon_file;
-	KzFaviconPrivate *priv = KZ_FAVICON_GET_PRIVATE (kzfav);
+	gchar *filename;
+	KzFaviconPrivate *priv = KZ_FAVICON_GET_PRIVATE(kzfav);
 
-	favicon_file = g_build_filename(g_get_home_dir(),
-				        "."PACKAGE,
-					"faviconrc",
-				   	NULL);
-	priv->uri_list = kz_profile_open(favicon_file, NULL);
-	g_free(favicon_file);
-	kz_profile_set_save_each_time(priv->uri_list, FALSE);
+	filename = get_favicon_filename();
+	priv->favicon_file = g_key_file_new();
+	if (g_file_test(filename, G_FILE_TEST_EXISTS))
+	{
+		g_key_file_load_from_file(priv->favicon_file, filename,
+					  G_KEY_FILE_NONE, NULL);
+	}
+	g_free(filename);
 
 	priv->uri_hash = g_hash_table_new_full (g_str_hash,
 						g_str_equal,
@@ -125,40 +131,22 @@ kz_favicon_init (KzFavicon *kzfav)
 						   NULL,
 						   (GDestroyNotify)g_object_unref);
 	kz_favicon_load_uri_list(kzfav);
-	kz_profile_close(priv->uri_list);
 }
 
 
 static void
 dispose (GObject *object)
 {
-	KzFavicon *kzfav = KZ_FAVICON(object);
-	gchar *favicon_file;
-	KzFaviconPrivate *priv = KZ_FAVICON_GET_PRIVATE (kzfav);
+	KzFaviconPrivate *priv = KZ_FAVICON_GET_PRIVATE(object);
 	
-	favicon_file = g_build_filename(g_get_home_dir(),
-				   	"."PACKAGE,
-					"faviconrc",
-				   	NULL);
-
-	priv->uri_list = kz_profile_open(favicon_file, NULL);
-	g_free(favicon_file);
-	kz_profile_set_save_each_time(priv->uri_list, FALSE);
-
-	kz_profile_delete_section(priv->uri_list, "URI");
-	priv->n_favicon = 0;
-	
-	/* convert hash table to KzProfileList */
-	g_hash_table_foreach(priv->uri_hash,
-			     (GHFunc)kz_favicon_uri_hash_to_uri_list,
-			     kzfav);
-
+	kz_favicon_store_uri_list(KZ_FAVICON(object));
 	g_hash_table_destroy(priv->pixbuf_hash);
-
-	kz_profile_close(priv->uri_list);
 	g_hash_table_destroy(priv->uri_hash);
 	
-	priv->uri_list = NULL;
+	if (priv->favicon_file)
+		g_key_file_free(priv->favicon_file);
+	priv->favicon_file = NULL;
+
 	priv->uri_hash = NULL;
 	priv->pixbuf_hash = NULL;
 
@@ -191,45 +179,70 @@ kz_favicon_get_instance (void)
 static void
 kz_favicon_load_uri_list (KzFavicon *kzfav)
 {
-	KzProfileList *p;
 	KzFaviconPrivate *priv = KZ_FAVICON_GET_PRIVATE (kzfav);
-	KzProfile *uri_list = priv->uri_list;
-	
-	for (p = uri_list->list; p; p = p->next)
+	gchar **uris;
+	gsize i, n_uris;
+
+	uris = g_key_file_get_keys(priv->favicon_file, "URI", &n_uris, NULL);
+	if (!uris)
+		return;
+
+	for (i = 0; i < n_uris; i++)
 	{
-		if (p->type == KZ_PROFILE_DATA_TYPE_KEY 
-		    && g_strcmp(p->section, "URI") == 0)
-		{
-			g_hash_table_insert(priv->uri_hash,
-					    g_strdup(p->key),
-					    g_strdup(p->value));
-		}
+		gchar *value;
+		value = g_key_file_get_value(priv->favicon_file,
+					     "URI",
+					     uris[i], NULL);
+		if (!value)
+			continue;
+		g_hash_table_insert(priv->uri_hash,
+				    uris[i], value);
 	}
+	g_free(uris);
 }
 
 
 static void
-kz_favicon_uri_hash_to_uri_list (gpointer key, gpointer value, KzFavicon *kzfav)
+kz_favicon_store_uri_list (KzFavicon *kzfav)
 {
 	KzFaviconPrivate *priv = KZ_FAVICON_GET_PRIVATE (kzfav);
-	if (priv->n_favicon > MAX_FAVICONS)
+	gchar **uris;
+	gsize n_uris, length;
+	gchar *contents;
+
+	uris = g_key_file_get_keys(priv->favicon_file, "URI", &n_uris, NULL);
+	if (!uris)
 		return;
+	
+	if (n_uris > MAX_FAVICONS)
+	{
+		gsize i;
+		for (i = MAX_FAVICONS; i < n_uris; i++)
+		{
+			g_key_file_remove_key(priv->favicon_file,
+					      "URI", uris[i], NULL);
+		}
+	}
 
-	kz_profile_set_value(priv->uri_list, "URI",
-			     (const gchar*)key, (const gchar *)value,
-			     strlen((gchar *)value) + 1,
-			     KZ_PROFILE_VALUE_TYPE_STRING);
+	contents = g_key_file_to_data(priv->favicon_file, &length, NULL);
+	if (contents)
+	{
+		gchar *filename;
 
-	priv->n_favicon++;
+		filename = get_favicon_filename();
+		g_file_set_contents(filename, contents, length, NULL);
+		g_free(filename);
+		g_free(contents);
+	}
+
+	g_strfreev(uris);
 }
 
 
 static gboolean
 idle_unref_dl(gpointer data)
 {
-	KzDownloader *dl = data;
-
-	g_object_unref(G_OBJECT(dl));
+	g_object_unref(data);
 
 	return FALSE;
 }
@@ -260,7 +273,7 @@ cb_downloader_load_complete (KzDownloader *dl, downloader_info_t *info)
 	kz_downloader_disconnect_signal(dl, info);
 
 	if (!g_file_set_contents(info->favicon_file_location,
-				 kz_downloader_get_string(dl),
+				 kz_downloader_get_buffer(dl),
 				 kz_downloader_get_size(dl),
 				 NULL) &&
 	    g_file_test(info->favicon_file_location, G_FILE_TEST_EXISTS))
@@ -270,9 +283,14 @@ cb_downloader_load_complete (KzDownloader *dl, downloader_info_t *info)
 	}
 	else
 	{
+		gchar *key;
+		key = create_profile_key_from_uri(uri);
+
 		g_hash_table_insert(priv->uri_hash,
-		 create_profile_key_from_uri(uri), 
-		 info->favicon_file_location);
+				    key, info->favicon_file_location);
+		g_key_file_set_string(priv->favicon_file,
+				      "URI", key,
+				      info->favicon_file_location);
 	}
 	g_free(info);
 
@@ -287,7 +305,7 @@ cb_downloader_load_complete (KzDownloader *dl, downloader_info_t *info)
 
 
 static void
-cb_downloader_load_error (KzDownloader *dl, downloader_info_t *info)
+cb_downloader_load_error (KzDownloader *dl, const gchar *message, downloader_info_t *info)
 {
 	gchar *uri;
 	uri = (gchar *)info->document_uri;
