@@ -52,7 +52,7 @@ kz_gesture_class_init (KzGestureClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->dispose = dispose;
 
