@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-icons.c 3081 2007-03-28 03:53:51Z kous $
+ *  $Id: kz-icons.c 3529 2009-01-20 05:53:23Z ikezoe $
  */
 
 #include "kazehakase.h"
@@ -117,5 +117,5 @@ kz_icons_init (void)
 		g_object_unref(kz_icon);
 	}
 
-	g_object_unref(G_OBJECT (factory));
+	g_object_unref(factory);
 }
