@@ -18,14 +18,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "kz-notebook.h"
+
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "gtk-utils.h"
 #include "kz-tab-label.h"
-#include "kz-notebook.h"
 #include "kz-actions.h"
+#include "kz-web.h"
 
 static const gpointer KZ_NOTEBOOK_GROUP = "KzNotebookGroup";
 
@@ -50,6 +52,7 @@ struct _KzNotebookPrivate
 #define KZ_NOTEBOOK_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_NOTEBOOK, KzNotebookPrivate))
 
 static void     dispose      (GObject         *object);
+static void     finalize     (GObject         *object);
 static void     set_property (GObject         *object,
 			      guint            prop_id,
 			      const GValue    *value,
@@ -96,12 +99,13 @@ kz_notebook_class_init (KzNotebookClass *klass)
 	GtkWidgetClass *widget_class;
 	GtkNotebookClass *notebook_class;
 
-	gobject_class  = (GObjectClass *)klass;
-	widget_class   = (GtkWidgetClass *)klass;
-	notebook_class = (GtkNotebookClass *)klass;
+	gobject_class  = G_OBJECT_CLASS(klass);
+	widget_class   = GTK_WIDGET_CLASS(klass);
+	notebook_class = GTK_NOTEBOOK_CLASS(klass);
 
 	/* GtkObject signals */
 	gobject_class->dispose      = dispose;
+	gobject_class->finalize     = finalize;
 	gobject_class->set_property = set_property;
 	gobject_class->get_property = get_property;
 
@@ -122,7 +126,7 @@ kz_notebook_class_init (KzNotebookClass *klass)
 				     G_PARAM_READWRITE |
 				     G_PARAM_CONSTRUCT_ONLY));
 
-	g_type_class_add_private (gobject_class, sizeof(KzNotebookPrivate));
+	g_type_class_add_private(gobject_class, sizeof(KzNotebookPrivate));
 }
 
 static void
@@ -206,32 +210,31 @@ dispose (GObject *object)
 		g_object_unref(priv->kz);
 		priv->kz = NULL;
 	}
-
-	if (priv->open_hist)
-	{
-		g_list_free(priv->open_hist);
-		priv->open_hist = NULL;
-	}
-	if (priv->view_hist)
-	{
-		g_list_free(priv->view_hist);
-		priv->view_hist = NULL;
-	}
 	if (priv->dragged_page)
 	{
 		g_object_unref(priv->dragged_page);
 		priv->dragged_page = NULL;
 	}
-	if (priv->tab_tree)
-	{
-		g_node_destroy(priv->tab_tree);
-		priv->tab_tree = NULL;
-	}
 
 	if (G_OBJECT_CLASS(kz_notebook_parent_class)->dispose)
 		G_OBJECT_CLASS(kz_notebook_parent_class)->dispose(object);
 }
 
+static void
+finalize (GObject *object)
+{
+	KzNotebookPrivate *priv = KZ_NOTEBOOK_GET_PRIVATE(object);
+
+	g_list_free(priv->open_hist);
+	g_list_free(priv->view_hist);
+
+	if (priv->tab_tree)
+		g_node_destroy(priv->tab_tree);
+
+	if (G_OBJECT_CLASS(kz_notebook_parent_class)->finalize)
+		G_OBJECT_CLASS(kz_notebook_parent_class)->finalize(object);
+}
+
 static gboolean
 scroll_event (GtkWidget *widget, GdkEventScroll *event)
 {
@@ -240,10 +243,10 @@ scroll_event (GtkWidget *widget, GdkEventScroll *event)
 	
 	current = gtk_notebook_get_nth_page(GTK_NOTEBOOK(widget),
 					    gtk_notebook_get_current_page(GTK_NOTEBOOK(widget)));
-	originator = gtk_get_event_widget ((GdkEvent *)event);
+	originator = gtk_get_event_widget((GdkEvent *)event);
 
 	/* ignore scroll events from the content of the page */
-	if (!originator || gtk_widget_is_ancestor (originator, current) || originator == current)
+	if (!originator || gtk_widget_is_ancestor(originator, current) || originator == current)
 		return FALSE;
 
 	return kz_notebook_scroll_tab(KZ_NOTEBOOK(widget), event->direction);
@@ -324,9 +327,9 @@ switch_page (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num)
 	KzTabLabel *tab;
 	gchar *title;
 	KzNotebookPrivate *priv = KZ_NOTEBOOK_GET_PRIVATE(notebook);
-	KzEmbed *kzembed = KZ_EMBED(gtk_notebook_get_nth_page(notebook, page_num));
+	KzWeb *kzweb = kz_notebook_get_nth_web(KZ_NOTEBOOK(notebook), page_num);
 
-	title = kz_embed_ensure_title(kzembed);
+	title = kz_web_ensure_title(kzweb);
 	if (title)
 	{
 		gtk_window_set_title(GTK_WINDOW(priv->kz), title);
@@ -337,10 +340,10 @@ switch_page (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num)
 		gtk_window_set_title(GTK_WINDOW(priv->kz), _("Kazehakase"));
 	}
 
-	priv->view_hist = g_list_remove(priv->view_hist, kzembed);
-	priv->view_hist = g_list_prepend(priv->view_hist, kzembed);
+	priv->view_hist = g_list_remove(priv->view_hist, kzweb);
+	priv->view_hist = g_list_prepend(priv->view_hist, kzweb);
 
-	tab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(notebook, GTK_WIDGET(kzembed)));
+	tab = kz_notebook_get_tab_label(KZ_NOTEBOOK(notebook), kzweb);
 	g_return_if_fail(tab);
 
 	if (kz_tab_label_get_state(tab) == KZ_TAB_LABEL_STATE_LOADED)
@@ -348,7 +351,7 @@ switch_page (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num)
 		const gchar *location;
 		gboolean focus;
 		kz_tab_label_set_state(tab, KZ_TAB_LABEL_STATE_NORMAL);
-		location = kz_embed_get_location(kzembed);
+		location = kz_web_get_location(kzweb);
 		if((!location || *location == 0) &&
 		   KZ_CONF_GET("Tab", "focus_loc_ent_new", focus, BOOL) &&
 		   focus)
@@ -364,8 +367,8 @@ switch_page (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num)
 static void
 cb_page_added (GtkNotebook *notebook, GtkWidget *child, guint page_num, gpointer data)
 {
-	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(notebook), child, TRUE);
-	gtk_notebook_set_tab_detachable(GTK_NOTEBOOK(notebook), child, TRUE);
+	gtk_notebook_set_tab_reorderable(notebook, child, TRUE);
+	gtk_notebook_set_tab_detachable(notebook, child, TRUE);
 }
 
 static void
@@ -383,14 +386,14 @@ cb_page_reordered (GtkNotebook *notebook,
 	if (sibchild)
 		sibtab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(notebook, sibchild));
 	if (sibtab)
-		sib_bookmark = sibtab->history;
+		sib_bookmark = KZ_BOOKMARK(sibtab->history);
 
 	kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(notebook, child));
 	g_object_ref(kztab->history);
-	kz_bookmark_remove(kz->tabs, kztab->history);
-	kz_bookmark_insert_before(kz->tabs, 
-				  kztab->history,
-				  sib_bookmark);
+	kz_bookmark_folder_remove(kz->tabs, KZ_BOOKMARK(kztab->history));
+	kz_bookmark_folder_insert_before(kz->tabs, 
+				  	 KZ_BOOKMARK(kztab->history),
+					 sib_bookmark);
 	g_object_unref(kztab->history);
 }
 
@@ -402,7 +405,7 @@ get_insert_tab_position (KzNotebook *notebook)
 
 	KZ_CONF_GET("Tab", "new_tab_position", pos_str, STRING);
 	
-	if (!pos_str || !strcasecmp(pos_str, "last"))
+	if (pos_str[0] == '\0' || !strcasecmp(pos_str, "last"))
 	{
 		pos = -1;
 	}
@@ -431,14 +434,11 @@ get_insert_tab_position (KzNotebook *notebook)
 		{
 			while (i < num)
 			{
-				GtkWidget *kzembed;
-				GtkWidget *label;
+				KzTabLabel *kztab;
 				KzTabLabelState state;
 
-				kzembed = GTK_WIDGET(gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), i));
-				label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
-								   kzembed);
-				state = kz_tab_label_get_state(KZ_TAB_LABEL(label));
+				kztab = kz_notebook_get_nth_tab_label(notebook, i);
+				state = kz_tab_label_get_state(kztab);
 				if (state == KZ_TAB_LABEL_STATE_NORMAL)
 					break;
 				i++;
@@ -451,24 +451,26 @@ get_insert_tab_position (KzNotebook *notebook)
 }
 
 gint
-kz_notebook_open_new_tab_at_pos (KzNotebook *notebook, KzEmbed *embed, KzTabLabel *label, gint pos)
+kz_notebook_open_new_tab_at_pos (KzNotebook *notebook, KzWeb *web, KzTabLabel *label, gint pos)
 {
 	KzNotebookPrivate *priv = KZ_NOTEBOOK_GET_PRIVATE(notebook);
 	gint inserted_pos;
 
+	gtk_widget_show_all(GTK_WIDGET(web));
+	gtk_widget_show(GTK_WIDGET(label));
 	inserted_pos = gtk_notebook_insert_page(GTK_NOTEBOOK(notebook),
-						GTK_WIDGET(embed),
+						GTK_WIDGET(web),
 						GTK_WIDGET(label),
 						pos);
 
 	/* add to this notebook's history */
-	priv->open_hist = g_list_prepend(priv->open_hist, embed);
+	priv->open_hist = g_list_prepend(priv->open_hist, web);
 
 	return inserted_pos;
 }
 
 gint
-kz_notebook_open_new_tab (KzNotebook *notebook, KzEmbed *embed, KzTabLabel *label)
+kz_notebook_open_new_tab (KzNotebook *notebook, KzWeb *web, KzTabLabel *label)
 {
 	gint pos;
 	GNode *node;
@@ -476,17 +478,17 @@ kz_notebook_open_new_tab (KzNotebook *notebook, KzEmbed *embed, KzTabLabel *labe
 
 	pos = get_insert_tab_position(notebook);
 
-	pos = kz_notebook_open_new_tab_at_pos(notebook, embed, label, pos);
+	pos = kz_notebook_open_new_tab_at_pos(notebook, web, label, pos);
 
 	/* insret node */
-	node = g_node_new(embed);
+	node = g_node_new(web);
 	g_node_append(priv->tab_tree, node);
 
 	return pos;
 }
 
 gint
-kz_notebook_open_new_tab_with_parent (KzNotebook *notebook, KzEmbed *embed, KzTabLabel *label, KzEmbed *parent)
+kz_notebook_open_new_tab_with_parent (KzNotebook *notebook, KzWeb *web, KzTabLabel *label, KzWeb *parent)
 {
 	gint pos;
 	GNode *node, *parent_node;
@@ -494,10 +496,10 @@ kz_notebook_open_new_tab_with_parent (KzNotebook *notebook, KzEmbed *embed, KzTa
 
 	pos = get_insert_tab_position(notebook);
 
-	pos = kz_notebook_open_new_tab_at_pos(notebook, embed, label, pos);
+	pos = kz_notebook_open_new_tab_at_pos(notebook, web, label, pos);
 	
 	/* insret node */
-	node = g_node_new(embed);
+	node = g_node_new(web);
 	parent_node = g_node_find(priv->tab_tree,
 				  G_IN_ORDER, G_TRAVERSE_ALL, parent);
 	if (parent_node)
@@ -509,23 +511,23 @@ kz_notebook_open_new_tab_with_parent (KzNotebook *notebook, KzEmbed *embed, KzTa
 }
 
 gint
-kz_notebook_prepend_new_tab (KzNotebook *notebook, KzEmbed *embed, KzTabLabel *label)
+kz_notebook_prepend_new_tab (KzNotebook *notebook, KzWeb *web, KzTabLabel *label)
 {
 	gint pos;
 	GNode *node;
 	KzNotebookPrivate *priv = KZ_NOTEBOOK_GET_PRIVATE(notebook);
 
-	pos = kz_notebook_open_new_tab_at_pos(notebook, embed, label, -1);
+	pos = kz_notebook_open_new_tab_at_pos(notebook, web, label, -1);
 
 	/* insret node */
-	node = g_node_new(embed);
+	node = g_node_new(web);
 	g_node_append(priv->tab_tree, node);
 
 	return pos;
 }
 
 gboolean
-kz_notebook_close_tab (KzNotebook *notebook, GtkWidget *widget)
+kz_notebook_close_tab (KzNotebook *notebook, KzWeb *web)
 {
 	gchar *ret_page = NULL;
 	KzTabLabel *kztab = NULL;
@@ -533,17 +535,16 @@ kz_notebook_close_tab (KzNotebook *notebook, GtkWidget *widget)
 	KzNotebookPrivate *priv = KZ_NOTEBOOK_GET_PRIVATE(notebook);
 	KzWindow *kz = priv->kz;
 	GNode *node, *child;
-	GtkWidget *current = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
-						       gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)));
-	kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
-							widget));
+	KzWeb *current = kz_notebook_get_nth_web(notebook,
+						 gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)));
+	kztab = kz_notebook_get_tab_label(notebook, web);
 
 	if (kztab && kz_tab_label_get_lock(kztab))
 	{
 		/* kztab is locked, so return without closing tab */
 		return FALSE;
 	}
-	else if (current != widget)
+	else if (current != web)
 	{
 		/* close tab which is not current , so there is nothing to consider */
 		goto CLOSE_TAB;
@@ -594,9 +595,9 @@ kz_notebook_close_tab (KzNotebook *notebook, GtkWidget *widget)
 	g_free(ret_page);
 
 CLOSE_TAB:
-	priv->view_hist = g_list_remove(priv->view_hist, widget);
+	priv->view_hist = g_list_remove(priv->view_hist, web);
 	node = g_node_find(priv->tab_tree,
-			   G_IN_ORDER, G_TRAVERSE_ALL, widget);
+			   G_IN_ORDER, G_TRAVERSE_ALL, web);
 	if (node)
 	{
 		/* move children */
@@ -611,7 +612,7 @@ CLOSE_TAB:
 
 		g_node_destroy(node);
 	}
-	gtk_widget_destroy(widget);
+	gtk_widget_destroy(GTK_WIDGET(web));
 
 	if (kz && gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) == 0)
 	{
@@ -634,13 +635,13 @@ kz_notebook_close_tabs (KzNotebook *notebook, KzNotebookCloseCondition condition
 
 	for (i = num - 1; i >= 0; i--)
 	{
-		GtkWidget *widget = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), i);
+		KzWeb *web = kz_notebook_get_nth_web(notebook, i);
 
 		if (i == pos) continue;
 		if (i > pos && condition == KZ_NOTEBOOK_CLOSE_BACKWARD) continue;
 		if (i < pos && condition == KZ_NOTEBOOK_CLOSE_FORWARD) continue;
 
-		kz_notebook_close_tab(notebook, widget);
+		kz_notebook_close_tab(notebook, web);
 	}
 	return TRUE;
 }
@@ -668,7 +669,7 @@ kz_notebook_move_tab (KzNotebook *src_notebook, KzNotebook *dest_notebook, GtkWi
 
 	/* create new tab label */
 	dest_priv = KZ_NOTEBOOK_GET_PRIVATE(dest_notebook);
-	new_kztab = KZ_TAB_LABEL(kz_tab_label_new(dest_priv->kz, KZ_EMBED(widget)));
+	new_kztab = KZ_TAB_LABEL(kz_tab_label_new(dest_priv->kz, KZ_WEB(widget)));
 
 	/* move the page to this window */
 	g_object_ref(widget);
@@ -680,13 +681,10 @@ kz_notebook_move_tab (KzNotebook *src_notebook, KzNotebook *dest_notebook, GtkWi
 
 	src_priv = KZ_NOTEBOOK_GET_PRIVATE(src_notebook);
 	/* remove view_hist */
-	src_priv->view_hist = g_list_remove(src_priv->view_hist,
-						widget);
+	src_priv->view_hist = g_list_remove(src_priv->view_hist, widget);
 	/* move open_hist */
-	src_priv->open_hist = g_list_remove(src_priv->open_hist,
-						widget);
-	dest_priv->open_hist = g_list_prepend(dest_priv->open_hist,
-					      widget);
+	src_priv->open_hist = g_list_remove(src_priv->open_hist, widget);
+	dest_priv->open_hist = g_list_prepend(dest_priv->open_hist, widget);
 	/* move tab tree */
 	node = g_node_find(src_priv->tab_tree,
 			   G_IN_ORDER, G_TRAVERSE_ALL, widget);
@@ -786,12 +784,36 @@ kz_notebook_scroll_tab (KzNotebook *notebook, GdkScrollDirection direction)
 	return FALSE;
 }
 
-GtkWidget *
-kz_notebook_get_nth_page (KzNotebook *notebook, gint page_num)
+KzWeb *
+kz_notebook_get_nth_web (KzNotebook *notebook, gint page_num)
 {
+	GtkWidget *widget;
+
 	g_return_val_if_fail(KZ_IS_NOTEBOOK(notebook), NULL);
 
-	return gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page_num);
+	widget = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page_num);
+
+	return widget ? KZ_WEB(widget) : NULL;
+}
+
+KzTabLabel *
+kz_notebook_get_tab_label (KzNotebook *notebook, KzWeb *web)
+{
+	return KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook), GTK_WIDGET(web)));
+}
+
+KzTabLabel *
+kz_notebook_get_nth_tab_label (KzNotebook *notebook, gint page_num)
+{
+	GtkWidget *widget;
+
+	g_return_val_if_fail(KZ_IS_NOTEBOOK(notebook), NULL);
+
+	widget = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page_num);
+	if (!widget)
+		return NULL;
+
+	return KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook), widget));
 }
 
 gint
@@ -826,49 +848,43 @@ kz_notebook_get_n_pages (KzNotebook *notebook)
 	return gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
 }
 
-KzEmbed *
-kz_notebook_get_sibling_embed (KzNotebook *notebook, KzEmbed *embed)
+KzWeb *
+kz_notebook_get_sibling_web (KzNotebook *notebook, KzWeb *web)
 {
 	gint pos;
-	GtkWidget *sibembed;
+	GtkWidget *sibweb;
 
 	g_return_val_if_fail(KZ_IS_NOTEBOOK(notebook), NULL);
-	g_return_val_if_fail(KZ_IS_EMBED(embed), NULL);
+	g_return_val_if_fail(KZ_IS_WEB(web), NULL);
 
-	pos = gtk_notebook_page_num(GTK_NOTEBOOK(notebook), GTK_WIDGET(embed));
+	pos = gtk_notebook_page_num(GTK_NOTEBOOK(notebook), GTK_WIDGET(web));
 	if (pos < 0)
 		return NULL;
 	pos++;
 	if (pos >= gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)))
 		return NULL;
 
-	sibembed = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), pos);
+	sibweb = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), pos);
 
-	if (sibembed)
-		return KZ_EMBED(sibembed);
-	else
-		return NULL;
+	return sibweb ? KZ_WEB(sibweb) : NULL;
 }
 
 KzTabLabel *
 kz_notebook_get_sibling_tab_label (KzNotebook *notebook, KzTabLabel *label)
 {
-	KzEmbed *sibembed;
-	GtkWidget *widget;
+	KzWeb *sibweb;
+	KzTabLabel *kztab;
 
 	g_return_val_if_fail(KZ_IS_NOTEBOOK(notebook), NULL);
 	g_return_val_if_fail(KZ_IS_TAB_LABEL(label), NULL);
-	g_return_val_if_fail(KZ_IS_EMBED(label->kzembed), NULL);
+	g_return_val_if_fail(KZ_IS_WEB(label->kzweb), NULL);
 
-	sibembed = kz_notebook_get_sibling_embed(notebook, label->kzembed);
-	if (!sibembed)
+	sibweb = kz_notebook_get_sibling_web(notebook, label->kzweb);
+	if (!sibweb)
 		return NULL;
 
-	widget = gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook), GTK_WIDGET(sibembed));
-	if (widget)
-		return KZ_TAB_LABEL(widget);
-	else
-		return NULL;
+	kztab = kz_notebook_get_tab_label(notebook, sibweb);
+	return kztab;
 }
 
 GtkWidget *
@@ -877,9 +893,36 @@ kz_notebook_get_dragged_page (KzNotebook *notebook)
 	return KZ_NOTEBOOK_GET_PRIVATE(notebook)->dragged_page;
 }
 
+KzWeb *
+kz_notebook_get_dragged_web (KzNotebook *notebook)
+{
+	return KZ_WEB(KZ_NOTEBOOK_GET_PRIVATE(notebook)->dragged_page);
+}
+
 GNode *
 kz_notebook_get_tree (KzNotebook *notebook)
 {
 	return KZ_NOTEBOOK_GET_PRIVATE(notebook)->tab_tree;
 }
 
+void
+kz_notebook_foreach_web (KzNotebook *notebook,
+			   GFunc       func,
+                           gpointer    user_data)
+{
+	GList *pages, *node;
+
+	pages = gtk_container_get_children(GTK_CONTAINER(notebook));
+	if (!pages)
+		return;
+
+	for (node = pages; node; node = g_list_next(node))
+	{
+		GtkWidget *web = node->data;
+		if (!web)
+			continue;
+		func(web, user_data);
+	}
+	g_list_free(pages);
+}
+
