@@ -114,9 +114,9 @@ kz_prefs_win_class_init (KzPrefsWinClass *klass)
 	GtkDialogClass *dialog_class;
 	GtkWidgetClass *widget_class;
 
-	object_class = (GObjectClass *) klass;
-	dialog_class = (GtkDialogClass *) klass;
-	widget_class = (GtkWidgetClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
+	dialog_class = GTK_DIALOG_CLASS(klass);
+	widget_class = GTK_WIDGET_CLASS(klass);
 
 	object_class->dispose      = dispose;
 
