@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-sidebar.c 2640 2006-12-02 09:57:25Z ikezoe $
+ *  $Id: kz-sidebar.c 3675 2009-02-14 01:40:31Z ikezoe $
  */
 
 #include "kz-sidebar.h"
@@ -85,7 +85,7 @@ kz_sidebar_class_init (KzSidebarClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->dispose      = kz_sidebar_dispose;
 	gobject_class->set_property = kz_sidebar_set_property;
