@@ -8,11 +8,11 @@
 G_BEGIN_DECLS
 
 #define KZ_TYPE_STATUSBAR		(kz_statusbar_get_type ())
-#define KZ_STATUSBAR(obj)		(GTK_CHECK_CAST ((obj), KZ_TYPE_STATUSBAR, KzStatusbar))
-#define KZ_STATUSBAR_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), KZ_TYPE_STATUSBAR, KzStatusbarClass))
-#define KZ_IS_STATUSBAR(obj)		(GTK_CHECK_TYPE ((obj), KZ_TYPE_STATUSBAR))
-#define KZ_IS_STATUSBAR_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), KZ_TYPE_STATUSBAR))
-#define KZ_STATUSBAR_GET_CLASS(obj)	(GTK_CHECK_GET_CLASS ((obj), KZ_TYPE_STATUSBAR, KzStatusbarClass))
+#define KZ_STATUSBAR(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), KZ_TYPE_STATUSBAR, KzStatusbar))
+#define KZ_STATUSBAR_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), KZ_TYPE_STATUSBAR, KzStatusbarClass))
+#define KZ_IS_STATUSBAR(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KZ_TYPE_STATUSBAR))
+#define KZ_IS_STATUSBAR_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), KZ_TYPE_STATUSBAR))
+#define KZ_STATUSBAR_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), KZ_TYPE_STATUSBAR, KzStatusbarClass))
 
 typedef struct _KzStatusbar      KzStatusbar;
 typedef struct _KzStatusbarClass KzStatusbarClass;
