@@ -38,7 +38,7 @@
 enum {
 	PROP_0,
 	PROP_KZ_WINDOW,
-	PROP_KZ_EMBED
+	PROP_KZ_WEB
 };
 
 
@@ -112,12 +112,12 @@ static void     cb_global_profile_changed       (KzProfile *profile,
 						 const gchar *old_value,
 						 KzTabLabel *kztab);
 
-/* callbacks for embed */
-static void cb_title_changed (KzEmbed *embed, const gchar *title, KzTabLabel *kztab);
-static void cb_location_changed (KzEmbed *embed, const gchar *location, KzTabLabel *kztab);
-static void cb_net_start     (KzEmbed *embed, KzTabLabel *kztab);
-static void cb_net_stop      (KzEmbed *embed, KzTabLabel *kztab);
-static void cb_progress      (KzEmbed *embed, gdouble ratio, KzTabLabel *kztab);
+/* callbacks for web */
+static void cb_title_changed (KzWeb *web, const gchar *title, KzTabLabel *kztab);
+static void cb_location_changed (KzWeb *web, const gchar *location, KzTabLabel *kztab);
+static void cb_net_start     (KzWeb *web, KzTabLabel *kztab);
+static void cb_net_stop      (KzWeb *web, KzTabLabel *kztab);
+static void cb_progress      (KzWeb *web, gdouble ratio, KzTabLabel *kztab);
 
 static void make_progress_circle (GtkWidget *widget);
 
@@ -176,11 +176,11 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 
 	g_object_class_install_property
 		(gobject_class,
-		 PROP_KZ_EMBED,
-		 g_param_spec_object ("kz-embed",
-				      _("KzEmbed"),
-				      _("The KzEmbed object to observe"),
-				      KZ_TYPE_EMBED,
+		 PROP_KZ_WEB,
+		 g_param_spec_object ("kz-web",
+				      _("KzWeb"),
+				      _("The KzWeb object to observe"),
+				      KZ_TYPE_WEB,
 				      G_PARAM_READWRITE |
 				      G_PARAM_CONSTRUCT));
 	g_type_class_add_private (gobject_class, sizeof(KzTabLabelPrivate));
@@ -195,7 +195,7 @@ kz_tab_label_init (KzTabLabel *kztab)
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
 	kztab->kz           = NULL;
-	kztab->kzembed      = NULL;
+	kztab->kzweb      = NULL;
 	kztab->state        = KZ_TAB_LABEL_STATE_NORMAL;
 	kztab->favicon      = gtk_image_new();
 	kztab->eventbox     = gtk_event_box_new();
@@ -291,32 +291,32 @@ kz_tab_label_init (KzTabLabel *kztab)
 }
 
 static void
-connect_embed_signals (KzTabLabel *kztab)
+connect_web_signals (KzTabLabel *kztab)
 {
-	g_signal_connect(kztab->kzembed, "kz-title",
+	g_signal_connect(kztab->kzweb, "kz-title",
 			 G_CALLBACK(cb_title_changed), kztab);
-	g_signal_connect(kztab->kzembed, "kz-location",
+	g_signal_connect(kztab->kzweb, "kz-location",
 			 G_CALLBACK(cb_location_changed), kztab);
-	g_signal_connect(kztab->kzembed, "kz-net-start",
+	g_signal_connect(kztab->kzweb, "kz-net-start",
 			 G_CALLBACK(cb_net_start), kztab);
-	g_signal_connect(kztab->kzembed, "kz-net-stop",
+	g_signal_connect(kztab->kzweb, "kz-net-stop",
 			 G_CALLBACK(cb_net_stop), kztab);
-	g_signal_connect(kztab->kzembed, "kz-progress",
+	g_signal_connect(kztab->kzweb, "kz-progress",
 			 G_CALLBACK(cb_progress), kztab);
 }
 
 static void
-disconnect_embed_signals (KzTabLabel *kztab)
+disconnect_web_signals (KzTabLabel *kztab)
 {
-	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+	g_signal_handlers_disconnect_by_func(kztab->kzweb,
 					     cb_net_start, kztab);
-	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+	g_signal_handlers_disconnect_by_func(kztab->kzweb,
 					     cb_net_stop, kztab);
-	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+	g_signal_handlers_disconnect_by_func(kztab->kzweb,
 					     cb_title_changed, kztab);
-	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+	g_signal_handlers_disconnect_by_func(kztab->kzweb,
 					     cb_location_changed, kztab);
-	g_signal_handlers_disconnect_by_func(kztab->kzembed,
+	g_signal_handlers_disconnect_by_func(kztab->kzweb,
 					     cb_progress, kztab);
 }
 
@@ -334,10 +334,10 @@ constructor (GType                  type,
 
 	kztab = KZ_TAB_LABEL(object);
 
-	connect_embed_signals(kztab);
+	connect_web_signals(kztab);
 
 	/* set label text */
-	title = kz_embed_ensure_title(kztab->kzembed);
+	title = kz_web_ensure_title(kztab->kzweb);
 	kz_tab_label_set_text(kztab, title);
 	g_free(title);
 
@@ -360,8 +360,8 @@ dispose (GObject *object)
 	g_signal_handlers_disconnect_by_func(KZ_GET_GLOBAL_PROFILE,
 					     G_CALLBACK(cb_global_profile_changed),
 					     kztab);
-	if (kztab->kzembed)
-		kz_tab_label_set_embed(kztab, NULL);
+	if (kztab->kzweb)
+		kz_tab_label_set_web(kztab, NULL);
 
 	if (kztab->history)
 	{
@@ -407,8 +407,8 @@ set_property (GObject         *object,
 	case PROP_KZ_WINDOW:
 		tablabel->kz = g_object_ref(g_value_get_object(value));
 		break;
-	case PROP_KZ_EMBED:
-		kz_tab_label_set_embed(tablabel, g_value_get_object(value));
+	case PROP_KZ_WEB:
+		kz_tab_label_set_web(tablabel, g_value_get_object(value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -430,8 +430,8 @@ get_property (GObject         *object,
 	case PROP_KZ_WINDOW:
 		g_value_set_object(value, tablabel->kz);
 		break;
-	case PROP_KZ_EMBED:
-		g_value_set_object(value, tablabel->kzembed);
+	case PROP_KZ_WEB:
+		g_value_set_object(value, tablabel->kzweb);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -441,16 +441,16 @@ get_property (GObject         *object,
 
 
 GtkWidget *
-kz_tab_label_new (KzWindow *kz, KzEmbed *kzembed)
+kz_tab_label_new (KzWindow *kz, KzWeb *kzweb)
 {
 	KzTabLabel *kztab;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
-	g_return_val_if_fail(KZ_IS_EMBED(kzembed), NULL);
+	g_return_val_if_fail(KZ_IS_WEB(kzweb), NULL);
 
 	kztab = g_object_new(KZ_TYPE_TAB_LABEL,
 			     "kz-window", kz,
-			     "kz-embed",  kzembed,
+			     "kz-web",  kzweb,
 			     NULL);
 
 	return GTK_WIDGET(kztab);
@@ -480,7 +480,7 @@ static gboolean
 idle_create_thumbnail (gpointer data)
 {
 	KzTabLabel *kztab;
-	KzEmbed *embed;
+	KzWeb *web;
 	EggPixbufThumbnailSize size = EGG_PIXBUF_THUMBNAIL_LARGE;
 	const gchar *uri;
 	guint last_modified;
@@ -488,15 +488,15 @@ idle_create_thumbnail (gpointer data)
 	kztab = data;
 	g_return_val_if_fail(KZ_IS_TAB_LABEL(kztab), FALSE);
 
-	embed = kztab->kzembed;
-	uri = kz_embed_get_location(embed);
+	web = kztab->kzweb;
+	uri = kz_web_get_location(web);
 	if (!uri)
 		return FALSE;
 
 	if (uri[0] == '\0')
 		return FALSE;
 
-	last_modified = kz_embed_get_last_modified(embed);
+	last_modified = kz_web_get_last_modified(web);
 	if (last_modified > 0)
 	{
 		GTime thumbnail_last_modified;
@@ -506,7 +506,7 @@ idle_create_thumbnail (gpointer data)
 			return FALSE;
 	}
 
-	kz_embed_create_thumbnail(embed, size);
+	kz_web_create_thumbnail(web, size);
 	return FALSE;
 }
 
@@ -535,7 +535,7 @@ kz_tab_label_set_state (KzTabLabel *kztab,
 		kz_tab_label_set_visited(kztab);
 	}
 
-	title = kz_embed_ensure_title(kztab->kzembed);
+	title = kz_web_ensure_title(kztab->kzweb);
 	kz_tab_label_set_text(kztab, title);
 	g_free(title);
 }
@@ -596,10 +596,10 @@ realize (GtkWidget *widget)
 
 	make_progress_circle(widget);
 
-	if (kz_embed_is_loading(kztab->kzembed))
+	if (kz_web_is_loading(kztab->kzweb))
 		kz_tab_label_set_state (kztab, KZ_TAB_LABEL_STATE_LOADING);
 
-	priv->javascript = kz_embed_get_allow_javascript(kztab->kzembed);
+	priv->javascript = kz_web_get_allow_javascript(kztab->kzweb);
 }
 
 static gboolean
@@ -622,14 +622,14 @@ button_press (GtkWidget *widget, GdkEventButton *event)
 #endif
 	if (event->button == 2)
 	{
-		kz_window_close_tab(kztab->kz, kztab->kzembed);
+		kz_window_close_tab(kztab->kz, kztab->kzweb);
 	}
 	if (event->type == GDK_2BUTTON_PRESS)
 	{
-		kz_embed_reload(kztab->kzembed,
+		kz_web_reload(kztab->kzweb,
 				(event->state & GDK_SHIFT_MASK) ?
-				KZ_EMBED_RELOAD_BYPASS_PROXY_AND_CACHE : 
-				KZ_EMBED_RELOAD_NORMAL);
+				KZ_WEB_RELOAD_BYPASS_PROXY_AND_CACHE : 
+				KZ_WEB_RELOAD_NORMAL);
 	}
 	if (GTK_WIDGET_CLASS(kz_tab_label_parent_class)->button_press_event)
 		GTK_WIDGET_CLASS(kz_tab_label_parent_class)->button_press_event(widget, event);
@@ -694,7 +694,7 @@ drag_data_received (GtkWidget *widget,
 
 	kztab = KZ_TAB_LABEL(widget);
 
-	g_return_if_fail (KZ_IS_EMBED(kztab->kzembed));
+	g_return_if_fail (KZ_IS_WEB(kztab->kzweb));
 
 	switch (info)
 	{
@@ -708,8 +708,7 @@ drag_data_received (GtkWidget *widget,
 		if (data->length < 0) return;
 
 		strings = g_strsplit((const gchar*)data->data, "\n", 2);
-		kz_embed_load_url(kztab->kzembed,
-				      strings[0]);
+		kz_web_load_uri(kztab->kzweb, strings[0]);
 		g_strfreev(strings);
 		break;
 	}
@@ -822,7 +821,7 @@ static void
 cb_close_button_clicked (GtkWidget *button, KzTabLabel *kztab)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	kz_window_close_tab(kztab->kz, kztab->kzembed);
+	kz_window_close_tab(kztab->kz, kztab->kzweb);
 }
 
 static void
@@ -862,10 +861,10 @@ cb_global_profile_changed (KzProfile *profile,
 	}
 }
 
-/* callbacks for embed */
+/* callbacks for web */
 
 static void
-cb_title_changed (KzEmbed *embed, const gchar *title, KzTabLabel *kztab)
+cb_title_changed (KzWeb *web, const gchar *title, KzTabLabel *kztab)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
@@ -873,7 +872,7 @@ cb_title_changed (KzEmbed *embed, const gchar *title, KzTabLabel *kztab)
 }
 
 static void
-cb_location_changed (KzEmbed *embed, const gchar *location, KzTabLabel *kztab)
+cb_location_changed (KzWeb *web, const gchar *location, KzTabLabel *kztab)
 {
 	KzBookmark *bookmark;
 
@@ -885,7 +884,7 @@ cb_location_changed (KzEmbed *embed, const gchar *location, KzTabLabel *kztab)
 }
 
 static void
-cb_net_start (KzEmbed *embed, KzTabLabel *kztab)
+cb_net_start (KzWeb *web, KzTabLabel *kztab)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
@@ -898,18 +897,21 @@ cb_net_start (KzEmbed *embed, KzTabLabel *kztab)
 }
 
 static void
-cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
+cb_net_stop (KzWeb *web, KzTabLabel *kztab)
 {
 	KzNotebook *note;
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE(kztab);
+	GList *history = NULL;
+	guint current_position = 0;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	kz_embed_get_history(KZ_EMBED(embed), 
-			     kztab->history);
+	kz_web_get_history(KZ_WEB(web), &history, &current_position);
+	kz_utils_site_list_to_bookmark_folder(history, current_position, kztab->history);
+	kz_site_list_free(history);
 
 	note = KZ_NOTEBOOK(kztab->kz->notebook);
-	if(kz_notebook_page_num(note, GTK_WIDGET(embed)) ==
+	if(kz_notebook_page_num(note, GTK_WIDGET(web)) ==
 	   kz_notebook_get_current_page(KZ_NOTEBOOK(note)))
 	{
 		kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_NORMAL);
@@ -922,7 +924,7 @@ cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 		bookmark = kz_bookmark_folder_get_current_bookmark(kztab->history);
 		if (bookmark)
 			last_visited = kz_bookmark_get_last_visited(bookmark);
-		last_modified = kz_embed_get_last_modified(embed);
+		last_modified = kz_web_get_last_modified(web);
 
 		if (last_modified != 0 && last_visited > last_modified)
 			kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_NORMAL);
@@ -935,20 +937,26 @@ cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 		const gchar *uri;
 		GdkPixbuf *favicon_pixbuf;
 
-		uri = kz_embed_get_location(KZ_EMBED(kztab->kzembed));
+		uri = kz_web_get_location(KZ_WEB(kztab->kzweb));
 		favicon_pixbuf = kz_favicon_get_pixbuf(KZ_GET_FAVICON, uri,
 						       GTK_ICON_SIZE_MENU);
-		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
-					  favicon_pixbuf);
 		if (favicon_pixbuf)
+		{
+			gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
+						  favicon_pixbuf);
 			g_object_unref(favicon_pixbuf);
+		}
+		else
+		{
+			gtk_image_clear(GTK_IMAGE(kztab->favicon));
+		}
 	}
 }
 
 
 
 static void
-cb_progress (KzEmbed *embed, gdouble ratio, KzTabLabel *kztab)
+cb_progress (KzWeb *web, gdouble ratio, KzTabLabel *kztab)
 {
 	GtkWidget *widget;	       
 	GdkGC *gc;
@@ -1018,8 +1026,8 @@ kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 	priv->lock = lock;
-	kz_embed_set_lock(kztab->kzembed, lock);
-	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
+	kz_web_set_lock(kztab->kzweb, lock);
+	kz_actions_set_sensitive(kztab->kz, kztab->kzweb);
 	
 	kz_bookmark_folder_set_lock(kztab->history, lock);
 	kz_tab_label_sync_to_profile(kztab);
@@ -1044,8 +1052,8 @@ kz_tab_label_set_javascript (KzTabLabel *kztab, gboolean javascript)
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 	priv->javascript = javascript;
-	kz_embed_set_allow_javascript(kztab->kzembed, javascript);
-	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
+	kz_web_set_allow_javascript(kztab->kzweb, javascript);
+	kz_actions_set_tab_sensitive(kztab->kz, kztab->kzweb);
 	
 	kz_bookmark_folder_set_javascript(kztab->history, javascript);
 	kz_tab_label_sync_to_profile(kztab);
@@ -1062,8 +1070,8 @@ cb_auto_refresh(KzTabLabel* kztab)
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 	KZ_CONF_GET("Tab", "auto_refresh_interval_sec", auto_refresh_interval_sec, INT);
 
-	kz_embed_reload(KZ_EMBED(kztab->kzembed),
-			KZ_EMBED_RELOAD_NORMAL);
+	kz_web_reload(KZ_WEB(kztab->kzweb),
+			KZ_WEB_RELOAD_NORMAL);
 
 	if (priv->auto_refresh_id != 0)
 		g_source_remove(priv->auto_refresh_id);
@@ -1084,6 +1092,7 @@ kz_tab_label_get_auto_refresh (KzTabLabel *kztab)
 	return priv->auto_refresh;
 
 }
+
 void
 kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 {
@@ -1112,7 +1121,7 @@ kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 		}
 	}
 	priv->auto_refresh = auto_refresh;
-	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
+	kz_actions_set_tab_sensitive(kztab->kz, kztab->kzweb);
 
 	kz_bookmark_folder_set_auto_refresh(kztab->history, auto_refresh);
 	kz_tab_label_sync_to_profile(kztab);
@@ -1121,7 +1130,9 @@ kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 void
 kz_tab_label_set_history (KzTabLabel *kztab, KzBookmarkFolder *history)
 {	
-        gboolean allow_javacript;
+	gboolean allow_javacript;
+	GList *site_list = NULL;
+	guint current_position = 0;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
@@ -1133,12 +1144,14 @@ kz_tab_label_set_history (KzTabLabel *kztab, KzBookmarkFolder *history)
 
 	if (kz_bookmark_folder_get_auto_refresh(history))
 		kz_tab_label_set_auto_refresh(kztab, TRUE);
-        
-        allow_javacript = kz_bookmark_folder_get_javascript(history);
-        kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), allow_javacript);
-        kz_tab_label_set_javascript(kztab, allow_javacript);
 
-	kz_embed_set_history(kztab->kzembed, kztab->history);
+	allow_javacript = kz_bookmark_folder_get_javascript(history);
+	kz_web_set_allow_javascript(KZ_WEB(kztab->kzweb), allow_javacript);
+	kz_tab_label_set_javascript(kztab, allow_javacript);
+
+	kz_utils_bookmark_folder_to_site_list(history, &site_list, &current_position);
+	kz_web_set_history(kztab->kzweb, site_list, current_position);
+	kz_site_list_free(site_list);
 }
 static void
 kz_tab_label_set_visited (KzTabLabel *kztab)
@@ -1154,7 +1167,7 @@ kz_tab_label_set_visited (KzTabLabel *kztab)
 		kz_bookmark_set_last_visited(bookmark, now.tv_sec);
 
 	/* find the bookmark which has the same uri from the system bookmark */
-	uri = kz_embed_get_location(kztab->kzembed);
+	uri = kz_web_get_location(kztab->kzweb);
 	bookmark = kz_bookmark_folder_find_bookmark_from_uri(KZ_BOOKMARK_FOLDER(KZ_GET_MENU_BOOKMARK),
 						      	     uri);
 	if (bookmark)
@@ -1162,19 +1175,19 @@ kz_tab_label_set_visited (KzTabLabel *kztab)
 }
 
 void
-kz_tab_label_set_embed (KzTabLabel *kztab, KzEmbed *kzembed)
+kz_tab_label_set_web (KzTabLabel *kztab, KzWeb *kzweb)
 {
-	if (kztab->kzembed)
+	if (kztab->kzweb)
 	{
-		disconnect_embed_signals(kztab);
-		g_object_unref(kztab->kzembed);
-		kztab->kzembed = NULL;
+		disconnect_web_signals(kztab);
+		g_object_unref(kztab->kzweb);
+		kztab->kzweb = NULL;
 	}
 
-	if (kzembed)
+	if (kzweb)
 	{
-		kztab->kzembed = g_object_ref(kzembed);
-		connect_embed_signals(kztab);
+		kztab->kzweb = g_object_ref(kzweb);
+		connect_web_signals(kztab);
 	}
 }
 
