@@ -24,7 +24,7 @@
 #include <gtk/gtk.h>
 
 #include "kz-objects.h"
-#include "kz-embed.h"
+#include "kz-web.h"
 #include "kz-bookmark-folder.h"
 
 G_BEGIN_DECLS
@@ -51,7 +51,7 @@ struct _KzTabLabel
 	GtkHBox parent;
 
 	KzWindow        *kz;
-	KzEmbed         *kzembed;
+	KzWeb         *kzweb;
 
 	KzTabLabelState  state;
 
@@ -73,7 +73,7 @@ struct _KzTabLabelClass
 
 GType            kz_tab_label_get_type          (void) G_GNUC_CONST;
 GtkWidget       *kz_tab_label_new               (KzWindow *kz,
-						 KzEmbed  *kzembed);
+						 KzWeb  *kzweb);
 
 void             kz_tab_label_set_text          (KzTabLabel *kztab,
 						 const gchar *text);
@@ -102,8 +102,8 @@ void             kz_tab_label_set_javascript    (KzTabLabel *kztab,
 
 void		 kz_tab_label_set_history       (KzTabLabel *kztab,
 						 KzBookmarkFolder *history);
-void		 kz_tab_label_set_embed		(KzTabLabel *kztab,
-						 KzEmbed    *kzembed);
+void		 kz_tab_label_set_web		(KzTabLabel *kztab,
+						 KzWeb    *kzweb);
 G_END_DECLS
 
 #endif /* __KZ_TAB_LABEL_H__ */
