@@ -37,7 +37,7 @@
 #include "kz-actions-popup.h"
 #include "kz-actions-tab.h"
 #include "kz-bookmark-bar.h"
-#include "kz-embed.h"
+#include "kz-web.h"
 #include "kz-tab-label.h"
 #include "kz-icons.h"
 #include "kazehakase.h"
@@ -56,6 +56,7 @@
 #include "kz-ext.h"
 #include "kz-statusbar.h"
 #include "kz-session.h"
+#include "kz-web.h"
 
 #define MAX_CLOSED_TABS 10
 
@@ -140,7 +141,7 @@ static gboolean scroll_event   (GtkWidget      *widget,
 				GdkEventScroll *event);
 /*
  * mozilla doesn't accept these signals, so we connect these funtions to
- * KzWindow instead of KzEmbed.
+ * KzWindow instead of KzWeb.
  */
 static gboolean motion_notify_event
 			       (GtkWidget      *widget,
@@ -155,16 +156,16 @@ static void     kz_window_append_tab     (KzWindow      *kz,
 static void     kz_window_remove_tab     (KzWindow      *kz,
 					  GtkWidget     *widget);
 
-static KzEmbed *kz_window_create_embed   (KzWindow      *kz);
+static KzWeb   *kz_window_create_web     (KzWindow      *kz);
 
-static void     kz_window_set_cur_embed_callbacks  (KzWindow   *kz,
-						    KzEmbed    *kzembed);
-static void     kz_window_unset_cur_embed_callbacks(KzWindow   *kz,
-						    KzEmbed    *kzembed);
-static void     kz_window_set_embed_callbacks      (KzWindow   *kz,
-						    KzEmbed    *kzembed);
-static void     kz_window_unset_embed_callbacks    (KzWindow   *kz,
-						    KzEmbed    *kzembed);
+static void     kz_window_set_cur_web_callbacks  (KzWindow *kz,
+						  KzWeb    *web);
+static void     kz_window_unset_cur_web_callbacks(KzWindow *kz,
+						  KzWeb    *web);
+static void     kz_window_set_web_callbacks      (KzWindow *kz,
+						  KzWeb    *web);
+static void     kz_window_unset_web_callbacks    (KzWindow *kz,
+						  KzWeb    *web);
 
 /* callbacks */
 static void     cb_profile_global_changed     (KzProfile       *profile,
@@ -227,39 +228,36 @@ static void     cb_sidebar_map                (GtkWidget       *widget,
 static void     cb_sidebar_unmap              (GtkWidget       *widget,
 					       GtkToggleAction *action);
  
-/* callbacks for embed */
-static void     cb_embed_title_changed         (KzEmbed     *embed,
+/* callbacks for web */
+static void     cb_web_title_changed         (KzWeb     *web,
+					      const gchar *title,
+                                              KzWindow    *kz);
+static void     cb_web_location_changed      (KzWeb     *web,
+					      const gchar *location,
+                                              KzWindow    *kz);
+static void     cb_web_link_message          (KzWeb     *web,
+						const gchar *message,
 						KzWindow    *kz);
-static void     cb_embed_location_changed      (KzEmbed     *embed,
+static void     cb_web_load_started          (KzWeb     *web,
 						KzWindow    *kz);
-static void     cb_embed_link_message          (KzEmbed     *embed,
+static void     cb_web_load_finished         (KzWeb     *web,
 						KzWindow    *kz);
-static void     cb_embed_load_started          (KzEmbed     *embed,
+static void     cb_web_new_window            (KzWeb     *web,
+						KzWeb    **new_web,
 						KzWindow    *kz);
-static void     cb_embed_load_finished         (KzEmbed     *embed,
+static void     cb_web_close_tab             (GtkObject   *obj,
 						KzWindow    *kz);
-static void     cb_embed_new_window            (KzEmbed     *embed,
-						KzEmbed    **new_embed,
-						KzWindow    *kz);
-static void     cb_embed_close_tab             (GtkObject   *obj,
-						KzWindow    *kz);
-static gint     cb_embed_dom_key_down          (KzEmbed     *embed,
-						KzEmbedEventKey *event,
-						KzWindow    *kz);
-static gint     cb_embed_dom_key_up            (KzEmbed     *embed,
-						KzEmbedEventKey *event,
-						KzWindow    *kz);
-static gint     cb_embed_dom_mouse_click       (KzEmbed     *embed,
+static gboolean     cb_web_dom_mouse_click   (KzWeb     *web,
 						KzEmbedEventMouse *event,
 						KzWindow    *kz);
-static gint     cb_embed_dom_mouse_down        (KzEmbed     *embed,
+static gboolean     cb_web_dom_mouse_down    (KzWeb     *web,
 						KzEmbedEventMouse *event,
 						KzWindow    *kz);
-static gint     cb_embed_dom_mouse_up          (KzEmbed *embed,
+static gboolean     cb_web_dom_mouse_up      (KzWeb *web,
 						KzEmbedEventMouse *event,
 						KzWindow    *kz);
 
-static gint     cb_embed_dom_mouse_over          (KzEmbed *embed,
+static gboolean     cb_web_dom_mouse_over    (KzWeb *web,
 						KzEmbedEventMouse *event,
 						KzWindow    *kz);
 
@@ -282,12 +280,12 @@ static void
 kz_window_class_init (KzWindowClass *klass)
 {
 	GObjectClass *gobject_class;
-	GtkObjectClass *object_class;
+	GtkObjectClass *gtk_object_class;
 	GtkWidgetClass *widget_class;
 
-	gobject_class = (GObjectClass *)   klass;
-	object_class  = (GtkObjectClass *) klass;
-	widget_class  = (GtkWidgetClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
+	gtk_object_class  = GTK_OBJECT_CLASS(klass);
+        widget_class  = GTK_WIDGET_CLASS(klass);
 
 	kz_window_signals[APPEND_TAB_SIGNAL]
 		= g_signal_new("append-tab",
@@ -320,7 +318,7 @@ kz_window_class_init (KzWindowClass *klass)
 				GTK_TYPE_WIDGET, G_TYPE_INT);
 
 	/* GtkObject signals */
-	object_class->destroy = destroy;
+	gtk_object_class->destroy = destroy;
 
 	/* GtkWidget signals */
 	widget_class->delete_event         = delete_event;
@@ -385,26 +383,28 @@ kz_window_setup_action_groups (KzWindow *kz)
 	kz_window_setup_action_group(kz->tabpop_actions, accel_group);
 }
 
+static void
+append_bookmark_bar_widget (KzBookmark *bookmark, KzWindow *kz)
+{
+	GtkWidget *bar = GTK_WIDGET(kz_bookmark_bar_new(kz, bookmark));
+	kz->bookmark_bars = g_list_append(kz->bookmark_bars, bar);
+}
+
 static void
 kz_window_init (KzWindow *kz)
 {
 	GtkWidget *menu_box;
 	GtkSettings *setting;
-	GList *children, *node;
 	GtkSettings *settings;
+	GList *node;
 	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE(kz);
 
 	/* init member */
 	kz->top_vbox           = gtk_vbox_new(FALSE, 0);
 
 	kz->bookmark_bars_area = gtk_vbox_new(FALSE, 0);
-	children = kz_bookmark_get_children(KZ_GET_BAR_BOOKMARK);
-	for (node = children; node; node = g_list_next(node))
-	{
-		GtkWidget *bar = GTK_WIDGET(kz_bookmark_bar_new(kz, node->data));
-		kz->bookmark_bars = g_list_append(kz->bookmark_bars, bar);
-	}
-	g_list_free(children);
+	kz_bookmark_folder_foreach_child(KZ_BOOKMARK_FOLDER(KZ_GET_BAR_BOOKMARK),
+					 (GFunc)append_bookmark_bar_widget, kz);
 
 	kz->statusbar          = kz_statusbar_new(kz);
 	kz->notebook           = kz_notebook_new(kz);
@@ -434,9 +434,9 @@ kz_window_init (KzWindow *kz)
 	
 	priv->sidebar_was_shown = FALSE;
 
-	kz->tabs            = kz_bookmark_pure_folder_new();
-	kz->closed_tabs     = kz_bookmark_pure_folder_new();
-	kz->history_search  = kz_bookmark_pure_folder_new();
+	kz->tabs            = KZ_BOOKMARK_FOLDER(kz_bookmark_folder_new("Window"));
+	kz->closed_tabs     = KZ_BOOKMARK_FOLDER(kz_bookmark_folder_new("Closed tabs"));
+	kz->history_search  = kz_bookmark_folder_new(NULL);
 	
 	kz_ext_setup(kz);
 
@@ -596,8 +596,6 @@ kz_window_init (KzWindow *kz)
 			 NULL);
 	gtk_key_theme_changed_cb(settings, 0, 0);
 
-	kz_bookmark_set_title(kz->tabs, "Window");
-
 	kz_window_restore_state(kz);
 }
 
@@ -606,19 +604,22 @@ static void
 kz_window_append_tab (KzWindow *kz, GtkWidget *widget, GtkWidget *parent)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	kz_actions_set_tab_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	kz_actions_set_tab_sensitive(kz, KZ_WINDOW_CURRENT_WEB(kz));
 }
 
 
 static void
 kz_window_remove_tab (KzWindow *kz, GtkWidget *widget)
 {
+	KzWeb *web;
+
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	if (!GTK_WIDGET_REALIZED(kz)) return;
 
-	kz_actions_set_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
-	kz_actions_set_tab_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	web = KZ_WINDOW_CURRENT_WEB(kz);
+	kz_actions_set_sensitive(kz, web);
+	kz_actions_set_tab_sensitive(kz, web);
 }
 
 
@@ -647,7 +648,7 @@ kz_window_sync_ui_level (KzWindow *kz)
 	GError *err = NULL;
 	gchar *ui_file;
 	GtkWidget *widget;
-	KzEmbed *kzembed;
+	KzWeb *kzweb;
 	KzWindowPrivate *priv;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
@@ -748,7 +749,7 @@ kz_window_sync_ui_level (KzWindow *kz)
 		{
 			kz_bookmark_menu_remove_tab_menuitems(GTK_MENU_SHELL(submenu), kz);
 			kz_bookmark_menu_append_tab_menuitems(GTK_MENU_SHELL(submenu), kz,
-							      kz->closed_tabs);
+							      KZ_BOOKMARK(kz->closed_tabs));
 		}
 	}
 
@@ -767,8 +768,9 @@ kz_window_sync_ui_level (KzWindow *kz)
 	kz_window_sync_proxy(kz);
 
 	/* for stop/reload button updating */
-	kzembed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
-	kz_actions_set_sensitive(kz, kzembed);
+	kzweb = KZ_WINDOW_CURRENT_WEB(kz);
+	if (kzweb)
+		kz_actions_set_sensitive(kz, kzweb);
 
 	/* smart bookmark */
 	kz_actions_update_smartbookmarks(kz, KZ_GET_SMART_BOOKMARK);
@@ -815,31 +817,31 @@ kz_window_open_new_tab (KzWindow *kz, const gchar *url)
 	return kz_window_open_new_tab_with_parent(kz, url, NULL);
 }
 
-
 /* It's temporary implementation */
-static KzEmbed *
-kz_window_create_embed (KzWindow *kz)
+static KzWeb *
+kz_window_create_web (KzWindow *kz)
 {
 	gchar *engine;
-	KzEmbed *kz_embed = NULL;
+	KzWeb *web = NULL;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
 	engine = KZ_CONF_GET_STR("Browser", "layout_engine");
 	if (engine)
 	{
-		kz_embed = KZ_EMBED(kz_embed_new(engine));
+		if (engine[0])
+			web = KZ_WEB(kz_web_new(engine, "gecko"));
 		g_free(engine);
 	}
 
-	if (!kz_embed)
-		kz_embed = KZ_EMBED(kz_embed_new("gecko"));
-	if (!kz_embed)
-		kz_embed = KZ_EMBED(kz_embed_new("gtk-webcore"));
-	if (!kz_embed)
-		kz_embed = KZ_EMBED(kz_embed_new("ie"));
+	if (!web)
+		web = KZ_WEB(kz_web_new("gecko", "gecko"));
+	if (!web)
+		web = KZ_WEB(kz_web_new("webkit_gtk", "webkit_gtk"));
+	if (!web)
+		web = KZ_WEB(kz_web_new("ie", "ie"));
 
-	return kz_embed;
+	return web;
 }
 
 
@@ -847,33 +849,31 @@ GtkWidget *
 kz_window_open_new_tab_with_parent (KzWindow *kz, const gchar *url,
 				    GtkWidget *parent)
 {
-	KzEmbed *kzembed;
+	KzWeb *web;
 	KzTabLabel *kztab, *sibtab;
 	
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 	if (url) g_return_val_if_fail(g_utf8_validate(url, strlen(url), NULL), NULL);
 
-	kzembed = kz_window_create_embed(kz);
-	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
+	web = kz_window_create_web(kz);
+	g_return_val_if_fail(web, NULL);
+	kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, web));
 
-	gtk_widget_show_all(GTK_WIDGET(kzembed));
-	gtk_widget_show(GTK_WIDGET(kztab));
-
-	kz_notebook_open_new_tab(KZ_NOTEBOOK(kz->notebook), kzembed, kztab);
+	kz_notebook_open_new_tab(KZ_NOTEBOOK(kz->notebook), web, kztab);
 	sibtab = kz_notebook_get_sibling_tab_label(KZ_NOTEBOOK(kz->notebook), kztab);
 	/* insert tab bookmark */
-	kz_bookmark_insert_before(kz->tabs, 
-				  kztab->history,
-				  sibtab ? sibtab->history : NULL);
+	kz_bookmark_folder_insert_before(kz->tabs,
+					 KZ_BOOKMARK(kztab->history),
+					 sibtab ? KZ_BOOKMARK(sibtab->history) : NULL);
 
-	kz_window_set_embed_callbacks(kz, kzembed);
+	kz_window_set_web_callbacks(kz, web);
 
 	g_signal_emit(kz, kz_window_signals[APPEND_TAB_SIGNAL],
-		      0, kzembed, parent);
+		      0, web, parent);
 
-	kz_embed_load_url(kzembed, url);
+	kz_web_load_uri(web, url);
 
-	return GTK_WIDGET(kzembed);
+	return GTK_WIDGET(web);
 }
 
 
@@ -882,7 +882,7 @@ kz_window_restore_tabs (KzWindow *kz, KzBookmark *tabs)
 {
 	GList *childtabs, *tabnode;
 	KzWindowPrivate *priv;
-	KzEmbed *curembed;
+	KzWeb *curweb;
 	gint pos;
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
@@ -892,52 +892,67 @@ kz_window_restore_tabs (KzWindow *kz, KzBookmark *tabs)
 	g_object_unref(kz->tabs);
 	kz->tabs = g_object_ref(tabs);
 
-	childtabs = kz_bookmark_get_children(tabs);
+	childtabs = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(tabs));
 	/* tabs */
 	for (tabnode = childtabs; tabnode; tabnode = g_list_next(tabnode))
 	{
-		KzEmbed *kzembed;
+		KzWeb *kzweb;
 		KzTabLabel *kztab;
-		KzBookmark *child;
-		kzembed = kz_window_create_embed(kz);
-		kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzembed));
-		child = KZ_BOOKMARK(tabnode->data);
+		KzBookmarkFolder *child;
+		kzweb = kz_window_create_web(kz);
+		kztab = KZ_TAB_LABEL(kz_tab_label_new(kz, kzweb));
+		child = KZ_BOOKMARK_FOLDER(tabnode->data);
 
-		gtk_widget_show_all(GTK_WIDGET(kzembed));
-		gtk_widget_show(GTK_WIDGET(kztab));
-		kz_notebook_prepend_new_tab(KZ_NOTEBOOK(kz->notebook), kzembed, kztab);
-		kz_window_set_embed_callbacks(kz, kzembed);
+		kz_notebook_prepend_new_tab(KZ_NOTEBOOK(kz->notebook), kzweb, kztab);
+		kz_window_set_web_callbacks(kz, kzweb);
 		kz_tab_label_set_history(kztab, child);
 	}
 	g_list_free(childtabs);
 
-	pos = kz_bookmark_get_current(tabs);
+	pos = kz_bookmark_folder_get_current_position(KZ_BOOKMARK_FOLDER(tabs));
 	kz_notebook_set_current_page(KZ_NOTEBOOK(kz->notebook), pos);
-	curembed = KZ_EMBED(KZ_WINDOW_NTH_PAGE(kz, pos));
-	if (curembed)
+	curweb = KZ_WINDOW_NTH_WEB(kz, pos);
+	if (curweb)
 	{
-		kz_window_set_cur_embed_callbacks(kz, curembed);
-		kz_actions_set_sensitive(kz, curembed);
-		kz_actions_set_tab_sensitive(kz, curembed);
+		kz_window_set_cur_web_callbacks(kz, curweb);
+		kz_actions_set_sensitive(kz, curweb);
+		kz_actions_set_tab_sensitive(kz, curweb);
 	}
 }
 
 
 void
-kz_window_close_tab (KzWindow *kz, GtkWidget *widget)
+kz_window_close_tab (KzWindow *kz, KzWeb *web)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(GTK_IS_WIDGET(widget));
+	g_return_if_fail(KZ_IS_WEB(web));
 
-	kz_notebook_close_tab(KZ_NOTEBOOK(kz->notebook), widget);
+	kz_window_unset_cur_web_callbacks(kz, web);
+	kz_window_unset_web_callbacks(kz, web);
+
+	kz_notebook_close_tab(KZ_NOTEBOOK(kz->notebook), web);
 }
 
+static void
+unset_each_web_callbacks (gpointer data, gpointer user_data)
+{
+	kz_window_unset_web_callbacks(KZ_WINDOW(user_data),
+					KZ_WEB(data));
+}
+
+static void
+disconnect_all_web_signals (KzWindow *kz)
+{
+        kz_notebook_foreach_web(KZ_NOTEBOOK(kz->notebook),
+				  unset_each_web_callbacks, kz);
+}
 
 void
 kz_window_close_all_tab(KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
+        disconnect_all_web_signals(kz);
 	kz_notebook_close_all_tab(KZ_NOTEBOOK(kz->notebook));
 }
 
@@ -946,46 +961,46 @@ kz_window_close_all_tab(KzWindow *kz)
 void
 kz_window_move_tab (KzWindow *src_kz, KzWindow *dest_kz, GtkWidget *widget)
 {
-	GtkNotebook *src_notebook, *dest_notebook;
-	GtkWidget *label;
-	KzEmbed *kzembed;
+	KzNotebook *src_notebook, *dest_notebook;
+	KzTabLabel *label;
+	KzWeb *kzweb;
 	KzTabLabel *new_kztab;
 
 	g_return_if_fail(KZ_IS_WINDOW(src_kz));
 	g_return_if_fail(KZ_IS_WINDOW(dest_kz));
-	g_return_if_fail(KZ_IS_EMBED(widget));
+	g_return_if_fail(KZ_IS_WEB(widget));
 
-	src_notebook = GTK_NOTEBOOK(src_kz->notebook);
-	dest_notebook = GTK_NOTEBOOK(dest_kz->notebook);
+	src_notebook = KZ_NOTEBOOK(src_kz->notebook);
+	dest_notebook = KZ_NOTEBOOK(dest_kz->notebook);
 
-	kzembed = KZ_EMBED(widget);
+	kzweb = KZ_WEB(widget);
 
-	label = gtk_notebook_get_tab_label(dest_notebook, widget);
+	label = kz_notebook_get_tab_label(dest_notebook, kzweb);
 
-	/* the kzembed is already the kz's child */
+	/* the kzweb is already the kz's child */
 	if (label) return;
 
-	label = gtk_notebook_get_tab_label(src_notebook, widget);
+	label = kz_notebook_get_tab_label(src_notebook, kzweb);
 	g_return_if_fail(label);
 
 	/* create new tab label */
-	new_kztab = KZ_TAB_LABEL(kz_tab_label_new(dest_kz, kzembed));
+	new_kztab = KZ_TAB_LABEL(kz_tab_label_new(dest_kz, kzweb));
 
 	/* move the page to this window */
-	kz_window_unset_cur_embed_callbacks(src_kz, kzembed);
-	kz_window_unset_embed_callbacks(src_kz, kzembed);
+	kz_window_unset_cur_web_callbacks(src_kz, kzweb);
+	kz_window_unset_web_callbacks(src_kz, kzweb);
 
-	kz_notebook_move_tab(KZ_NOTEBOOK(src_notebook), KZ_NOTEBOOK(dest_notebook), widget);
+	kz_notebook_move_tab(src_notebook, dest_notebook, widget);
 
-	kz_bookmark_prepend(dest_kz->tabs, new_kztab->history);
-	kz_window_set_embed_callbacks(dest_kz, kzembed);
+	kz_bookmark_folder_prepend(dest_kz->tabs, KZ_BOOKMARK(new_kztab->history));
+	kz_window_set_web_callbacks(dest_kz, kzweb);
 
 	g_signal_emit(src_kz,
 		      kz_window_signals[REMOVE_TAB_SIGNAL],
-		      0, kzembed);
+		      0, kzweb);
 	g_signal_emit(dest_kz,
 		      kz_window_signals[APPEND_TAB_SIGNAL],
-		      0, kzembed, NULL);
+		      0, kzweb, NULL);
 }
 
 
@@ -1027,7 +1042,7 @@ delete_event (GtkWidget *widget, GdkEventAny *event)
 	kz_window_store_state(kz);
 
 	KZ_FREEZE_SESSION;
-	kz_notebook_close_all_tab(KZ_NOTEBOOK(kz->notebook));
+	kz_window_close_all_tab(kz);
 	KZ_THAW_SESSION;
 
 	return FALSE;
@@ -1068,7 +1083,7 @@ key_press_event(GtkWidget *widget, GdkEventKey *event)
 
 	kz = KZ_WINDOW(widget);
 
-	/* In an attempt to get the mozembed playing nice with things like emacs keybindings
+	/* In an attempt to get the mozweb playing nice with things like emacs keybindings
 	 * we are passing important events to the focused child widget before letting the window's
 	 * base handler see them. This is *completely against* stated gtk2 policy but the 
 	 * 'correct' behaviour is exceptionally useless. We need to keep an eye out for 
@@ -1116,13 +1131,17 @@ key_press_event(GtkWidget *widget, GdkEventKey *event)
 	return GTK_WIDGET_CLASS(kz_window_parent_class)->key_press_event(widget, event);
 }
 
-
 static void
 destroy (GtkObject *object)
 {
 	KzWindow *kz = KZ_WINDOW(object);
 	KzWindowPrivate *priv = KZ_WINDOW_GET_PRIVATE(kz);
 
+	if (kz->notebook) {
+		disconnect_all_web_signals(kz);
+		kz->notebook = NULL;
+	}
+
 	if (GTK_OBJECT_CLASS(kz_window_parent_class)->destroy)
 		GTK_OBJECT_CLASS(kz_window_parent_class)->destroy(object);
 
@@ -1162,9 +1181,10 @@ destroy (GtkObject *object)
 		gtk_widget_destroy(kz->popup_tablist);
 	kz->popup_tablist = NULL;
 
-	if (kz->tabs)
+	if (kz->tabs) {
 		/* CAUTION: make KzApp sure to remove tabs before freeing it. */
 		g_object_unref(kz->tabs);
+	}
 	kz->tabs = NULL;
 
 	if (kz->closed_tabs)
@@ -1265,7 +1285,7 @@ kz_window_store_state (KzWindow *kz)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 	priv = KZ_WINDOW_GET_PRIVATE(kz);
 
-	kz_profile_set_save_each_time(KZ_GET_GLOBAL_PROFILE, FALSE);
+	kz_profile_set_auto_save(KZ_GET_GLOBAL_PROFILE, FALSE);
 	/*
 	 * window size
 	 */
@@ -1352,7 +1372,7 @@ kz_window_store_state (KzWindow *kz)
 		}
 	}
 
-	kz_profile_set_save_each_time(KZ_GET_GLOBAL_PROFILE, TRUE);
+	kz_profile_set_auto_save(KZ_GET_GLOBAL_PROFILE, TRUE);
 	kz_profile_save(KZ_GET_GLOBAL_PROFILE);
 }
 
@@ -1441,154 +1461,134 @@ kz_window_restore_state (KzWindow *kz)
 
 
 static void
-kz_window_set_cur_embed_callbacks(KzWindow *kz, KzEmbed *kzembed)
+kz_window_set_cur_web_callbacks(KzWindow *kz, KzWeb *kzweb)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
-
-	/*
-	 * key event signal
-	 */
-	g_signal_connect(kzembed,
-			 "kz-dom-key-down",
-			 G_CALLBACK(cb_embed_dom_key_down), kz);
-	g_signal_connect(kzembed,
-			 "kz-dom-key-up",
-			 G_CALLBACK(cb_embed_dom_key_up), kz);
+	g_return_if_fail(KZ_IS_WEB(kzweb));
 
 	/*
 	 * mouse event signal
 	 */
-	g_signal_connect(kzembed,
+	g_signal_connect(kzweb,
 			 "kz-dom-mouse-down",
-			 G_CALLBACK(cb_embed_dom_mouse_down), kz);
-	g_signal_connect(kzembed,
+			 G_CALLBACK(cb_web_dom_mouse_down), kz);
+	g_signal_connect(kzweb,
 			 "kz-dom-mouse-up",
-			 G_CALLBACK(cb_embed_dom_mouse_up), kz);
-	g_signal_connect(kzembed,
+			 G_CALLBACK(cb_web_dom_mouse_up), kz);
+	g_signal_connect(kzweb,
 			 "kz-dom-mouse-over",
-			 G_CALLBACK(cb_embed_dom_mouse_over), kz);
-	g_signal_connect(kzembed, "kz-dom-mouse-click",
-			 G_CALLBACK(cb_embed_dom_mouse_click), kz);
+			 G_CALLBACK(cb_web_dom_mouse_over), kz);
+	g_signal_connect(kzweb, "kz-dom-mouse-click",
+			 G_CALLBACK(cb_web_dom_mouse_click), kz);
 
 	/*
 	 * other callbacks
 	 */
-	g_signal_connect(kzembed, "kz-title",
-			 G_CALLBACK(cb_embed_title_changed), kz);
-	g_signal_connect(kzembed, "kz-location",
-			 G_CALLBACK(cb_embed_location_changed), kz);
-	g_signal_connect(kzembed, "kz-link-message",
-			 G_CALLBACK(cb_embed_link_message), kz);
+	g_signal_connect(kzweb, "kz-title",
+			 G_CALLBACK(cb_web_title_changed), kz);
+	g_signal_connect(kzweb, "kz-location",
+			 G_CALLBACK(cb_web_location_changed), kz);
+	g_signal_connect(kzweb, "kz-link-message",
+			 G_CALLBACK(cb_web_link_message), kz);
 }
 
 
 static void
-kz_window_unset_cur_embed_callbacks(KzWindow *kz, KzEmbed *kzembed)
+kz_window_unset_cur_web_callbacks(KzWindow *kz, KzWeb *kzweb)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
-
-	/*
-	 * key event signal
-	 */
-	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_dom_key_down), kz);
-	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_dom_key_up), kz);
+	g_return_if_fail(KZ_IS_WEB(kzweb));
 
 	/*
 	 * mouse event signal
 	 */
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_dom_mouse_down), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_dom_mouse_down), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_dom_mouse_up), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_dom_mouse_up), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_dom_mouse_over), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_dom_mouse_over), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_dom_mouse_click), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_dom_mouse_click), kz);
 
 	/*
 	 * other callbacks
 	 */
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_title_changed), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_title_changed), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_location_changed), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_location_changed), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_link_message), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_link_message), kz);
 }
 
 
 static void
-kz_window_set_embed_callbacks(KzWindow *kz, KzEmbed *kzembed)
+kz_window_set_web_callbacks(KzWindow *kz, KzWeb *kzweb)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
+	g_return_if_fail(KZ_IS_WEB(kzweb));
 
 	/*
 	 * other callbacks
 	 */
-	g_signal_connect(kzembed, "kz-net-start",
-			 G_CALLBACK(cb_embed_load_started), kz);
-	g_signal_connect(kzembed, "kz-net-stop",
-			 G_CALLBACK(cb_embed_load_finished), kz);
-	g_signal_connect(kzembed, "kz-new-window",
-			 G_CALLBACK(cb_embed_new_window), kz);
-	g_signal_connect(kzembed, "destroy",
-			 G_CALLBACK(cb_embed_close_tab), kz);
+	g_signal_connect(kzweb, "kz-net-start",
+			 G_CALLBACK(cb_web_load_started), kz);
+	g_signal_connect(kzweb, "kz-net-stop",
+			 G_CALLBACK(cb_web_load_finished), kz);
+	g_signal_connect(kzweb, "kz-new-window",
+			 G_CALLBACK(cb_web_new_window), kz);
+	g_signal_connect(kzweb, "destroy",
+			 G_CALLBACK(cb_web_close_tab), kz);
 }
 
 
 static void
-kz_window_unset_embed_callbacks(KzWindow *kz, KzEmbed *kzembed)
+kz_window_unset_web_callbacks(KzWindow *kz, KzWeb *kzweb)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
+	g_return_if_fail(KZ_IS_WEB(kzweb));
 
 	/*
 	 * other callbacks
 	 */
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_load_started), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_load_started), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_load_finished), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_load_finished), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_new_window), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_new_window), kz);
 	g_signal_handlers_disconnect_by_func
-		(kzembed,
-		 G_CALLBACK(cb_embed_close_tab), kz);
+		(kzweb,
+		 G_CALLBACK(cb_web_close_tab), kz);
 }
 
-
 void
 kz_window_load_url (KzWindow *kz, const gchar *url)
 {
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+        KzWeb *web;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
+        web = KZ_WINDOW_CURRENT_WEB(kz);
 	if (kz_notebook_get_n_pages(KZ_NOTEBOOK(kz->notebook)) < 1 ||
-	    !KZ_IS_EMBED(widget))
+	    !web)
 	{
 		kz_window_open_new_tab(kz, url);
 	}
 	else
 	{
-		kz_embed_load_url(KZ_EMBED(widget), url);
+		kz_web_load_uri(web, url);
 	}
 }
 
@@ -1596,39 +1596,36 @@ kz_window_load_url (KzWindow *kz, const gchar *url)
 const gchar *
 kz_window_get_title (KzWindow *kz)
 {
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+        KzWeb *web;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
-	if (KZ_IS_EMBED(widget))
-		return kz_embed_get_title(KZ_EMBED(widget));
-	else
-		return NULL;
+        web = KZ_WINDOW_CURRENT_WEB(kz);
+
+	return web ? kz_web_get_title(web) : NULL;
 }
 
 
 const gchar *
 kz_window_get_uri (KzWindow *kz)
 {
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+        KzWeb *web;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
 
-	if (KZ_IS_EMBED(widget))
-		return kz_embed_get_location(KZ_EMBED(widget));
-	else
-		return NULL;
+        web = KZ_WINDOW_CURRENT_WEB(kz);
+
+	return web ? kz_web_get_location(web) : NULL;
 }
 
 
-GtkWidget *
-kz_window_get_tab_label (KzWindow *kz, GtkWidget *widget)
+KzTabLabel *
+kz_window_get_tab_label (KzWindow *kz, KzWeb *web)
 {
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
-	g_return_val_if_fail(GTK_IS_WIDGET(widget), NULL);
+	g_return_val_if_fail(KZ_IS_WEB(web), NULL);
 
-	return gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-					  widget);
+	return kz_notebook_get_tab_label(KZ_NOTEBOOK(kz->notebook), web);
 }
 
 
@@ -1757,7 +1754,6 @@ cb_bookmark_bars_insert_child (KzBookmark *bookmark,
 			       KzWindow *kz)
 {
 	GtkWidget *bar;
-	GList *children;
 	gint pos;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(child));
@@ -1770,11 +1766,9 @@ cb_bookmark_bars_insert_child (KzBookmark *bookmark,
 	gtk_widget_show(bar);
 
 	/* reorder */
-	children = kz_bookmark_get_children(bookmark);
-	pos = g_list_index(children, child);
+	pos = kz_bookmark_folder_get_child_index(KZ_BOOKMARK_FOLDER(bookmark), child);
 	gtk_box_reorder_child(GTK_BOX(kz->bookmark_bars_area),
 			      bar, pos);
-	g_list_free(children);
 }
 
 
@@ -1838,7 +1832,7 @@ cb_bookmark_bars_reordered (KzBookmark *bookmark, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	children = kz_bookmark_get_children(bookmark);
+	children = kz_bookmark_folder_get_children(KZ_BOOKMARK_FOLDER(bookmark));
 	for (node = children; node; node = g_list_next(node))
 	{
 		GtkWidget *bar;
@@ -1921,19 +1915,19 @@ static void
 cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 			 guint page_num, KzWindow *kz)
 {
-	KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_NTH_PAGE(kz, page_num));
-	KzEmbed *cur = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz));
+	KzWeb *kzweb = KZ_WINDOW_NTH_WEB(kz, page_num);
+	KzWeb *cur = KZ_WINDOW_CURRENT_WEB(kz);
 	const gchar *location;
 	GtkAction *action;
 	KzTabLabel *tab;
 
-	g_return_if_fail(KZ_IS_EMBED(cur));
-	g_return_if_fail(KZ_IS_EMBED(kzembed));
+	g_return_if_fail(KZ_IS_WEB(cur));
+	g_return_if_fail(KZ_IS_WEB(kzweb));
 
-	kz_window_unset_cur_embed_callbacks(kz, cur);
-	kz_window_set_cur_embed_callbacks(kz, kzembed);
+	kz_window_unset_cur_web_callbacks(kz, cur);
+	kz_window_set_cur_web_callbacks(kz, kzweb);
 
-	location = kz_embed_get_location(kzembed);
+	location = kz_web_get_location(kzweb);
 
 	action = gtk_action_group_get_action(kz->actions, "LocationEntry");
 	if (KZ_IS_ENTRY_ACTION(action))
@@ -1960,20 +1954,20 @@ cb_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	if (KZ_IS_ZOOM_ACTION(action))
 	{
 		gint ratio;
-		ratio = kz_embed_zoom_get(kzembed);
+		ratio = kz_web_get_zoom_ratio(kzweb);
 		kz_zoom_action_set_ratio(KZ_ZOOM_ACTION(action), ratio);
 	}
 
-	tab = KZ_TAB_LABEL(kz_window_get_tab_label(kz, GTK_WIDGET(kzembed)));
+	tab = kz_window_get_tab_label(kz, kzweb);
 	g_return_if_fail(tab);
 
-	kz_actions_set_sensitive(kz, kzembed);
-	kz_actions_set_tab_sensitive(kz, kzembed);
+	kz_actions_set_sensitive(kz, kzweb);
+	/* kz_actions_set_tab_sensitive(kz, kzweb); */
 
 	/* set current_position in history */
 	/* FIXME: KzSession should block automatically */
 	if (!kz_session_is_frozen(KZ_SESSION(KZ_GET_CURRENT_SESSION)))
-		kz_bookmark_set_current(kz->tabs, page_num);
+		kz_bookmark_folder_set_current_position(kz->tabs, page_num);
 }
 
 static void
@@ -2034,142 +2028,97 @@ cb_sidebar_unmap (GtkWidget *widget, GtkToggleAction *action)
  *                                                                           *
  *****************************************************************************/
 static void
-cb_embed_title_changed (KzEmbed *embed, KzWindow *kz)
+cb_web_title_changed (KzWeb *web, const gchar *title, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	if (embed == KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)))
-	{
-		const gchar *title
-			= kz_embed_get_title(KZ_EMBED(embed));
-		gtk_window_set_title(GTK_WINDOW(kz), (gchar *) title);
-	}
+	if (web == KZ_WINDOW_CURRENT_WEB(kz))
+		gtk_window_set_title(GTK_WINDOW(kz), title);
 }
 
 
 static void
-cb_embed_location_changed (KzEmbed *embed, KzWindow *kz)
+cb_web_location_changed (KzWeb *web, const gchar *location, KzWindow *kz)
 {
-	const char *newLocation;
-
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	newLocation = kz_embed_get_location(KZ_EMBED(embed));
-	if (newLocation)
-	{
-		if (embed == KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)))
-			kz_window_set_location_entry_text(kz, newLocation);
-	}
-	else
-	{
-		/* g_print("failed to get location!\n"); */
-	}
+	if (location && web == KZ_WINDOW_CURRENT_WEB(kz))
+		kz_window_set_location_entry_text(kz, location);
 
-	kz_actions_set_sensitive(kz, embed);
+	kz_actions_set_sensitive(kz, web);
 }
 
 
 static void
-cb_embed_link_message (KzEmbed *embed, KzWindow *kz)
+cb_web_link_message (KzWeb *web, const gchar *message, KzWindow *kz)
 {
-	gchar *newMessage;
-
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	newMessage = kz_embed_get_link_message(KZ_EMBED(embed));
 	if(kz->statusbar) 
-	{
-		kz_statusbar_set_link_text(KZ_STATUSBAR(kz->statusbar), newMessage);
-	}
-
-	g_free(newMessage);
+		kz_statusbar_set_link_text(KZ_STATUSBAR(kz->statusbar), message);
 }
 
 
 static void
-cb_embed_load_started (KzEmbed *embed, KzWindow *kz)
+cb_web_load_started (KzWeb *web, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kz_actions_set_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	kz_actions_set_sensitive(kz, KZ_WINDOW_CURRENT_WEB(kz));
 
 	kz_feed_info_change_state(KZ_FEED_INFO(kz->feed_info));
 }
 
 
 static void
-cb_embed_load_finished (KzEmbed *embed, KzWindow *kz)
+cb_web_load_finished (KzWeb *web, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kz_actions_set_sensitive(kz, KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(kz)));
+	kz_actions_set_sensitive(kz, KZ_WINDOW_CURRENT_WEB(kz));
 
 	kz_feed_info_change_state(KZ_FEED_INFO(kz->feed_info));
 }
 
 
 static void
-cb_embed_new_window (KzEmbed *embed, KzEmbed **newEmbed,
-		     KzWindow *kz)
+cb_web_new_window (KzWeb *web, KzWeb **newWeb, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	*newEmbed = KZ_EMBED(kz_window_open_new_tab(kz, NULL));
-	gtk_widget_show(GTK_WIDGET(*newEmbed));
+	*newWeb = KZ_WEB(kz_window_open_new_tab(kz, NULL));
+	gtk_widget_show(GTK_WIDGET(*newWeb));
 }
 
 
 static void
-cb_embed_close_tab (GtkObject *obj, KzWindow *kz)
+cb_web_close_tab (GtkObject *obj, KzWindow *kz)
 {
-	KzEmbed *kzembed;
+	KzWeb *kzweb;
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kzembed = KZ_EMBED(obj);
+	kzweb = KZ_WEB(obj);
 
-	kz_window_unset_cur_embed_callbacks(kz, kzembed);
-	kz_window_unset_embed_callbacks(kz, kzembed);
+	kz_window_unset_cur_web_callbacks(kz, kzweb);
+	kz_window_unset_web_callbacks(kz, kzweb);
 
-	kz_notebook_close_tab(KZ_NOTEBOOK(kz->notebook), GTK_WIDGET(obj));
+	kz_notebook_close_tab(KZ_NOTEBOOK(kz->notebook), kzweb);
 	kz_statusbar_set_link_text(KZ_STATUSBAR(kz->statusbar), NULL);
 
 	g_signal_emit(kz, kz_window_signals[REMOVE_TAB_SIGNAL],
-		      0, kzembed);
+		      0, kzweb);
 }
 
 
-static gint
-cb_embed_dom_key_down (KzEmbed *embed, KzEmbedEventKey *event, KzWindow *kz)
-{
-
-	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
-
-	kz_actions_set_sensitive(kz, embed);
-
-	return FALSE;
-}
-
-
-static gint
-cb_embed_dom_key_up (KzEmbed *embed, KzEmbedEventKey *event, KzWindow *kz)
-{
-	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
-
-	kz_actions_set_sensitive(kz, embed);
-
-	return FALSE;
-}
-
-
-static gint
-cb_embed_dom_mouse_click (KzEmbed *embed, KzEmbedEventMouse *event,
+static gboolean
+cb_web_dom_mouse_click (KzWeb *web, KzEmbedEventMouse *event,
 			  KzWindow *kz)
 {
 	gint button;
      	glong type;
 	KzTabLabel *kztab = NULL;
-	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
+	GtkWidget *widget;
 
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
@@ -2181,24 +2130,24 @@ cb_embed_dom_mouse_click (KzEmbed *embed, KzEmbedEventMouse *event,
 		if ((type & KZ_CONTEXT_LINK) && event->cinfo.link)
 		{
 			const gchar *location;
+			gint current_page;
 
-			kztab = KZ_TAB_LABEL
-				(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-							    widget));
+			current_page = kz_notebook_get_current_page(KZ_NOTEBOOK(kz->notebook));
+			kztab = kz_notebook_get_nth_tab_label(KZ_NOTEBOOK(kz->notebook),
+							      current_page);
 
 			/* with Shift key or in locked tab, open new tab */
 			if ((event->modifier & KZ_SHIFT_KEY) ||
 			    (kztab && kz_tab_label_get_lock(kztab)))
 			{
 				kz_window_open_new_tab_with_parent
-					(kz, event->cinfo.link, GTK_WIDGET(embed));
+					(kz, event->cinfo.link, GTK_WIDGET(web));
 				return TRUE;
 			}
-			location = kz_embed_get_location(KZ_EMBED(embed));
+			location = kz_web_get_location(KZ_WEB(web));
 			if (!strncmp(location, "history-search:", 15))
 			{
-				kz_embed_load_url(KZ_EMBED(embed), 
-						  event->cinfo.link);
+				kz_web_load_uri(KZ_WEB(web), event->cinfo.link);
 			}
 		}
 		break;
@@ -2209,7 +2158,7 @@ cb_embed_dom_mouse_click (KzEmbed *embed, KzEmbedEventMouse *event,
 			KZ_CONF_GET("Tab", "focus_mid_click_link", focus_mid_click_link, BOOL);
 
 			widget = kz_window_open_new_tab_with_parent(kz, event->cinfo.link,
-							   	    GTK_WIDGET(embed));
+							   	    GTK_WIDGET(web));
 
 			if(((event->modifier & KZ_SHIFT_KEY) != 0) ^ focus_mid_click_link)
 			{
@@ -2232,8 +2181,8 @@ cb_embed_dom_mouse_click (KzEmbed *embed, KzEmbedEventMouse *event,
 }
 
 
-static gint
-cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
+static gboolean
+cb_web_dom_mouse_down (KzWeb *kzweb, KzEmbedEventMouse *event,
 			 KzWindow *kz)
 {
 	gint button;
@@ -2315,17 +2264,17 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 		}
 		/* start gesture */
 
-		gtk_widget_get_pointer(GTK_WIDGET(kzembed), &x, &y);
+		gtk_widget_get_pointer(GTK_WIDGET(kzweb), &x, &y);
 
 		priv->start_x = x;
 		priv->start_y = y;
 
-		gdk_window_get_root_origin(GTK_WIDGET(kzembed)->window,
+		gdk_window_get_root_origin(GTK_WIDGET(kzweb)->window,
 					    &win_x, &win_y);
-		gdk_window_get_position(GTK_WIDGET(kzembed)->window,
+		gdk_window_get_position(GTK_WIDGET(kzweb)->window,
 					 &win_x_pos, &win_y_pos);
 
-		gtk_widget_get_pointer(GTK_WIDGET(kzembed),
+		gtk_widget_get_pointer(GTK_WIDGET(kzweb),
 					&x, &y);
 		kz_gesture_start(priv->gesture, 0, x, y);
 
@@ -2343,25 +2292,25 @@ cb_embed_dom_mouse_down (KzEmbed *kzembed, KzEmbedEventMouse *event,
 		break;
 	}
 
-	kz_actions_set_selection_sensitive(kz, kzembed);
+	kz_actions_set_selection_sensitive(kz, kzweb);
 
 	return FALSE;
 }
 
 
-static gint
-cb_embed_dom_mouse_up (KzEmbed *embed, KzEmbedEventMouse *event,
+static gboolean
+cb_web_dom_mouse_up (KzWeb *web, KzEmbedEventMouse *event,
 		       KzWindow *kz)
 {
 	g_return_val_if_fail(KZ_IS_WINDOW(kz), FALSE);
 
-	kz_actions_set_selection_sensitive(kz, embed);
+	kz_actions_set_selection_sensitive(kz, web);
 
 	return FALSE;
 }
 
-static gint
-cb_embed_dom_mouse_over (KzEmbed *embed, KzEmbedEventMouse *event,
+static gboolean
+cb_web_dom_mouse_over (KzWeb *web, KzEmbedEventMouse *event,
 		         KzWindow *kz)
 {
 	static glong previous_event_context = KZ_CONTEXT_NONE;
@@ -2376,11 +2325,10 @@ cb_embed_dom_mouse_over (KzEmbed *embed, KzEmbedEventMouse *event,
 	type = event->cinfo.context;
 	if ((type & KZ_CONTEXT_LINK) && event->cinfo.link)
 	{
-		GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
-		GdkDisplay *display = gtk_widget_get_display(widget);
+		KzWeb *web = KZ_WINDOW_CURRENT_WEB(kz);
+		GdkDisplay *display = gtk_widget_get_display(GTK_WIDGET(web));
 		gint x, y;
 
-		display = gtk_widget_get_display(widget);
 		gdk_display_get_pointer(display, NULL, &x, &y, NULL);
 
 		kz_popup_preview_start(kz->popup, event->cinfo.link, event->cinfo.img, x, y);
@@ -2391,7 +2339,7 @@ cb_embed_dom_mouse_over (KzEmbed *embed, KzEmbedEventMouse *event,
 	}
 
 	previous_event_context = type;
-	kz_actions_set_selection_sensitive(kz, embed);
+	kz_actions_set_selection_sensitive(kz, web);
 
 	return FALSE;
 }
@@ -2404,15 +2352,15 @@ motion_notify_event (GtkWidget *widget,
 	gint x, y;
 	/* GdkModifierType state; */
 	KzWindowPrivate *priv;
-	KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_CURRENT_PAGE(KZ_WINDOW(widget)));
+	KzWeb *kzweb =KZ_WINDOW_CURRENT_WEB(KZ_WINDOW(widget));
 
-	if (!kzembed)
+	if (!kzweb)
 		return FALSE;
 
 	kz = KZ_WINDOW(widget);
 	priv = KZ_WINDOW_GET_PRIVATE(kz);
 
-	gtk_widget_get_pointer(GTK_WIDGET(kzembed), &x, &y);
+	gtk_widget_get_pointer(GTK_WIDGET(kzweb), &x, &y);
 
 	if (abs(x - priv->start_x) > 2 ||
 	    abs(y - priv->start_y) > 2)
@@ -2425,7 +2373,7 @@ motion_notify_event (GtkWidget *widget,
 		kz_gesture_update_position(priv->gesture, x, y);
 	}
 
-	kz_actions_set_selection_sensitive(kz, kzembed);
+	kz_actions_set_selection_sensitive(kz, kzweb);
 
 	return FALSE;
 }
@@ -2515,9 +2463,8 @@ focus_out_event (GtkWidget *widget,
 		{
 			kz_gesture_cancel(priv->gesture);
 			if (gdk_pointer_is_grabbed())
-			{
 				gdk_pointer_ungrab(gtk_get_current_event_time());
-			}
+
 			kz_statusbar_set_gesture_text(KZ_STATUSBAR(KZ_WINDOW(widget)->statusbar), NULL);
 		}
 	}
@@ -2525,21 +2472,22 @@ focus_out_event (GtkWidget *widget,
 }
 
 void
-kz_window_append_closed_tab (KzWindow *kz, KzBookmark *bookmark)
+kz_window_append_closed_tab (KzWindow *kz, KzBookmarkFolder *bookmark)
 {
-	GList *children;
-
-	kz_bookmark_prepend(kz->closed_tabs, bookmark);
+	KzBookmark *last;
+	kz_bookmark_folder_prepend(kz->closed_tabs, KZ_BOOKMARK(bookmark));
 
 	/* removed the oldest one */
-	children = kz_bookmark_get_children(kz->closed_tabs);
-	if (children)
-	{
-		KzBookmark *last;
-		last = g_list_nth_data(children, MAX_CLOSED_TABS);
-		if (last)
-			kz_bookmark_remove(kz->closed_tabs, last);
-		g_list_free(children);
-	}
+	last = kz_bookmark_folder_get_nth_child(kz->closed_tabs, MAX_CLOSED_TABS);
+	if (last)
+		kz_bookmark_folder_remove(kz->closed_tabs, last);
+}
+
+void
+kz_window_switch_layout_engine (KzWindow *kz, const gchar *engine_name)
+{
+	KzWeb *web = KZ_WINDOW_CURRENT_WEB(kz);
+
+	kz_web_change_engine(KZ_WEB(web), engine_name);
 }
 
