@@ -24,11 +24,13 @@
 
 #include <gtk/gtk.h>
 
+#include "kz-objects.h"
 #include "kz-gesture.h"
 #include "kz-embed-event.h"
-#include "kz-embed.h"
+#include "kz-web.h"
 #include "kz-popup-preview.h"
-#include "kz-bookmark.h"
+#include "kz-bookmark-folder.h"
+#include "kz-notebook.h"
 
 G_BEGIN_DECLS
 
@@ -39,7 +41,6 @@ G_BEGIN_DECLS
 #define KZ_IS_WINDOW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), KZ_TYPE_WINDOW))
 #define KZ_WINDOW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), KZ_TYPE_WINDOW, KzWindowClass))
 
-typedef struct _KzWindow	KzWindow;
 typedef struct _KzWindowClass	KzWindowClass;
 
 struct _KzWindow
@@ -71,8 +72,8 @@ struct _KzWindow
 	GtkUIManager   *menu_merge;
 
 	/* bookmarks */
-	KzBookmark *tabs;
-	KzBookmark *closed_tabs;
+	KzBookmarkFolder *tabs;
+	KzBookmarkFolder *closed_tabs;
 	KzBookmark *history_search;
 
 	/* handle for favicon */
@@ -103,7 +104,7 @@ GtkWidget        *kz_window_open_new_tab_with_parent (KzWindow    *kz,
 						      const gchar *url,
 						      GtkWidget   *parent);
 void              kz_window_close_tab                (KzWindow    *kz,
-						      GtkWidget   *widget);
+						      KzWeb     *web);
 void              kz_window_close_all_tab            (KzWindow    *kz);
 void		  kz_window_move_tab 		     (KzWindow    *src_kz,
 						      KzWindow    *dest_kz,
@@ -122,8 +123,8 @@ void              kz_window_update_gesture_items     (KzWindow    *kz);
 const gchar      *kz_window_get_title                (KzWindow    *kz);
 const gchar      *kz_window_get_uri                  (KzWindow    *kz);
 
-GtkWidget        *kz_window_get_tab_label            (KzWindow    *kz,
-						      GtkWidget   *widget);
+KzTabLabel       *kz_window_get_tab_label            (KzWindow    *kz,
+						      KzWeb     *web);
 
 GNode            *kz_window_get_tree                 (KzWindow    *kz);
 
@@ -140,12 +141,14 @@ const KzEmbedEventMouse *
 		  kz_window_get_mouse_event_info     (KzWindow    *kz);
 
 void		  kz_window_append_closed_tab	     (KzWindow    *kz,
-						      KzBookmark  *bookmark);
+						      KzBookmarkFolder  *tab);
+void		  kz_window_switch_layout_engine     (KzWindow    *kz,
+						      const gchar *engine_name);
 
-#define KZ_WINDOW_NTH_PAGE(kz, n) \
-	(KZ_IS_WINDOW(kz) ? gtk_notebook_get_nth_page(GTK_NOTEBOOK(KZ_WINDOW(kz)->notebook),n) : NULL)
-#define KZ_WINDOW_CURRENT_PAGE(kz) \
-	(KZ_IS_WINDOW(kz) ? KZ_WINDOW_NTH_PAGE(kz, gtk_notebook_get_current_page(GTK_NOTEBOOK(kz->notebook))) : NULL)
+#define KZ_WINDOW_NTH_WEB(kz, n) \
+	(KZ_IS_WINDOW(kz) ? kz_notebook_get_nth_web(KZ_NOTEBOOK(KZ_WINDOW(kz)->notebook),n) : NULL)
+#define KZ_WINDOW_CURRENT_WEB(kz) \
+	(KZ_IS_WINDOW(kz) ? KZ_WINDOW_NTH_WEB(kz, kz_notebook_get_current_page(KZ_NOTEBOOK(kz->notebook))) : NULL)
 
 #define KZ_WINDOW_SET_SENSITIVE(kz, action_name, sensitive)		       \
 {									       \
@@ -153,7 +156,7 @@ void		  kz_window_append_closed_tab	     (KzWindow    *kz,
 		GtkAction *action;					       \
 		action = gtk_action_group_get_action(kz->actions,	       \
 						     action_name);	       \
-		if (action) g_object_set(action, "sensitive", sensitive, NULL);\
+		if (action) gtk_action_set_sensitive(action, sensitive);       \
 	}								       \
 }
 
@@ -163,7 +166,7 @@ void		  kz_window_append_closed_tab	     (KzWindow    *kz,
 		GtkAction *action;					       \
 		action = gtk_action_group_get_action(kz->actions, 	       \
 						     action_name);	       \
-		if (action) g_object_set(action, "visible", visible, NULL);    \
+		if (action) gtk_action_set_visible(action, visible);           \
 	}								       \
 }
 
