@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xml.c 3484 2008-03-14 03:21:32Z ikezoe $
+ *  $Id: kz-xml.c 3840 2009-09-11 03:37:18Z ikezoe $
  */
 
 #include "kz-xml.h"
@@ -35,6 +35,7 @@ struct _KzXMLPrivate
 #define KZ_XML_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_XML, KzXMLPrivate))
 
 static void dispose      (GObject    *object);
+static void finalize     (GObject    *object);
 
 static void kz_xml_attr_free    (KzXMLAttr *attr);
 gchar *get_cdata_as_plaintext(gpointer content);
@@ -47,10 +48,11 @@ kz_xml_class_init (KzXMLClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->dispose  = dispose;
-	g_type_class_add_private (object_class, sizeof(KzXMLPrivate));
+	object_class->finalize = finalize;
+	g_type_class_add_private(object_class, sizeof(KzXMLPrivate));
 }
 
 
@@ -72,15 +74,6 @@ dispose (GObject *object)
 {
 	KzXML *xml = KZ_XML(object);
 
-	g_free(xml->file);
-	xml->file = NULL;
-
-	g_free(xml->dtd);
-	xml->dtd = NULL;
-
-	g_free(xml->encoding);
-	xml->encoding = NULL;
-
 	if (xml->root)
 		kz_xml_node_unref(xml->root);
 	xml->root = NULL;
@@ -90,11 +83,24 @@ dispose (GObject *object)
 }
 
 
+static void
+finalize (GObject *object)
+{
+	KzXML *xml = KZ_XML(object);
+
+	g_free(xml->file);
+	g_free(xml->dtd);
+	g_free(xml->encoding);
+
+	if (G_OBJECT_CLASS (kz_xml_parent_class)->finalize)
+		G_OBJECT_CLASS (kz_xml_parent_class)->finalize(object);
+}
+
+
 KzXML *
 kz_xml_new (void)
 {
-	KzXML *xml = g_object_new(KZ_TYPE_XML, NULL);
-	return xml;
+	return g_object_new(KZ_TYPE_XML, NULL);
 }
 
 
@@ -252,7 +258,7 @@ kz_xml_load_xml (KzXML *xml, const gchar *buffer, guint length)
 	g_return_val_if_fail(KZ_IS_XML(xml), FALSE);
 	if (buffer == NULL) return FALSE;
 
-	priv = KZ_XML_GET_PRIVATE (xml);
+	priv = KZ_XML_GET_PRIVATE(xml);
 	context = priv->context;
 	if (!context)
 	{
@@ -278,7 +284,7 @@ kz_xml_load_xml (KzXML *xml, const gchar *buffer, guint length)
 		g_warning("XML parse error!: %s", error->message);
 		g_error_free(error);
 	}
-	g_markup_parse_context_free (context);
+	g_markup_parse_context_free(context);
 	priv->context = NULL;
 
 	return ret;
@@ -293,16 +299,15 @@ kz_xml_load (KzXML *xml, const gchar *filename)
 	gboolean ret;
 	GError *error = NULL;
 
-	ret = g_file_get_contents (filename, &buffer, &length, &error);
+	ret = g_file_get_contents(filename, &buffer, &length, &error);
 
 	if (error)
 	{
-		g_warning(error->message);
+		g_warning("%s", error->message);
 		g_error_free(error);
+		return FALSE;
 	}
 
-	if (!ret) return FALSE;
-
 	ret = kz_xml_load_xml(xml, buffer, length);
 	g_free(buffer);
 
@@ -313,30 +318,17 @@ kz_xml_load (KzXML *xml, const gchar *filename)
 gboolean
 kz_xml_save (KzXML *xml, const gchar *filename)
 {
-	FILE *fp;
 	const gchar *file = filename ? filename : xml->file;
 	gchar *str;
-	size_t len;
+	gboolean success;
 
 	g_return_val_if_fail(file && *file, FALSE);
 
-	fp = fopen(file, "w");
-	if (!fp) return FALSE;
-
 	str = kz_xml_node_to_xml(xml->root);
-
-	if (!str || !*str)
-	{
-		g_free(str);
-		fclose(fp);
-		return FALSE;
-	}
-
-	len = fwrite(str, strlen(str), 1, fp);
-	fclose(fp);
+	success = g_file_set_contents(file, str, -1, NULL);
 	g_free(str);
 
-	return TRUE;
+	return success;
 }
 
 
@@ -447,6 +439,18 @@ kz_xml_node_unref(KzXMLNode *node)
 }
 
 
+static KzXMLAttr *
+kz_xml_attr_new (const gchar *name, const gchar *value)
+{
+	KzXMLAttr *attr;
+
+	attr = g_slice_new0(KzXMLAttr);
+	attr->name  = g_strdup(name);
+	attr->value = g_strdup(value);
+
+	return attr;
+}
+
 static void
 kz_xml_attr_free (KzXMLAttr *attr)
 {
@@ -480,8 +484,8 @@ kz_xml_node_append_xml_string (KzXMLNode *node, GString *gstr)
 		}
 
 		if (!node->children)
-			g_string_append(gstr, "/");
-		g_string_append(gstr, ">");
+			g_string_append_c(gstr, '/');
+		g_string_append_c(gstr, '>');
 	}
 
 	if (element || node->type == KZ_XML_NODE_DOC_ROOT)
@@ -610,12 +614,36 @@ kz_xml_node_name_is (KzXMLNode *node, const gchar *name)
 	g_return_val_if_fail(element, FALSE);
 	g_return_val_if_fail(element->name, FALSE);
 
-	if (!strcmp(element->name, name))
-		return TRUE;
-	else
-		return FALSE;
+	return (!strcmp(element->name, name));
 }
 
+static gint
+compare_attr_name (gconstpointer a, gconstpointer b)
+{
+	const KzXMLAttr *attr_a = a;
+	const gchar *attr_name = b;
+
+	return g_strcmp0(attr_a->name, attr_name);
+}
+
+
+gboolean
+kz_xml_node_has_attr (KzXMLNode *node, const gchar *attr_name)
+{
+	KzXMLElement *element;
+	GList *list;
+
+	g_return_val_if_fail(node, FALSE);
+	g_return_val_if_fail(node->type == KZ_XML_NODE_ELEMENT, FALSE);
+	g_return_val_if_fail(attr_name, FALSE);
+
+	element = node->content;
+	g_return_val_if_fail(element, FALSE);
+
+        list = g_list_find_custom(element->attrs, attr_name,
+                                  (GCompareFunc)compare_attr_name);
+        return list ? TRUE : FALSE;
+}
 
 const gchar *
 kz_xml_node_get_attr (KzXMLNode *node, const gchar *attr_name)
@@ -623,22 +651,16 @@ kz_xml_node_get_attr (KzXMLNode *node, const gchar *attr_name)
 	KzXMLElement *element;
 	GList *list;
 
-	g_return_val_if_fail(node, NULL);
-	g_return_val_if_fail(node->type == KZ_XML_NODE_ELEMENT, NULL);
-	g_return_val_if_fail(attr_name, NULL);
+	g_return_val_if_fail(node, FALSE);
+	g_return_val_if_fail(node->type == KZ_XML_NODE_ELEMENT, FALSE);
+	g_return_val_if_fail(attr_name, FALSE);
 
 	element = node->content;
-	g_return_val_if_fail(element, NULL);
+	g_return_val_if_fail(element, FALSE);
 
-	for (list = element->attrs; list; list = g_list_next(list))
-	{
-		KzXMLAttr *attr = list->data;
-
-		if (!strcmp(attr_name, attr->name))
-			return attr->value;
-	}
-
-	return NULL;
+        list = g_list_find_custom(element->attrs, attr_name,
+                                  (GCompareFunc)compare_attr_name);
+        return list ? ((KzXMLAttr*)list->data)->value : NULL;
 }
 
 
@@ -681,9 +703,7 @@ kz_xml_node_set_attr (KzXMLNode *node, const gchar *name, const gchar *value)
 
 	if (!found)
 	{
-		attr = g_slice_new0(KzXMLAttr);
-		attr->name  = g_strdup(name);
-		attr->value = g_strdup(value);
+		attr = kz_xml_attr_new(name, value);
 		element->attrs = g_list_append(element->attrs, attr);
 	}
 }
@@ -739,10 +759,7 @@ kz_xml_node_prev (KzXMLNode *node)
 	if (!list) return NULL;
 
 	list = g_list_previous(list);
-	if (list)
-		return list->data;
-	else
-		return NULL;
+	return list ? list->data : NULL;
 }
 
 
@@ -760,10 +777,7 @@ kz_xml_node_first_child (KzXMLNode *node)
 {
 	g_return_val_if_fail(node, NULL);
 
-	if (node->children)
-		return node->children->data;
-	else
-		return NULL;
+	return node->children ? node->children->data : NULL;
 }
 
 
@@ -776,10 +790,7 @@ kz_xml_node_last_child (KzXMLNode *node)
 
 	list = g_list_last(node->children);
 
-	if (list)
-		return list->data;
-	else
-		return NULL;
+	return list ? list->data : NULL;
 }
 
 
@@ -831,10 +842,7 @@ kz_xml_node_is_element (KzXMLNode *node)
 {
 	g_return_val_if_fail(node, FALSE);
 
-	if (node->type == KZ_XML_NODE_ELEMENT)
-		return TRUE;
-	else
-		return FALSE;
+	return (node->type == KZ_XML_NODE_ELEMENT);
 }
 
 
@@ -911,10 +919,7 @@ kz_xml_node_is_text (KzXMLNode *node)
 {
 	g_return_val_if_fail(node, FALSE);
 
-	if (node->type == KZ_XML_NODE_TEXT)
-		return TRUE;
-
-	return FALSE;
+	return (node->type == KZ_XML_NODE_TEXT);
 }
 
 
@@ -926,8 +931,6 @@ kz_xml_text_node_replace_text (KzXMLNode *node, const gchar *text)
 
 	g_free(node->content);
 	node->content = g_strdup(text);
-
-	return;
 }
 
 
@@ -1061,7 +1064,7 @@ get_cdata_as_plaintext(gpointer content)
 	str = (char*)content;
 
 	i = j = strlen("<![CDATA[");
-	while(str[i] != '\0')
+	while (str[i] != '\0')
 	{
 		if(str[i] == ']' && str[i+3] == '\0')
 		{
@@ -1073,8 +1076,8 @@ get_cdata_as_plaintext(gpointer content)
 	cdata_text = g_strndup(str+j, i-j);
 	plain_text = remove_tag((const gchar*)cdata_text, (gsize)strlen(cdata_text));
 
-	if(cdata_text)
-		g_free(cdata_text);
+	g_free(cdata_text);
 
 	return plain_text;
 }
+
