@@ -28,6 +28,7 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <dbus/dbus-glib.h>
 
 #ifndef G_OS_WIN32
 #include <signal.h>
@@ -115,17 +116,16 @@ parse_error_cb (GOptionContext *context,
 		 gpointer	  data,
 		 GError        **error)
 {
-	fprintf (stderr, "%s\n", (*error)->message);
-	g_error_free (*error);
-	return;
+	fprintf(stderr, "%s\n", (*error)->message);
+	g_error_free(*error);
 }
 
 static void
 free_goption()
 {
-	g_free (geometry_string);
-	g_strfreev (action_array);
-	g_option_context_free (context);
+	g_free(geometry_string);
+	g_strfreev(action_array);
+	g_option_context_free(context);
 }
 
 /* XXX: This function has side effect! */
@@ -227,7 +227,6 @@ activate_action(const gchar *text)
 	KzWindow *kz;
 	KzTabLabel *kztab;
 	const GList *window_list;
-	GtkWidget *widget;
 	GtkAction *action;
 	gchar *action_name;
 	gint action_type;
@@ -297,15 +296,11 @@ activate_action(const gchar *text)
 	{
 	 case TAB_ACTION:
 		if (tab_id == -1)
-			widget = KZ_WINDOW_CURRENT_PAGE(kz);
+			kztab = kz_notebook_get_nth_tab_label(KZ_NOTEBOOK(kz->notebook), tab_id);
 		else
-			widget = KZ_WINDOW_NTH_PAGE(kz, tab_id - 1);
-		if (!widget)
-			widget = KZ_WINDOW_CURRENT_PAGE(kz);
-		if (!widget)
+			kztab = kz_notebook_get_nth_tab_label(KZ_NOTEBOOK(kz->notebook), tab_id);
+		if (!kztab)
 			break;
-		kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(kz->notebook),
-								widget));
 
 		action = gtk_action_group_get_action(kz->tabpop_actions,
 						     action_name);
@@ -446,6 +441,9 @@ main (int argc, char *argv[])
 	GError *error = NULL;
 	gboolean restore = FALSE;
 	KzApp *kz_app;
+#ifndef G_OS_WIN32
+	struct sigaction act;
+#endif
 
         /* initialize */
 #ifdef G_OS_WIN32
@@ -463,6 +461,7 @@ main (int argc, char *argv[])
 	textdomain(GETTEXT_PACKAGE);
 
 	if (!g_thread_supported()) g_thread_init(NULL);
+	dbus_g_thread_init();
 
         /* initialize goption */
 	context = g_option_context_new (parameter_string);
@@ -508,7 +507,6 @@ main (int argc, char *argv[])
 	/* signal handler for safe termination */
 	/* TODO: need win32 version? */
 #ifndef G_OS_WIN32
-	struct sigaction act;
 	memset(&act, 0, sizeof(struct sigaction));
 	act.sa_handler = kz_sighandle_exit;
 	sigaction(SIGINT,	&act, NULL);
@@ -538,7 +536,7 @@ main (int argc, char *argv[])
 
 	/* process arguments */
 	process_args(argc_copy, argv_copy);
-	g_strfreev (argv_copy);
+	g_strfreev(argv_copy);
 
 	/* event loop */
 	gtk_main();
