@@ -29,7 +29,6 @@
 
 #include "kz-io.h"
 #include "kz-http.h"
-#include "kz-file.h"
 #include "kz-marshalers.h"
 
 #define BUFFER_SIZE 1024
@@ -134,7 +133,7 @@ kz_io_class_init (KzIOClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = (GObjectClass *) klass;
+	object_class = G_OBJECT_CLASS(klass);
 
 	object_class->dispose      = dispose;
 	object_class->set_property = set_property;
@@ -362,14 +361,6 @@ kz_io_new (const gchar *uri)
 	{
 		/* io = KZ_IO(kz_ftp_new(uri)); */
 	}
-	else if (g_str_has_prefix(uri, "file://"))
-	{
-		io = KZ_IO(kz_file_new(uri + 7));
-	}	    
-	else
-	{
-		io = KZ_IO(kz_file_new(uri));
-	}	    
 
 	return io;
 }
