@@ -38,11 +38,11 @@ static void       prefs_bookmark_response        (GtkWidget *widget,
 
 static KzPrefsWinPageEntry prefs_entry =
 {
-	path:            N_("/Bookmark"),
-	priority_hint:   0,
-	ui_level:        ~KZ_UI_LEVEL_BEGINNER,
-	create:          prefs_bookmark_create,
-	response:        prefs_bookmark_response,
+	/* .path          = */ N_("/Bookmark"),
+	/* .priority_hint = */ 0,
+	/* .ui_level      = */ ~KZ_UI_LEVEL_BEGINNER,
+	/* .create        = */ prefs_bookmark_create,
+	/* .response      = */ prefs_bookmark_response,
 };
 
 
@@ -51,6 +51,7 @@ typedef struct _KzPrefsBookmark
 	GtkWidget     *main_vbox;
 	GtkWidget     *bookmark_cutoff_strnum;
 	GtkWidget     *keep_bookmark_menu;
+	GtkWidget     *always_open_new;
 	GtkWidget     *confirm_open_bookmarks;
 	gboolean       changed;
 } KzPrefsBookmark;
@@ -79,7 +80,7 @@ prefs_bookmark_create (void)
 	GtkWidget *label, *check, *spin;
 	GtkAdjustment *adj;
 	gint bookmark_max_strnum;
-	gboolean keep = FALSE, confirm = TRUE;
+	gboolean keep = FALSE, open_new = FALSE, confirm = TRUE;
 
 	main_vbox = gtk_vbox_new(FALSE, 0);
 	prefsui->main_vbox = main_vbox;
@@ -146,6 +147,15 @@ prefs_bookmark_create (void)
 			 G_CALLBACK(cb_changed), prefsui);
 	gtk_widget_show(check);
 
+	check = gtk_check_button_new_with_mnemonic(_("Always open in new tab"));
+	prefsui->always_open_new = check;
+	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
+	KZ_CONF_GET("Global", "always_open_new", open_new, BOOL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), open_new);
+	g_signal_connect(check, "toggled",
+			 G_CALLBACK(cb_changed), prefsui);
+	gtk_widget_show(check);
+
 	check = gtk_check_button_new_with_mnemonic(_("_Confirmation of opening all bookmarks in bookmark folder"));
 	prefsui->confirm_open_bookmarks = check;
 	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
@@ -188,6 +198,10 @@ prefs_bookmark_response (GtkWidget *widget, gint response)
 			(GTK_TOGGLE_BUTTON(prefsui->keep_bookmark_menu));
 		KZ_CONF_SET("Global", "keep_bookmark_menu", keep, BOOL);
 
+		keep = gtk_toggle_button_get_active
+			(GTK_TOGGLE_BUTTON(prefsui->always_open_new));
+		KZ_CONF_SET("Global", "always_open_new", keep, BOOL);
+		
 		keep = gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON(prefsui->confirm_open_bookmarks));
 		KZ_CONF_SET("Global", "confirm_open_bookmarks", keep, BOOL);
