@@ -21,11 +21,11 @@
 #ifndef __PREFS_FONT_H__
 #define __PREFS_FONT_H__
 
-#include "kz-prefswin.h"
+#include "kz-prefs-win.h"
 
 G_BEGIN_DECLS
 
-KzPrefsWinPageEntry *prefs_font_get_entry (void);
+KzPrefsWinPageEntry *prefs_font_get_entry (gint idx);
 
 G_END_DECLS
 
