@@ -59,11 +59,11 @@ static void       prefs_gesture_response (GtkWidget *widget,
 
 static KzPrefsWinPageEntry prefs_entry =
 {
-	path:            N_("/Gesture"),
-	priority_hint:   0,
-	ui_level:        ~KZ_UI_LEVEL_BEGINNER,
-	create:          prefs_gesture_create,
-	response:        prefs_gesture_response,
+	/* .path          = */ N_("/Gesture"),
+	/* .priority_hint = */ 0,
+	/* .ui_level      = */ ~KZ_UI_LEVEL_BEGINNER,
+	/* .create        = */ prefs_gesture_create,
+	/* .response      = */ prefs_gesture_response,
 };
 
 
@@ -726,7 +726,7 @@ prefs_gesture_create (void)
 			 G_CALLBACK(cb_selection_changed), self);
 
 	renderer = gtk_cell_renderer_text_new();
-	g_object_set(G_OBJECT(renderer), "xalign", 0.0, NULL);
+	g_object_set(renderer, "xalign", 0.0, NULL);
 	gtk_tree_view_insert_column_with_attributes(treeview, -1,
 						    _("Action"), renderer,
 						    "text", COLUMN_ACTION,
@@ -738,7 +738,7 @@ prefs_gesture_create (void)
 	gtk_tree_view_column_set_resizable(col, TRUE);
 
 	renderer = gtk_cell_renderer_text_new();
-	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
+	g_object_set(renderer, "xalign", 0.0, NULL);
 	gtk_tree_view_insert_column_with_attributes(treeview, -1,
 						    _("Gesture"), renderer,
 						    "text", COLUMN_GESTURE,
