@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs_key_accel.c 2863 2007-03-14 06:43:16Z ikezoe $
+ *  $Id: prefs_key_accel.c 3531 2009-01-21 04:32:05Z ikezoe $
  */
 
 /*
@@ -490,7 +490,7 @@ prefs_key_accel_create (void)
 			 G_CALLBACK(accel_path_selection_changed), self);
 
 	renderer = gtk_cell_renderer_text_new();
-	g_object_set(G_OBJECT(renderer), "xalign", 0.0, NULL);
+	g_object_set(renderer, "xalign", 0.0, NULL);
 	gtk_tree_view_insert_column_with_attributes(treeview, -1,
 						    _("Path"), renderer,
 						    "text", COLUMN_LABEL,
@@ -502,7 +502,7 @@ prefs_key_accel_create (void)
 	gtk_tree_view_column_set_resizable(col, TRUE);
 
 	renderer = gtk_cell_renderer_text_new();
-	g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);
+	g_object_set(renderer, "xalign", 0.0, NULL);
 	gtk_tree_view_insert_column_with_attributes(treeview, -1,
 						    _("Accel"), renderer,
 						    "text", COLUMN_ACCEL,
