@@ -129,7 +129,7 @@ prefs_lang_create (void)
 
 	GTKUTIL_CREATE_MENU(option_menu, languages, n_languages, name);
 
-	str = KZ_CONF_GET_STR("Language", "charset_default");
+	str = KZ_CONF_GET_STR("Language", "intl.charset.default");
 	GTKUTIL_SET_DEFAULT_HISTORY(option_menu, str,
 			    languages, n_languages, code);
 	if (str)
@@ -162,7 +162,7 @@ prefs_lang_create (void)
 			    n_encoding_autodetectors,
 			    title);
 	
-	str = KZ_CONF_GET_STR("Language", "charset_detector");
+	str = KZ_CONF_GET_STR("Language", "intl.charset.detector");
 	GTKUTIL_SET_DEFAULT_HISTORY(option_menu, str,
 			    encoding_autodetectors, n_encoding_autodetectors,
 			    name);
