@@ -566,7 +566,7 @@ prefs_proxy_create (void)
 		gboolean use_same_proxy;
 		gboolean sel;
 
-		g_object_get(G_OBJECT(item),
+		g_object_get(item,
 			     "name", &name,
 			     "http_host", &http_host,
 			     "http_port", &http_port,
