@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- +  $Id: prefs_ui_editor.c,v 1.3 2004/10/17 07:25:18 makeinu Exp $
+ +  $Id: prefs_ui_editor.c 2863 2007-03-14 06:43:16Z ikezoe $
  */
 
 #include "prefs_ui_editor.h"
@@ -37,11 +37,11 @@ static void           prefs_ui_editor_response        (GtkWidget *widget,
 
 static KzPrefsWinPageEntry prefs_entry =
 {
-	path:            N_("/UI Editor"),
-	priority_hint:   0,
-	ui_level:        KZ_UI_LEVEL_EXPERT,
-	create:          prefs_ui_editor_create,
-	response:        prefs_ui_editor_response,
+	/* .path          = */ N_("/UI Editor"),
+	/* .priority_hint = */ 0,
+	/* .ui_level      = */ KZ_UI_LEVEL_EXPERT,
+	/* .create        = */ prefs_ui_editor_create,
+	/* .response      = */ prefs_ui_editor_response,
 };
 
 
