@@ -41,7 +41,7 @@ enum
 	COLUMN_ICON,
 	COLUMN_TITLE,
 	COLUMN_TITLE_COLOR,
-	COLUMN_EMBEDER,
+	COLUMN_WEBER,
 	N_COLUMNS
 };
 
@@ -69,8 +69,8 @@ static void         build_tab_tree         (KzTabTree    *tabtree,
 					    GNode        *parent);
 static void         build_tab_list         (KzTabTree    *tabtree);
 static GtkTreeIter *find_node              (GtkTreeStore *store,
-					    KzEmbed      *kzembed);
-static void         remove_all_embed_signals
+					    KzWeb      *kzweb);
+static void         remove_all_web_signals
 					   (KzTabTree    *tabtree);
 
 /* signal handlers for TreeView */
@@ -95,7 +95,7 @@ static void     cb_append_tab     (KzWindow        *kz,
 				   GtkWidget       *parent,
 				   KzTabTree       *tabtree);
 static void     parent_append     (KzTabTree * tabtree, 
-				   KzEmbed * kzembed, 
+				   KzWeb * kzweb, 
 				   GtkTreeIter * parent, 
 				   GtkTreeIter * iter);
 static void     cb_remove_tab     (KzWindow        *kz,
@@ -106,12 +106,13 @@ static void     cb_reorder_tab    (KzWindow        *kz,
 				   gint             pos,
 				   KzTabTree       *tabtree);
 
-/* signal handlers for embed */
-static void     cb_net_start      (KzEmbed         *embed,
+/* signal handlers for web */
+static void     cb_net_start      (KzWeb         *web,
 				   KzTabTree       *tabtree);
-static void     cb_net_stop       (KzEmbed         *embed,
+static void     cb_net_stop       (KzWeb         *web,
 				   KzTabTree       *tabtree);
-static void     cb_title_changed  (KzEmbed         *embed,
+static void     cb_title_changed  (KzWeb         *web,
+				   const gchar     *title,
 				   KzTabTree       *tabtree);
 
 
@@ -155,7 +156,7 @@ kz_tab_tree_class_init (KzTabTreeClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->constructor  = constructor;
 	gobject_class->dispose      = dispose;
@@ -348,7 +349,7 @@ dispose (GObject *obj)
 
 	if (tabtree->store)
 	{
-		remove_all_embed_signals(tabtree);
+		remove_all_web_signals(tabtree);
 		g_object_unref(tabtree->store);
 		tabtree->store = NULL;
 	}
@@ -452,12 +453,12 @@ build_tab_tree (KzTabTree *tabtree, GNode *parent)
 	node = g_node_first_child(parent);
 	for (; node; node = g_node_next_sibling(node))
 	{
-		KzEmbed *kzembed = node->data;
+		KzWeb *kzweb = node->data;
 		GNode *child = g_node_first_child(node);
 
-		if (KZ_IS_EMBED(kzembed))
+		if (KZ_IS_WEB(kzweb))
 		{
-			parent_append(tabtree, kzembed, parent_iter, &iter);
+			parent_append(tabtree, kzweb, parent_iter, &iter);
 		}
 		else
 		{
@@ -491,23 +492,15 @@ build_tab_list (KzTabTree *tabtree)
 
 	for (i = 0; i < num; i++)
 	{
-		KzEmbed *kzembed = KZ_EMBED(KZ_WINDOW_NTH_PAGE(kz, i));
-
-		if (KZ_IS_EMBED(kzembed))
-		{
-			parent_append(tabtree, kzembed, NULL, &iter);
-		}
-		else
-		{
-			g_warning("KzTabTree: Invalid tree item!");
-		}
+		KzWeb *kzweb = KZ_WINDOW_NTH_WEB(kz, i);
+		parent_append(tabtree, kzweb, NULL, &iter);
 	}
 }
 
 
 typedef struct _FindNode
 {
-	KzEmbed  *kzembed;
+	KzWeb  *kzweb;
 	GtkTreeIter *iter;
 } FindNode;
 
@@ -517,14 +510,14 @@ find_node_func (GtkTreeModel *model,
 		GtkTreePath *path, GtkTreeIter *iter,
 		FindNode *data)
 {
-	KzEmbed *kzembed = NULL;
+	KzWeb *kzweb = NULL;
 
 	if (!iter) return FALSE;
 
 	gtk_tree_model_get(model, iter,
-			   COLUMN_EMBEDER, &kzembed,
+			   COLUMN_WEBER, &kzweb,
 			   COLUMN_TERMINATOR);
-	if (kzembed && data->kzembed && kzembed == data->kzembed)
+	if (kzweb && data->kzweb && kzweb == data->kzweb)
 	{
 		data->iter = gtk_tree_iter_copy(iter);
 		return TRUE;
@@ -535,11 +528,11 @@ find_node_func (GtkTreeModel *model,
 
 
 static GtkTreeIter *
-find_node (GtkTreeStore *store, KzEmbed *kzembed)
+find_node (GtkTreeStore *store, KzWeb *kzweb)
 {
 	FindNode data;
 	data.iter = NULL;
-	data.kzembed = kzembed;
+	data.kzweb = kzweb;
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store),
 			       (GtkTreeModelForeachFunc) find_node_func,
 			       &data);
@@ -548,11 +541,11 @@ find_node (GtkTreeStore *store, KzEmbed *kzembed)
 
 
 static gboolean
-remove_embed_signal_func (GtkTreeModel *model,
+remove_web_signal_func (GtkTreeModel *model,
 			  GtkTreePath *path, GtkTreeIter *iter,
 			  gpointer data)
 {
-	KzEmbed *kzembed = NULL;
+	KzWeb *kzweb = NULL;
 	KzTabTree *tabtree;
 
 	g_return_val_if_fail(KZ_IS_TAB_TREE(data), FALSE);
@@ -561,17 +554,17 @@ remove_embed_signal_func (GtkTreeModel *model,
 	g_return_val_if_fail(iter, FALSE);
 
 	gtk_tree_model_get(model, iter,
-			   COLUMN_EMBEDER, &kzembed,
+			   COLUMN_WEBER, &kzweb,
 			   COLUMN_TERMINATOR);
-	g_return_val_if_fail(kzembed, FALSE);
+	g_return_val_if_fail(kzweb, FALSE);
 
-	g_signal_handlers_disconnect_by_func(kzembed,
+	g_signal_handlers_disconnect_by_func(kzweb,
 					     G_CALLBACK(cb_title_changed),
 					     tabtree);
-	g_signal_handlers_disconnect_by_func(kzembed,
+	g_signal_handlers_disconnect_by_func(kzweb,
 					     G_CALLBACK(cb_net_start),
 					     tabtree);
-	g_signal_handlers_disconnect_by_func(kzembed,
+	g_signal_handlers_disconnect_by_func(kzweb,
 					     G_CALLBACK(cb_net_stop),
 					     tabtree);
 
@@ -580,13 +573,13 @@ remove_embed_signal_func (GtkTreeModel *model,
 
 
 static void
-remove_all_embed_signals (KzTabTree *tabtree)
+remove_all_web_signals (KzTabTree *tabtree)
 {
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 	g_return_if_fail(GTK_IS_TREE_MODEL(tabtree->store));
 
 	gtk_tree_model_foreach(GTK_TREE_MODEL(tabtree->store),
-			       remove_embed_signal_func,
+			       remove_web_signal_func,
 			       tabtree);
 }
 
@@ -603,7 +596,7 @@ cb_cursor_changed(GtkTreeView *tree_view, KzTabTree *tabtree)
 	GtkTreePath *treepath;
 	GtkTreeIter iter;
 	KzWindow *kz;
-	KzEmbed *kzembed = NULL;
+	KzWeb *kzweb = NULL;
 	gint num;
 
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
@@ -619,12 +612,12 @@ cb_cursor_changed(GtkTreeView *tree_view, KzTabTree *tabtree)
 				&iter, treepath);
 	gtk_tree_path_free(treepath);
 	gtk_tree_model_get(GTK_TREE_MODEL(tabtree->store), &iter,
-			   COLUMN_EMBEDER, &kzembed,
+			   COLUMN_WEBER, &kzweb,
 			   COLUMN_TERMINATOR);
-	if (!kzembed) return;
+	if (!kzweb) return;
 
 	num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
-				   GTK_WIDGET(kzembed));
+				   GTK_WIDGET(kzweb));
 	g_return_if_fail(num >= 0);
 
 	tabtree->switching = TRUE;
@@ -640,8 +633,8 @@ cb_tree_view_button_release (GtkWidget *widget, GdkEventButton *event,
 	GtkTreePath *treepath;
 	GtkTreeIter iter;
 	KzWindow *kz;
-	KzEmbed *kzembed = NULL;
-	GtkWidget *label;
+	KzWeb *kzweb = NULL;
+	KzTabLabel *label;
 
 	g_return_val_if_fail(KZ_IS_TAB_TREE(tabtree), FALSE);
 
@@ -654,16 +647,16 @@ cb_tree_view_button_release (GtkWidget *widget, GdkEventButton *event,
 				&iter, treepath);
 	gtk_tree_path_free(treepath);
 	gtk_tree_model_get(GTK_TREE_MODEL(tabtree->store), &iter,
-			   COLUMN_EMBEDER, &kzembed,
+			   COLUMN_WEBER, &kzweb,
 			   COLUMN_TERMINATOR);
-	if (!kzembed) return FALSE;
+	if (!kzweb) return FALSE;
 
 	switch (event->button) {
 	case 3:
-		label = kz_window_get_tab_label(kz, GTK_WIDGET(kzembed));
-		g_return_val_if_fail(KZ_IS_TAB_LABEL(label), FALSE);
+		label = kz_window_get_tab_label(kz, kzweb);
+		g_return_val_if_fail(label, FALSE);
 
-		kz_actions_tab_popup_menu_modal(KZ_TAB_LABEL(label),
+		kz_actions_tab_popup_menu_modal(label,
 						event->button, event->time);
 		return TRUE;
 	default:
@@ -706,7 +699,7 @@ static void
 cb_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 		guint page_num, KzTabTree *tabtree)
 {
-	GtkWidget *widget;
+	KzWeb *web;
 	GtkTreeIter *iter;
 	GtkTreePath *treepath;
 	KzTabLabel *label;
@@ -715,8 +708,8 @@ cb_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 
 	if (tabtree->switching) return;
 
-	widget = kz_notebook_get_nth_page(KZ_NOTEBOOK(notebook), page_num);
-	iter = find_node(tabtree->store, KZ_EMBED(widget));
+	web = kz_notebook_get_nth_web(KZ_NOTEBOOK(notebook), page_num);
+	iter = find_node(tabtree->store, web);
 	/* g_return_if_fail(iter); */
 	if (!iter) return;
 
@@ -727,10 +720,10 @@ cb_switch_page (GtkNotebook *notebook, GtkNotebookPage *page,
 	tabtree->switching = TRUE;
 	gtk_tree_view_set_cursor(tabtree->tree_view, treepath, NULL, FALSE);
 
-	label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(notebook, widget));
+	label = kz_notebook_get_tab_label(KZ_NOTEBOOK(notebook), web);
 	
 	gtk_tree_store_set(tabtree->store, iter,
-			   COLUMN_TITLE,   kz_embed_ensure_title(KZ_EMBED(widget)),
+			   COLUMN_TITLE, kz_web_ensure_title(web),
 			   COLUMN_TITLE_COLOR, label_color[label->state],
 			   COLUMN_TERMINATOR);
 
@@ -752,12 +745,12 @@ cb_append_tab (KzWindow *kz, GtkWidget *widget, GtkWidget *parent,
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	KzEmbed * kzembed = KZ_EMBED(widget); 
+	KzWeb * kzweb = KZ_WEB(widget);
 
 	g_return_if_fail(GTK_IS_WIDGET(widget));
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	parent_append(tabtree, kzembed, find_node(tabtree->store, KZ_EMBED(parent)), &iter);
+	parent_append(tabtree, kzweb, find_node(tabtree->store, KZ_WEB(parent)), &iter);
 	
 	path = gtk_tree_model_get_path(GTK_TREE_MODEL(tabtree->store), &iter);
 	gtk_tree_view_expand_to_path(tabtree->tree_view, path);
@@ -766,16 +759,16 @@ cb_append_tab (KzWindow *kz, GtkWidget *widget, GtkWidget *parent,
 
 
 static void 
-parent_append(KzTabTree * tabtree, KzEmbed * kzembed, GtkTreeIter * parent, GtkTreeIter * iter) {
+parent_append(KzTabTree * tabtree, KzWeb * kzweb, GtkTreeIter * parent, GtkTreeIter * iter) {
 
 	const gchar *color;
-	gchar *title = kz_embed_ensure_title(kzembed);
-	const gchar *uri   = kz_embed_get_location(kzembed);
+	gchar *title = kz_web_ensure_title(kzweb);
+	const gchar *uri   = kz_web_get_location(kzweb);
 	GdkPixbuf *favicon = kz_favicon_get_pixbuf(KZ_GET_FAVICON,
 						   uri,
 						   GTK_ICON_SIZE_MENU);
 
-	KzTabLabel * label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(tabtree->sidebar->kz->notebook), GTK_WIDGET(kzembed)));
+	KzTabLabel * label = kz_notebook_get_tab_label(KZ_NOTEBOOK(tabtree->sidebar->kz->notebook), kzweb);
 	color = label_color[label->state];
 
 	gtk_tree_store_append(tabtree->store, iter, parent);
@@ -783,16 +776,16 @@ parent_append(KzTabTree * tabtree, KzEmbed * kzembed, GtkTreeIter * parent, GtkT
 			   COLUMN_ICON,        favicon,
 			   COLUMN_TITLE,       title,
 			   COLUMN_TITLE_COLOR, color,
-			   COLUMN_EMBEDER,     kzembed,
+			   COLUMN_WEBER,     kzweb,
 			   COLUMN_TERMINATOR);
 
-	g_signal_connect(kzembed, "kz-title",
+	g_signal_connect(kzweb, "kz-title",
 			 G_CALLBACK(cb_title_changed),
 			 tabtree);
-	g_signal_connect(kzembed, "kz-net-start",
+	g_signal_connect(kzweb, "kz-net-start",
 			 G_CALLBACK(cb_net_start),
 			 tabtree);
-	g_signal_connect(kzembed, "kz-net-stop",
+	g_signal_connect(kzweb, "kz-net-stop",
 			 G_CALLBACK(cb_net_stop),
 			 tabtree);
 
@@ -805,14 +798,14 @@ parent_append(KzTabTree * tabtree, KzEmbed * kzembed, GtkTreeIter * parent, GtkT
 static void
 move_to(KzTabTree * tabtree, GtkTreeIter * child, GtkTreeIter * parent)
 {
-	KzEmbed * kzembed;
+	KzWeb * kzweb;
 	GtkTreeIter newparent, newchild;
 	GtkTreePath *path;
 	
         g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 	
-	gtk_tree_model_get(GTK_TREE_MODEL(tabtree->store), child, COLUMN_EMBEDER, &kzembed, COLUMN_TERMINATOR);
-	parent_append(tabtree, kzembed, parent, &newparent);
+	gtk_tree_model_get(GTK_TREE_MODEL(tabtree->store), child, COLUMN_WEBER, &kzweb, COLUMN_TERMINATOR);
+	parent_append(tabtree, kzweb, parent, &newparent);
 
 	path = gtk_tree_model_get_path(GTK_TREE_MODEL(tabtree->store),
                                        child);
@@ -844,7 +837,7 @@ cb_remove_tab (KzWindow *kz, GtkWidget *widget,
 	g_return_if_fail(GTK_IS_WIDGET(widget));
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	parent = find_node(tabtree->store, KZ_EMBED(widget));
+	parent = find_node(tabtree->store, KZ_WEB(widget));
 
 	while (gtk_tree_model_iter_children(GTK_TREE_MODEL(tabtree->store), &child, parent))
 	{
@@ -886,16 +879,16 @@ cb_reorder_tab (KzWindow *kz,
  *                                                                           *
  *****************************************************************************/
 static void
-cb_net_start (KzEmbed *embed, KzTabTree *tabtree)
+cb_net_start (KzWeb *web, KzTabTree *tabtree)
 {
 	GtkTreeIter *iter;
 	gchar *title;
 
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	title = kz_embed_ensure_title(KZ_EMBED(embed));
+	title = kz_web_ensure_title(KZ_WEB(web));
 
-	iter = find_node (tabtree->store, KZ_EMBED(embed));
+	iter = find_node(tabtree->store, KZ_WEB(web));
 	if (iter)
 	{
 		gtk_tree_store_set(tabtree->store, iter,
@@ -910,19 +903,19 @@ cb_net_start (KzEmbed *embed, KzTabTree *tabtree)
 
 
 static void
-cb_net_stop (KzEmbed *embed, KzTabTree *tabtree)
+cb_net_stop (KzWeb *web, KzTabTree *tabtree)
 {
 	GtkTreeIter *iter;
 	gchar *title;
 
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	title = kz_embed_ensure_title(KZ_EMBED(embed));
+	title = kz_web_ensure_title(KZ_WEB(web));
 
-	iter = find_node (tabtree->store, KZ_EMBED(embed));
+	iter = find_node(tabtree->store, KZ_WEB(web));
 	if (iter)
 	{
-		KzTabLabel * label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(tabtree->sidebar->kz->notebook), GTK_WIDGET(embed)));
+		KzTabLabel * label = kz_notebook_get_tab_label(KZ_NOTEBOOK(tabtree->sidebar->kz->notebook), web);
 
 		gtk_tree_store_set(tabtree->store, iter,
 				   COLUMN_TITLE,       title,
@@ -938,19 +931,16 @@ cb_net_stop (KzEmbed *embed, KzTabTree *tabtree)
 
 
 static void
-cb_title_changed (KzEmbed *embed, KzTabTree *tabtree)
+cb_title_changed (KzWeb *web, const gchar *title, KzTabTree *tabtree)
 {
-	gchar *title;
 	GtkTreeIter *iter;
 
 	g_return_if_fail(KZ_IS_TAB_TREE(tabtree));
 
-	title = kz_embed_ensure_title(KZ_EMBED(embed));
-
-	iter = find_node (tabtree->store, KZ_EMBED(embed));
+	iter = find_node(tabtree->store, KZ_WEB(web));
 	if (iter)
 	{
-                KzTabLabel * label = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(tabtree->sidebar->kz->notebook), GTK_WIDGET(embed)));
+                KzTabLabel * label = kz_notebook_get_tab_label(KZ_NOTEBOOK(tabtree->sidebar->kz->notebook), web);
 
 		gtk_tree_store_set(tabtree->store, iter,
 				   COLUMN_TITLE, title,
@@ -959,6 +949,4 @@ cb_title_changed (KzEmbed *embed, KzTabTree *tabtree)
 
 		gtk_tree_iter_free(iter);
 	}
-
-	g_free(title);
 }
