@@ -290,7 +290,7 @@ cb_thumbnail_activate (KzThumbnailsView *view,
 		       KzWindow *kz)
 {
 	KzBookmarkFolder *tab;
-	GtkWidget *kzembed;
+	GtkWidget *kzweb;
 	KzTabLabel *kztab;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
@@ -299,8 +299,8 @@ cb_thumbnail_activate (KzThumbnailsView *view,
 	tab = KZ_BOOKMARK_FOLDER(kz_bookmark_folder_get_nth_child(kz->closed_tabs, index));
 	g_return_if_fail(tab);
 
-	kzembed = kz_window_open_new_tab(kz, NULL);
-	kztab = kz_notebook_get_tab_label(KZ_NOTEBOOK(kz->notebook), KZ_EMBED(kzembed));
+	kzweb = kz_window_open_new_tab(kz, NULL);
+	kztab = kz_notebook_get_tab_label(KZ_NOTEBOOK(kz->notebook), KZ_WEB(kzweb));
 	kz_tab_label_set_history(KZ_TAB_LABEL(kztab), tab);
 
 	kz_bookmark_folder_remove(kz->closed_tabs, KZ_BOOKMARK(tab));
