@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 3S30, Boston, MA 02111-1307, USA.
  *
- * $Id: kz-dlist.c 2644 2006-12-02 12:19:08Z ikezoe $
+ * $Id: kz-dlist.c 3675 2009-02-14 01:40:31Z ikezoe $
  */
 
 #include "kz-dlist.h"
@@ -88,7 +88,7 @@ kz_dlist_class_init (KzDListClass *klass)
 {
 	GObjectClass *gobject_class;
 
-	gobject_class = (GObjectClass *) klass;
+	gobject_class = G_OBJECT_CLASS(klass);
 
 	gobject_class->dispose      = dispose;
 	gobject_class->set_property = set_property;
