@@ -23,7 +23,6 @@
 
 #include "kz-paned.h"
 
-#include "gobject-utils.h"
 #include "gtk-utils.h"
 #include <glib/gi18n.h>
 
@@ -33,12 +32,10 @@ enum {
 };
 
 /* object class */
-static void kz_paned_class_init     (KzPanedClass   *klass);
-static void kz_paned_init           (KzPaned        *kzpaned);
-static void kz_paned_size_request   (GtkWidget      *widget,
-				     GtkRequisition *requisition);
-static void kz_paned_size_allocate  (GtkWidget      *widget,
-				     GtkAllocation  *allocation);
+static void size_request   (GtkWidget      *widget,
+                            GtkRequisition *requisition);
+static void size_allocate  (GtkWidget      *widget,
+                            GtkAllocation  *allocation);
 
 static void kz_paned_rotate_position      (KzPaned *kzpaned);
 static void kz_paned_flip_position        (KzPaned *kzpaned);
@@ -78,8 +75,6 @@ static void kz_paned_set_divider_position (KzPaned *kzpaned);
 	GTK_PANED(kzpaned)->orientation = GTK_ORIENTATION_VERTICAL;	\
 }
 
-static GtkPanedClass *parent_class = NULL;
-
 static gint kz_paned_signals[LAST_SIGNAL] = {0};
 
 static void (* kz_paned_hsize_request)  (GtkWidget       *widget,
@@ -91,9 +86,7 @@ static void (* kz_paned_hsize_allocate) (GtkWidget       *widget,
 static void (* kz_paned_vsize_allocate) (GtkWidget       *widget,
 					 GtkAllocation   *alocation);
 
-KZ_OBJECT_GET_TYPE(kz_paned, "KzPaned", KzPaned,
-		   kz_paned_class_init, kz_paned_init,
-		   GTK_TYPE_PANED)
+G_DEFINE_TYPE(KzPaned, kz_paned, GTK_TYPE_PANED)
 
 static void
 kz_paned_class_init (KzPanedClass *klass)
@@ -117,12 +110,10 @@ kz_paned_class_init (KzPanedClass *klass)
 	kz_paned_vsize_allocate =
 		((GtkWidgetClass *)vpaned_class)->size_allocate;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	widget_class  = (GtkWidgetClass *) klass;
 
-	widget_class->size_request  = kz_paned_size_request;
-	widget_class->size_allocate = kz_paned_size_allocate;
+	widget_class->size_request  = size_request;
+	widget_class->size_allocate = size_allocate;
 	
 	klass->changed_position = NULL;
 
@@ -359,8 +350,7 @@ kz_paned_flip_position(KzPaned *kzpaned)
 
 
 static void
-kz_paned_size_request (GtkWidget      *widget,
-		       GtkRequisition *requisition)
+size_request (GtkWidget *widget, GtkRequisition *requisition)
 {
 	KzPaned *kzpaned;
 	g_return_if_fail (KZ_IS_PANED (widget));
@@ -383,8 +373,7 @@ kz_paned_size_request (GtkWidget      *widget,
 
 
 static void
-kz_paned_size_allocate (GtkWidget     *widget,
-			GtkAllocation *allocation)
+size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
 	KzPaned *kzpaned;
 	g_return_if_fail (KZ_IS_PANED (widget));
