@@ -53,6 +53,8 @@ struct _KzPanedClass
 GType      kz_paned_get_type (void) G_GNUC_CONST;
 GtkWidget *kz_paned_new      (void);
 
+gboolean   kz_paned_is_showing_all_children (KzPaned *kzpaned);
+void       kz_paned_set_separator_position (KzPaned *kzpaned, gint position);
 void       kz_paned_set_position  (KzPaned *kzpaned, GtkPositionType position);
 
 G_END_DECLS
